#ifndef ADDDATABASE_H
#define ADDDATABASE_H

#include <QDialog>
#include <QStackedWidget>
#include <QVBoxLayout>


#include <QLabel>
#include <QLineEdit>
#include <QGroupBox>
#include <QCheckBox>
#include <QTreeView>
#include <QDirModel>
#include <QPushButton>
#include <QProgressBar>
#include <QHeaderView>
#include <QThread>


#include "DbManager.h"
#include "Const.h"
#include "Timer.h"
#include "MainWindow.h"


class DbManager;
class MainWindow;
class SetupPage;
class ProgressPage;



class MyThread : public QThread
{
  Q_OBJECT
      
  public:
    MyThread(SetupPage* setupPage);
    void run();
  private slots:
    void updateTime();
  private:
    SetupPage* setupPage_;
};



class AddDatabase : public QDialog
{
  Q_OBJECT

  public:
    AddDatabase(MainWindow* mainWindow, QWidget* parent = 0);
    SetupPage* setupPage;
    ProgressPage* progressPage;
    
//     void updateEvents(ProcessSection section, double estimateTime);
    void updateEvents();
    bool isCancelled();

  protected:
    void reject();
    void accept();
    
  private slots:

  private:
    Timer* timer_;
    QStackedWidget* stackedWidget;
    DbManager* dbManager_;
    MainWindow* mainWindow_;
    void readSettings();
    void writeSettings();
    double estimateTime1_;
    double estimateTime2_;
    double estimateTime3_;
    
  friend class SetupPage;
  friend class ProgressPage;
};



class SetupPage : public QWidget
{
  Q_OBJECT

  public:
    SetupPage(AddDatabase* addDatabase, QWidget* parent = 0);
    QTimer* timer_;
  public slots:
    void updateTime();
    
  private slots:
    void checkFirstPage();
    void start();

  private:
    int timeSeconds_;
    AddDatabase* addDatabase_;
    QLabel* databaseNameLabel;
    QLabel* directoryNameLabel;
    QLineEdit* databaseNameLineEdit;
    QGroupBox* groupBox;
    QCheckBox* recursiveDirCheckBox;
    QCheckBox* acceptDuplicatesCheckBox;
    QTreeView* treeView;
    QDirModel* dirModel;
    QPushButton* cancelButton;
    QPushButton* startButton;
    
    friend class ProgressPage;
};



class ProgressPage : public QWidget
{
  Q_OBJECT

  public:
    ProgressPage(AddDatabase* addDatabase, QWidget* parent = 0);
    
    QProgressBar* progressBarInitializeGames;
    QProgressBar* progressBarDuplicateCheck;
    QProgressBar* progressBarParseGames;
    QProgressBar* progressBarInitializePatterns;
    QProgressBar* progressBarOverall;

    void setLabelDuplicateGamesData(u32 duplicatesTotal);
    void setLabelTotalGamesData(u32 gamesTotal);
    
    bool hasCancelled_;
    bool isBackClicked_;
    
  private slots:
    void backClick();
    void cancelClick();

  private:
    
    void back();
    void cancel();
    
    AddDatabase* addDatabase_;
    
    QLabel* labelInitializeGames;
    QLabel* labelDuplicateCheck;
    QLabel* labelParseGames;
    QLabel* labelInitializePatterns;
    QLabel* labelOverall;
    
    QLabel* labelCurrentTime;
    QLabel* labelEstimateTime;
    QLabel* labelTotalGames;
    QLabel* labelDuplicateGames;
    QLabel* labelCurrentTimeData;
    QLabel* labelEstimateTimeData;
    QLabel* labelTotalGamesData;
    QLabel* labelDuplicateGamesData;
    
    QPushButton* cancelButton;
    QPushButton* backButton;
    QPushButton* finishButton;
    
    friend class SetupPage;
    friend class AddDatabase;
};

#endif
