//Board.cpp
//baduK++ 0.1 - Neil Mclean 4/4/2006

#include <iostream>
#include <vector>
#include <fstream>
#include <QDebug>

#include "Board.h"
#include "Sgf.h"

using namespace std;

//-----------------------------------------------------
//! Point
//-----------------------------------------------------
Point::Point() {}
Point::Point(i8 row, i8 col, u8 value)
{
  x = row;
  y = col;
  stone = value;
}


//-----------------------------------------------------
//! Board
//-----------------------------------------------------
Board::Board() 
{
  clear();
}



//using shared_ptr so do clear to delete objects automatically
Board::~Board() 
{
  if (Points.empty() == false) { 
    // delete all points in container
    for (unsigned int i = 0; i < Points.size(); i++) {
      delete Points[i];
    }
    Points.clear();
  }
}



//Setup a board to be in empty state
void Board::clear() 
{
  for (int i = 0; i < 19; i++) {
    for (int j = 0; j < 19; j++) {
      board_[i][j] = EMPTY;
    }
  }
}


//Empty captures vector - list of stones captured
void Board::clearCaptures()
{
//   qDebug("clear captures called");
  if (captures.empty() == false) { //do for each new move here
    captures.clear();
  }
}


//Used to access private captures list from FreeSearch 
u16 Board::getCaptureCount() const 
{
  return captures.size(); 
}


FullMove* Board::getCaptureStone(u16 value)
{
  return captures[value];
}


void Board::setStone(const i8 x, const i8 y, const u8 value) 
{
  board_[x][y] = value;
}


u8 Board::getStone(const i8 x, const i8 y) const
{
  return board_[x][y];
}



//----------------------------------------------------------------------
//! GO LOGIC SECTION
//credit to 'problematic' java applet for basis of this GO LOGIC code
//I wonder how other this GO LOGIC could be implemented.
//I know it could be improved but it is simple and works so I leave it.
//-------------------------------------------------------------------

//Calls to make a group and check if alive or dead 
//If group has zero liberties set the groups stones to be Empty
bool Board::removeDeadGroup(const i8 x, const i8 y, const u8 enemy) 
{
//   cout << "removeDead x=" << (int) x << " y=" << int (y) << endl;
  bool isValid;
  if (x >= 0 && x <= 19 && y >= 0 && y <= 19) {
    isValid = true;
  }
  else {
    isValid = false;
  }
  if ((true == isValid) && (true == checkDeadGroup(x, y)) &&
       (enemy == board_[x][y])) {
    try {
      for (u32 i = 0; i < Points.size(); i++) {
        board_[Points[i]->x][Points[i]->y] = EMPTY; 
        FullMove* capture = new FullMove;
        capture->x = Points[i]->x;
        capture->y = Points[i]->y;
        captures.push_back(capture);
        //cout << "capture xy=" << (int)move->x << " " << (int)move->y << endl;
      }
    }
    catch (bad_alloc&) { 
      cout << "Error allocating memory." << endl; 
      abort();
    }
//     qDebug("removeDeadGroup return true");
    return true;
  }
//   qDebug("removeDeadGroup return false");
  return false;
}



//Calls makegroup to make a group of stones
//Calls count liberties to determine if alive or dead
// (x >= 0 && x <= 19 && y >= 0 && y <= 19)  always true
bool Board::checkDeadGroup(const i8 x, const i8 y)
{
 // qDebug("checkDeadGroup");
  assert((x >= 0) && (x <= 19) && (y >= 0) && (y <= 19));
  if (board_[x][y] != EMPTY) { 
//     qDebug("checkDeadGroup != EMPTY");

    if (Points.empty() == false) { 
    // delete all points in container
      for (unsigned int i = 0; i < Points.size(); i++) {
        delete Points[i];
      }
      Points.clear();
    }
    makeGroup(x, y, board_[x][y]);
  //  qDebug("after makeGroup");

    u16 libs = computeLiberties(); //if no libs then true;
    if (0 == libs) {
      //cout << "0 zero libs xy=" << (int)x << " " << (int)y << endl;
      //string dud; cin >> dud;
// 	qDebug("checkDeadGroup return true");
      return true;
    }
  }
//   qDebug("checkDeadGroup return false");
  return false;
}




//------------------------------------------------------
//recursively add same colour stones to make a group
bool Board::makeGroup(i8 x, i8 y, const u8 colour) 
{
  
//   cout << "make group xy=" << (int)x << " " << (int)y << endl;
 

  bool isValid;
  if ((x >= 0) && (x <= 18) && (y >= 0) && (y <= 18)) {
    isValid = true;
  }
  else {
    isValid = false;
  }
  if ((isValid == true) && (colour == board_[x][y])) {
    //go through all points in group already to check if point is in it
    u16 i = 0;
    bool hasPoint = false;
    while ((i < Points.size()) && (false == hasPoint)) {
      if ((Points.at(i)->x == x) && (Points.at(i)->y == y)) { 
        hasPoint = true;
      }
      ++i;
    }
    if (hasPoint == false) {
      try {
//  		qDebug("before new point");
//!FIXME we have a problem here allocating memory
        Point* point = new Point(x, y, colour);
//  		qDebug("this should not be called");
// 		if (point == 0) {
// 			qDebug("point ==0");
// 		}
        Points.push_back(point); //add stone to group  - why is this not caught for win32 error in catch
// 		qDebug("after new point");
      }
      catch (bad_alloc&) { 
        cout << "Error allocating memory." << endl; 
        abort();
      }
      catch (...) {
        cout << "catch bad alloc here?" << endl; 
        abort();
      }

      //Check the new point's surrounding points to form group
      makeGroup(x - 1, y, colour);
      makeGroup(x, y + 1, colour);
      makeGroup(x + 1, y, colour);
      makeGroup(x, y - 1, colour);
      return true;
    }
  }
  return false;
}



//---------------------------------------------------------------------
//counts liberties of a group.
//counts some liberties twice, but since only for death check its ok
//---------------------------------------------------------------------
u16 Board::computeLiberties() const 
{
  u16 libs = 0;
  for (u32 i = 0; i < Points.size(); i++) {
    if ((Points[i]->x >= 1) && 
         (EMPTY == board_[(Points[i]->x) - 1][(Points[i]->y)])) {
      ++libs;
    }
    if ((Points[i]->y >= 1) &&
         (EMPTY == board_[(Points[i]->x)][(Points[i]->y) - 1])) {
      ++libs;
    }
    if ((Points[i]->x <= 17) &&
         (EMPTY == board_[(Points[i]->x) + 1][(Points[i]->y)])) {
      ++libs;
    }
    if ((Points[i]->y <= 17) &&
         (EMPTY == board_[(Points[i]->x)][(Points[i]->y) + 1])) {
      ++libs;
    }
  }
  return libs;
}

//---------------------------------------------------------------------
//! END OF GO LOGIC
//---------------------------------------------------------------------



//swap values of positions in x axis - horizontal mirror
void Board::xAxisMirror() 
{ 
  u8 colour;
  for (int x = 0; x < MID_BOARD; x++) {
    for (int y = 0; y < 19; y++) {
      colour = board_[x][y];
      board_[x][y] = board_[END_BOARD - x][y];
      board_[END_BOARD - x][y] = colour;
    }
  }
}


//swap values of positions in y axis - vertical mirror
void Board::yAxisMirror() 
{
  u8 colour;
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < MID_BOARD; y++) {
      colour = board_[x][y];
      board_[x][y] = board_[x][END_BOARD - y];
      board_[x][END_BOARD - y] = colour;
    }
  }
}


//Diagonal from (top right) to (bottom left)
// ***o    process one half of the mirror only
// **o*    or it will double process and cancel!
// *o**
// o***
void Board::diagonalMirror() 
{
  int x = END_BOARD;
  int y = START_BOARD;
  while (y <= END_BOARD) {
    while (x >= 0) {
      u8 colour = board_[x][y];
      board_[x][y] = board_[(END_BOARD - y)][(END_BOARD - x)];
      board_[(END_BOARD - y)][(END_BOARD - x)] = colour;
      --x;
    }
    ++y;
    x = END_BOARD - y;
  }
}




// Setup a temporary board and rotate current board onto it,
// Then copy rotated board back to current board.
void Board::rotate()
{
  Board board;
  i8 stone;
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
      stone = board_[x][y];
      board.setStone(END_BOARD - y, x, stone);
    }
  }
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
      board_[x][y] = board.getStone(x, y);
    }
  }
}



//Copies current board to board pointer passed.
//Used by GUI MainBoard to navigate the board.
void Board::copyBoard(boost::shared_ptr<Board> board)
{
  for (int i = 0; i < 19; i++) {
    for (int j = 0; j < 19; j++) {
      board->board_[i][j] = board_[i][j];
    }
  }
}




// Count the number of stones on the board
u16 Board::getBoardStoneCount()
{
  u16 count = 0;
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
//       assert((board[x][y] <= 2) && (board[x][y] >= 0));
      if (0 != board_[x][y]) {
        count++;
      }
    }
  }
  return count;
}



////////////////////////////////////////////////////////////////
//! Debug Code
////////////////////////////////////////////////////////////////
void Board::printBoard() const 
{
  system("clear"); //unix only
  cout << "  ABCDEFGHIJKLMNOPQRS" << endl;
  for (int i = 0; i < 19; i++) {
    u8 c = ('A'+i);
    cout << c << " ";
    for (int j=0; j<19;j++) {
      if (BLACK == board_[j][i]) cout << "@";
      if (WHITE == board_[j][i]) cout << "o";
      if (EMPTY == board_[j][i]) cout << ".";
    }
    cout << endl;
  }
}
