#ifndef BOARD_H
#define BOARD_H

#include <vector>

#include "Const.h"
#include "Sgf.h"

#ifdef WIN32
  #include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
  #include "boost/shared_ptr.hpp"
#endif



class Point {
  public:
    Point();
    Point(i8 row, i8 col, u8 value);
    i8 x;
    i8 y;
    u8 stone;
};



class Board {
  public:
    Board();
    ~Board();
    void clear();
    void printBoard() const;
    void setStone(const i8 x, const i8 y, const u8 value); //sets colour
    u8 getStone(const i8 x, const i8 y) const; // returns colour
    bool checkDeadGroup(const i8 x, const i8 y);
    bool removeDeadGroup(const i8 x, const i8 y, const u8 enemy);
    void xAxisMirror();
    void yAxisMirror();
    void diagonalMirror();
    void rotate();
    void copyBoard(boost::shared_ptr<Board> board);
    u16 getBoardStoneCount();
    u16 getCaptureCount() const; // used to access captures
    void clearCaptures();
    FullMove* getCaptureStone(u16 value);

  private:
    bool makeGroup(i8 x, i8 y, const u8 stone);
    u16 computeLiberties() const;
    vector<Point*> Points;    
    vector< FullMove* > captures; 
    i8 board_[19][19];
    
};

#endif
