// Configuration.cpp
// baduK 0.1 - Neil Mclean 
// Started 12/7/2006

#include <QApplication>
#include <QEvent>
#include <QSettings>
#include <QCloseEvent>
#include <QLabel>
#include <QSpinBox>
#include <QListWidget>
#include <QApplication>

#include "Configuration.h"


MainOptions::MainOptions(QWidget* parent) : QWidget(parent)
{
  QGroupBox *configGroup = new QGroupBox(tr("Analysis configuration"));
  QLabel* labelNextMoveCount = new QLabel(this);
  spinBox_ = new QSpinBox(this);
  QHBoxLayout* nextMoveLayout = new QHBoxLayout;
  nextMoveLayout->addWidget(labelNextMoveCount);
  nextMoveLayout->addWidget(spinBox_);  
  labelNextMoveCount->setText(QApplication::translate("MainOptions", "Show Next Moves", 0, 
                              QApplication::UnicodeUTF8));  
  configGroup->setLayout(nextMoveLayout);
  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(configGroup);
  mainLayout->addStretch(1);
  setLayout(mainLayout);
}



MainOptions::~MainOptions() {}

  
Configuration::Configuration()
{
  contentsWidget = new QListWidget;
  contentsWidget->setViewMode(QListView::IconMode);
  contentsWidget->setIconSize(QSize(96, 84));
  contentsWidget->setMovement(QListView::Static);
  contentsWidget->setMaximumWidth(128);
  contentsWidget->setSpacing(12);

  pagesWidget = new QStackedWidget;
  mainOptions_ = new MainOptions;
  pagesWidget->addWidget(mainOptions_);

  QPushButton *closeButton = new QPushButton(tr("&Close"));
  QPushButton *okButton = new QPushButton(tr("&Ok"));
  createIcons();
  contentsWidget->setCurrentRow(0);

  connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
  connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));

  QHBoxLayout *horizontalLayout = new QHBoxLayout;
  horizontalLayout->addWidget(contentsWidget);
  horizontalLayout->addWidget(pagesWidget, 1);

  QHBoxLayout *buttonsLayout = new QHBoxLayout;
  buttonsLayout->addStretch(1);
  buttonsLayout->addWidget(okButton);
  buttonsLayout->addWidget(closeButton);
  
  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addLayout(horizontalLayout);
  mainLayout->addStretch(1);
  mainLayout->addSpacing(12);
  mainLayout->addLayout(buttonsLayout);
  setLayout(mainLayout);
  readSettings();
  setWindowTitle(tr("baduK Config Dialog"));
}



Configuration::~Configuration() {}



void Configuration::createIcons()
{
  QListWidgetItem *configButton = new QListWidgetItem(contentsWidget);
  configButton->setIcon(QIcon(":/resources/icons/configure.png"));
  configButton->setText(tr("General"));
  configButton->setTextAlignment(Qt::AlignHCenter);
  configButton->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

  connect(contentsWidget,
          SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)),
          this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)));
}



void Configuration::changePage(QListWidgetItem *current, QListWidgetItem *previous)
{
  if (false == current) {
    current = previous;
  }
  pagesWidget->setCurrentIndex(contentsWidget->row(current));
}



void Configuration::accept()
{
  QSettings settings;
  settings.beginGroup("Configuration");
  settings.setValue("NextMoveCount", mainOptions_->spinBox_->value());
  settings.endGroup();
  close();
}



void Configuration::closeEvent(QCloseEvent *event)
{
  writeSettings();
  event->accept();
}



void Configuration::readSettings()
{
  QSettings settings;
  settings.beginGroup("Configuration");
  QPoint pos = settings.value("pos", QPoint(0, 420)).toPoint();
  QSize size = settings.value("size", QSize(400, 300)).toSize();
  int num = settings.value("NextMoveCount", 20).toInt();
  settings.endGroup();
  mainOptions_->spinBox_->setValue(num);
  resize(size);
  move(pos);
}



void Configuration::writeSettings()
{
  QSettings settings;
  settings.beginGroup("Configuration");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.endGroup();
}
