#ifndef DATABASE_H
#define DATABASE_H

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <map>

#include "Const.h"
#include "Game.h"
#include "Directory.h"
#include "Pattern.h"
#include "WholeBoard.h"
#include "FreeSearch.h"
#include "AddDatabase.h"
#include "DbManager.h"


#ifdef WIN32
  #include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
  #include "boost/shared_ptr.hpp"
#endif



class DbManager;

class Database {
  public:
    Database();
    Database(string name, string dirpath, DbManager* dbManager = 0);
    // returns vector of games matching search
    vector<boost::shared_ptr<GamesList> > search(const u16 dbIndex, 
        const u16 moveNumber, const u64 value); 
    
    void printWB(const u16 moveNumber);
    void printWB();
    bool addCollection(const string& dir, const bool recursive);
    void initializeGames();
    void initializeGame(u32 index);
    bool duplicateCheck(bool eraseDuplicates);
    void parseGames();
    void initializePatterns();
    
    
    void openDB();
    void closeDB();
    void printPosuk(u16 moveNumber);
    void printfiles();
    void printGames() const;
    bool showGame(const u32 gameno);
    bool showMenu(const u32 gameno);
    bool traverseGames();
    
    u32 getAllGamesTotal();
    u32 getDuplicatesTotal();
    u32 getGamesTotal();
    string getDatabaseDirPath();
    bool isDir(const string& dir) const; 
    map<u32,boost::shared_ptr<Game> > Games; //Games Indexes
    string getDir(const u32 index) const;
    void loadGamesFormatIndex();
    
    ~Database();
  private:
    
    string dbName_;
    string dbDirpath_; //Where Database files are read / write
    DbManager* dbManager_;
    u32 gamesTotal_;
    u32 duplicatesTotal_;
    map<u32,u32> gamesFormatIdx; // index to something ?
    
    
    
    bool addDirs(const string& dir,const bool recursive);
    void addFiles(const u32 idx, const string& dir);
    void removeDuplicates(bool eraseDuplicates);
    bool startLog();
    
    //! For file input & output from Database
    void writeGamesFormatIndex();
    void readGamesFormatIndex();
    void readDirectory();
    void writeDirectory();
    void readGame();
    void writeGame();
    void readFnvSignatures(); //Called from readGame
    void writeFnvSignatures(); 
    void openInFileStream(const string& fileName);
    void openOutFileStream(const string& fileName);
    
    //-------------------------------------------------------------
    //Data Members
    //-------------------------------------------------------------
    map<u32,boost::shared_ptr<Directory> > Dirs;
    
    multimap<u64,u32> fnvSignatures; //duplicates search
    vector<u32> duplicates; //confirmed duplicates
    
    ifstream ifs_; //For Database read / write indexes
    ofstream ofs_;
    WholeBoard WB; //instance of WholeBoard patterns
    FreeSearch FS;
};

#endif
