// DatabaseTab.cpp
// baduk 0.1 - Neil Mclean
// Started 12/7/2006

#include <QApplication>
#include <QHeaderView>
#include <QVBoxLayout>
#include <QEvent>
#include <QContextMenuEvent>
#include <QVariant>
#include <QSettings>
#include <QScrollBar>
#include <QColorDialog>
#include <QStatusBar> 
#include <QAbstractItemModel>
#include <QStandardItemModel>
#include <QModelIndex>

#ifdef WIN32
  #include <C:\Boost\include\boost-1_33_1\boost\lexical_cast.hpp>
#else
  #include "boost/lexical_cast.hpp"
#endif

#include "DatabaseTab.h"
#include "AddDatabase.h"

using boost::lexical_cast;



/*
Qt::ItemFlags MyModel::flags ( const QModelIndex & index ) const
{
//   qDebug("flags");
   Qt::ItemFlags defaultFlags = QStandardItemModel::flags(index);
//   if (index.isValid())
//     return Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled | defaultFlags;
//   else
//     return Qt::ItemIsDropEnabled | defaultFlags;
  
   return Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled | defaultFlags;
}
*/


/*
bool MyModel::removeRow(int row, const QModelIndex &parent)
{
  std::cout << "removeRow " << row << std::endl;
  if (parent.isValid()) {
    return false;
  }
  return true;
}
*/


MyModel::MyModel(QObject *parent) : QStandardItemModel(parent)
{
}


QStringList MyModel::mimeTypes () const
{
  // qDebug("mime types");
  QStringList list;
  list << "baduK/x-my-mime-type";
  return list;
}


/*
Qt::DropActions MyModel::supportedDropActions() const
{
  std::cout << "supported drop actions" << std::endl;
  return Qt::CopyAction | Qt::MoveAction;
}
*/


/*
bool MyModel::dropMimeData ( const QMimeData * data, Qt::DropAction action, int row, int column, const QModelIndex & parent )
{
  qDebug("dropMimeData");
//   if (action == Qt::CopyAction) {
//      return true;
//   }
//   else {
//     return false;
//   }
}
*/



void MyModel::emit_layoutChanged ( )
{
//   std::cout << "my model emit layout changed" << std::endl;
  emit layoutChanged();
}


/*******************************************************888
// Subclass of TreeView to allow dragging and dropping of items to sort order of databases
***************************************************************/
MyTreeView::MyTreeView(DatabaseTab* parent) : QTreeView(parent)
{
  isDragging_ = false;
}



void MyTreeView::mousePressEvent(QMouseEvent *event)
{
  QTreeView::mousePressEvent(event);
  if ( event->button() == Qt::LeftButton ) {
    dragStartPosition = event->pos();
  }
}


 
void MyTreeView::mouseMoveEvent(QMouseEvent *event)
{
  if ( !(event->buttons() & Qt::LeftButton) ) {
    isDragging_ = false;
    return;
  }
 
  if (((event->pos() - dragStartPosition).manhattanLength()) < 
        QApplication::startDragDistance()) {
    isDragging_ = false;
    return;
  }
  isDragging_ = true;
  QModelIndex index = currentIndex(); //   Select Row on treeView
  mimeData = new QMimeData;
  QByteArray data;
  mimeData->setData ("baduK/x-my-mime-type", data );

  drag = new QDrag(this);
  drag->setMimeData(mimeData);
  drag->setHotSpot(event->pos() - rect().topLeft());
  dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
}



void MyTreeView::dragEnterEvent(QDragEnterEvent *event)
{
  if (event->mimeData()->hasFormat ("baduK/x-my-mime-type")) {
    event->acceptProposedAction();
  }
  else {
    event->ignore();
  }
}



void MyTreeView::dropEvent(QDropEvent *event)
{
  if (event->mimeData()->hasFormat ("baduK/x-my-mime-type")) {
    event->setDropAction(Qt::MoveAction);
    // qDebug ("drop accepted");
    isDragging_ = false;
    
  }
  else {
    event->ignore();
    isDragging_ = false;
  }
  repaint();
}



void MyTreeView::setupModel(MyModel* model)
{
  model_ = model;
}



void MyTreeView::dragMoveEvent (QDragMoveEvent *event)
{
  // std::cout << "drag move event" << std::endl;
  dropSite = event->answerRect();  
  model_->emit_layoutChanged();
}



// Draws the Drag and drop line to indicate where the item will be placed in the list
void MyTreeView::paintEvent ( QPaintEvent* event )
{
  // qDebug("paint Event");  
  QTreeView::paintEvent (event);
  
  // if dragging draw the line
  if (true == isDragging_) {
    QPainter painter ( viewport() ); // Get the painter object created
    QModelIndex modelIndex;
    QModelIndex index = model()->index(0, 0, modelIndex);
    QRect rect = visualRect ( index );
    int rowHeight = rect.height(); //Get height of each row

    int x, y, w, h;
    dropSite.getRect ( &x, &y, &w, &h );
    int iRow = model()->rowCount();
    iRow = iRow * rowHeight; // get the last item position

    // Determine last item position and is drop is past that then draw line there.
    
    y -= (rowHeight / 2); // use the middle of row to change drop location
    if (y < 0) {
      y = 0;
    }
    QPoint point(x,y);
    index = indexAt ( point );
    rect = visualRect ( index );
    int i = rect.y() + rowHeight; // drop location starts at end of first item
    // if mouse position creater than last row but index is 0 then we select last psoition
    if ((y >= iRow) && (i == rowHeight)) {
      i = iRow;
    }
    
    QBrush brush(Qt::black, Qt::Dense4Pattern);
    QPen pen;
    pen.setWidth(2);
    pen.setBrush(brush);
    painter.setPen(pen);
    painter.drawLine ( 0, i, width(), i );
  }
  event->accept();
}

			

/*
void MyTreeView::dropEvent(QDropEvent *event)
{
  std::cout << "dropevent" << std::endl;
// //   textBrowser->setPlainText(event->mimeData()->text());
//   mimeTypeCombo->clear();
//   mimeTypeCombo->addItems(event->mimeData()->formats());
//   event->accept();
  event->acceptProposedAction();
}
*/



DatabaseTab::DatabaseTab(DatabaseViewer* databaseViewer, QWidget *parent) : QWidget(parent),
  databaseViewer_(databaseViewer)
{
  treeView = new MyTreeView(this);
  model = new MyModel;
  
  treeView->setSelectionBehavior(QAbstractItemView::SelectRows);
  treeView->setSelectionMode(QAbstractItemView::SingleSelection);
  treeView->setEditTriggers(QAbstractItemView::NoEditTriggers);
  treeView->setItemsExpandable (false);
  treeView->setRootIsDecorated(false);
  treeView->setAlternatingRowColors (false);
  
//   proxyModel = new QSortFilterProxyModel();
//   proxyModel->setSourceModel(model);
//   treeView->setModel(proxyModel);
  treeView->setModel(model);
  treeView->setupModel(model);
  treeView->header()->setClickable(false);
  treeView->header()->setSortIndicatorShown(false);
//   treeView->header()->setStretchLastSection (false);
  
  selmod = treeView->selectionModel();
  treeView->setDragEnabled(true);
  treeView->setDropIndicatorShown(true);
  treeView->setAcceptDrops(true);
  treeView->setAcceptDrops(true);
  
  treeView->setUpdatesEnabled(true);
  treeView->setMouseTracking (true);
  setMouseTracking (true);
  
  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(treeView);
  setLayout(layout);
  
  databaseMenu = new QMenu("Database menu", this);
  databaseMenu->setTitle("Database menu");
  addDatabaseAct = new QAction(QIcon(":/resources/icons/wizard.png"), tr("&Add database"), this);
  
  removeDatabaseAct = new QAction(QIcon(":/resources/icons/editdelete.png"), tr("&Remove"), this);
  
  setStatusDatabaseAct = new QAction(tr("&Active"), this);
  setStatusDatabaseAct->setCheckable(true);
  
  selectOnlyDatabaseAct = new QAction(tr("&Select only this Database"), this);
  selectAllDatabasesAct = new QAction(tr("&Select all Databases"), this);  
  setColourAct = new QAction(QIcon(":/resources/icons/colorize.png"), tr("&Colour"), this);
  setColourAct->setEnabled(false);
  databaseMenu->addAction (addDatabaseAct);
  databaseMenu->addAction (removeDatabaseAct);
  databaseMenu->addSeparator ();
  databaseMenu->addAction (setStatusDatabaseAct);
  databaseMenu->addAction (selectOnlyDatabaseAct);
  databaseMenu->addAction (selectAllDatabasesAct);
  databaseMenu->addSeparator ();
  databaseMenu->addAction (setColourAct);

  connect(addDatabaseAct, SIGNAL(triggered()), this, SLOT(addDatabaseWizard()));
  connect(removeDatabaseAct, SIGNAL(triggered()), this, SLOT(removeDatabase()));
  connect(setStatusDatabaseAct, SIGNAL(triggered(bool)), this, 
          SLOT(changeStatusDatabase(bool)));
  connect(selectOnlyDatabaseAct, SIGNAL(triggered(bool)), this, 
          SLOT(selectOnlyDatabase()));
  connect(selectAllDatabasesAct, SIGNAL(triggered(bool)), this,
          SLOT(selectAllDatabases()));
  connect(setColourAct, SIGNAL(triggered()), this, SLOT(changeColourDatabase()));
  
  model->insertColumns(0,6);
  setupModel(); // Populate Database Tree View with database header information  
}



DatabaseTab::~DatabaseTab()
{
}



void DatabaseTab::writeSettings()
{
  QSettings settings;
  settings.beginGroup("DatabaseTab");
  settings.setValue("header_Database_Size", 
                    treeView->header()->sectionSize (headerPosDatabase_));
  settings.setValue("header_Created_Size", 
                    treeView->header()->sectionSize (headerPosCreated_));
  settings.setValue("header_Active_Size", 
                    treeView->header()->sectionSize (headerPosActive_));
  settings.setValue("header_Games_Size", 
                    treeView->header()->sectionSize (headerPosGames_));
  settings.setValue("header_Directory_Size", 
                    treeView->header()->sectionSize (headerPosDirectory_));
  settings.setValue("header_Recursive_Size", 
                    treeView->header()->sectionSize (headerPosRecursive_));
  settings.setValue("header_Database_Pos", 
                    treeView->header()->visualIndex (headerPosDatabase_));
  settings.setValue("header_Created_Pos", 
                    treeView->header()->visualIndex (headerPosCreated_));
  settings.setValue("header_Active_Pos", 
                    treeView->header()->visualIndex (headerPosActive_));
  settings.setValue("header_Games_Pos", 
                    treeView->header()->visualIndex (headerPosGames_));
  settings.setValue("header_Directory_Pos", 
                    treeView->header()->visualIndex (headerPosDirectory_));
  settings.setValue("header_Recursive_Pos", 
                    treeView->header()->visualIndex (headerPosRecursive_));
  settings.endGroup();
}


// To display popup menu on right click
void DatabaseTab::contextMenuEvent(QContextMenuEvent *event)
{
  event->accept();
  QModelIndex index = treeView->currentIndex();
  index = model->index(index.row(), headerPosActive_, modelindex);
  if (index.row() >= 0) {
    index = model->index(index.row(), headerPosActive_, modelindex);
    QVariant checked = index.data();
    setStatusDatabaseAct->setChecked(checked.toBool());
    databaseMenu->exec(event->globalPos());
  }
}



bool DatabaseTab::event(QEvent *e)
{
  return QWidget::event(e);
}



void DatabaseTab::setupModel()
{
  model->clear();
  //! Weird on Linux columns dont get deleted when model is cleared
  //! But on Windows we need to insert the columns after a clear();
  #ifdef WIN32
    model->insertColumns(0,6);
  #endif
  
  modelindex = model->index(0, 0, modelindex);
  
  QSettings settings;
  settings.beginGroup("DatabaseTab");
  headerPosDatabase_ = settings.value("header_Database_Pos", 0).toInt();
  headerPosCreated_ = settings.value("header_Created_Pos", 1).toInt();
  headerPosActive_ = settings.value("header_Active_Pos", 2).toInt();
  headerPosGames_ = settings.value("header_Games_Pos", 3).toInt();
  headerPosDirectory_ = settings.value("header_Directory_Pos", 4).toInt();
  headerPosRecursive_ = settings.value("header_Recursive_Pos", 5).toInt();
  
  model->setHeaderData ( headerPosDatabase_, Qt::Horizontal, "Database", Qt::EditRole );
  model->setHeaderData ( headerPosCreated_, Qt::Horizontal, "Created", Qt::EditRole );
  model->setHeaderData ( headerPosActive_, Qt::Horizontal, "Active", Qt::EditRole );
  model->setHeaderData ( headerPosGames_, Qt::Horizontal, "Games", Qt::EditRole );
  model->setHeaderData ( headerPosDirectory_, Qt::Horizontal, "Directory", Qt::EditRole );
  model->setHeaderData ( headerPosRecursive_, Qt::Horizontal, "Recursive", Qt::EditRole );

  
  int size = settings.value("header_Database_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosDatabase_, size);

  size = settings.value("header_Created_Size", 150).toInt();
  treeView->header()->resizeSection(headerPosCreated_, size);

  size = settings.value("header_Active_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosActive_, size);

  size = settings.value("header_Games_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosGames_, size);

  size = settings.value("header_Directory_Size", 150).toInt();
  treeView->header()->resizeSection(headerPosDirectory_, size);

  size = settings.value("header_Recursive_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosRecursive_, size);
  treeView->header()->setStretchLastSection (false);
  
  settings.endGroup();

  DbManager* dbManager = databaseViewer_->mainWindow_->dbManager_;
  model->insertRows(0, dbManager->databases.size(), modelindex);
  QVariant data;
  QModelIndex index;
  dbManager->printDatabases();
    
    for (unsigned int i = 0; i < dbManager->databases.size(); i++) {
        //std::string red = dbManager->databases[i]->red;
        //std::string green = dbManager->databases[i]->green;
        //std::string blue = dbManager->databases[i]->blue;
        //QColor color(lexical_cast<int>(red), lexical_cast<int>(green), lexical_cast<int>(blue));

        data = dbManager->databases[i]->name.c_str();
        index = model->index(i, headerPosDatabase_, modelindex);
        model->setData(index, data);
//         model->setData(index, color, Qt::BackgroundColorRole);
        data = dbManager->databases[i]->dateCreated.c_str();
        index = model->index(i, headerPosCreated_, modelindex);
        model->setData(index, data);
//         model->setData(index, color, Qt::BackgroundColorRole);
        data = dbManager->databases[i]->active.c_str();
        index = model->index(i, headerPosActive_, modelindex);
        model->setData(index, data);
//         model->setData(index, color, Qt::BackgroundColorRole);
        data = dbManager->databases[i]->gamesTotal.c_str();
        index = model->index(i, headerPosGames_, modelindex);
        model->setData(index, data);
//         model->setData(index, color, Qt::BackgroundColorRole);
        data = dbManager->databases[i]->sgfsPath.c_str();
        index = model->index(i, headerPosDirectory_, modelindex);
        model->setData(index, data);
//         model->setData(index, color, Qt::BackgroundColorRole);
        data =dbManager->databases[i]->recursive.c_str();
        index = model->index(i, headerPosRecursive_, modelindex);
        model->setData(index, data);
      }
}



void DatabaseTab::addDatabaseWizard()
{
  databaseViewer_->mainWindow_->addDatabaseWizard();
}


void DatabaseTab::sectionsMoved()
{
  writeSettings();
}



void DatabaseTab::removeDatabase()
{
  QModelIndex index = treeView->currentIndex();
  index = model->index(index.row(), headerPosDatabase_, modelindex);
  QString name = model->data(index).toString();
  std::string strName = name.toStdString();
  databaseViewer_->mainWindow_->dbManager_->deleteDatabase(strName); // Name of database
  setupModel();
}



// FIXME
// Must be better way to save true/false without using strings
void DatabaseTab::changeStatusDatabase(bool on)
{
  QModelIndex index = treeView->currentIndex();
  index = model->index(index.row(), headerPosActive_, modelindex);
  QVariant Vstatus;
  string status;
  if (true == on) {
    status = "true";
    Vstatus = "true";
  }
  else if (false == on) {
    status = "false";
    Vstatus = "false";
  }
  model->setData(index, Vstatus);
  databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->active = status;
  databaseViewer_->mainWindow_->dbManager_->writeDbList();
}



void DatabaseTab::selectOnlyDatabase()
{
  std::string status = "false";
  QVariant Vstatus = "false";
  QModelIndex index;
  for (int row = 0; row < model->rowCount(index.parent()); ++row) {
    index = model->index(row, headerPosActive_, modelindex);
    model->setData(index, Vstatus);
    databaseViewer_->mainWindow_->dbManager_->databases[row]->active = status;
  }
  
  // Now set the selected Database to true
  index = treeView->currentIndex();
  index = model->index(index.row(), headerPosActive_, modelindex);
  status = "true";
  Vstatus = "true";
  model->setData(index, Vstatus);
  databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->active = status;
  databaseViewer_->mainWindow_->dbManager_->writeDbList();
}



// Set all Databases to False - non active
void DatabaseTab::selectAllDatabases()
{
//   QModelIndex index = proxyModel->mapToSource(treeView->currentIndex());
  std::string status = "true";
  QVariant Vstatus = "true";
  QModelIndex index;
  for (int row = 0; row < model->rowCount(index.parent()); ++row) {
    index = model->index(row, headerPosActive_, modelindex);
    model->setData(index, Vstatus);
    databaseViewer_->mainWindow_->dbManager_->databases[row]->active = status;
  }
  databaseViewer_->mainWindow_->dbManager_->writeDbList();
}



void DatabaseTab::changeColourDatabase()
{
  QModelIndex index = treeView->currentIndex();
  QModelIndex index2 = model->index(index.row(), headerPosColour_, modelindex);
  std::string red = databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->red;
  std::string green = databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->green;
  std::string blue = databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->blue;
  int colourRed = lexical_cast<int>(red);
  int colourGreen = lexical_cast<int>(green);
  int colourBlue = lexical_cast<int>(blue);
  QColor colour(colourRed, colourGreen, colourBlue, 255);
  colour = QColorDialog::getColor(colour, this);
  if (colour.isValid()) {
    // Fill row of database table with colour
    for (int col = 0; col < model->columnCount(index.parent()); ++col) {
      QModelIndex sibling = index.sibling(index.row(), col);
      model->setData(sibling, colour, Qt::BackgroundColorRole);
    }
  }
    red = lexical_cast<std::string>(colour.red());
    green = lexical_cast<std::string>(colour.green());
    blue = lexical_cast<std::string>(colour.blue());
    databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->red = red;
    databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->green = green;
    databaseViewer_->mainWindow_->dbManager_->databases[index.row()]->blue = blue;
    databaseViewer_->mainWindow_->dbManager_->writeDbList();
}

