
#include <QEvent>
#include <QCloseEvent>
#include <QSettings>
#include <QVBoxLayout>
#include "DatabaseViewer.h"


DatabaseViewer::DatabaseViewer(MainWindow* mainWindow, QWidget *parent) : QWidget(parent)
{
  mainWindow_ = mainWindow;
  tabWidget = new QTabWidget;
  tabWidget->setTabPosition (QTabWidget::West);
  setMouseTracking(true);
  gamesTab = new GamesTab(mainWindow->dbManager_, this);
  databaseTab = new DatabaseTab(this);
  tabWidget->addTab(gamesTab, tr("Games"));
  tabWidget->addTab(databaseTab, tr("Databases"));
  
  QVBoxLayout *mainLayout = new QVBoxLayout;
  mainLayout->addWidget(tabWidget);
  mainLayout->setMargin (0);
  setLayout(mainLayout);
  setWindowTitle(tr("Games and Databases"));
  setWindowIcon(QIcon(":/resources/icons/misc.png"));
  readSettings();
  tabWidget->setCurrentWidget(databaseTab);
}



void DatabaseViewer::closeEvent(QCloseEvent *event)
{
  writeSettings();
  mainWindow_->databaseViewerAct_->setChecked(false);
  event->accept();
}



void DatabaseViewer::readSettings()
{
  QSettings settings;
  settings.beginGroup("DatabaseViewer");
  QPoint pos = settings.value("pos", QPoint(350, 500)).toPoint();
  QSize size = settings.value("size", QSize(600, 200)).toSize();
  settings.endGroup();
  resize(size);
  move(pos);
}



void DatabaseViewer::writeSettings()
{
  QSettings settings;
  settings.beginGroup("DatabaseViewer");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.endGroup();
  databaseTab->writeSettings();
  gamesTab->writeSettings();
}


