//DbManager.cpp
//baduK++ 0.1 - Neil Mclean 7/3/2006 - Last Updated 7/6/2006

#include <iostream>
#include <string>
#include <map>
#include <cassert>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\date_time\gregorian\gregorian.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\lexical_cast.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\filesystem\operations.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\filesystem\fstream.hpp>
#else
#include "boost/date_time/gregorian/gregorian.hpp"
#include "boost/lexical_cast.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#endif



#include "DbManager.h"
#include "Database.h"
#include "Const.h"

typedef vector<boost::shared_ptr<DatabaseHeader> > dbListT;
typedef map<u16, boost::shared_ptr<Continuations> > mapContinue;

using namespace std;

using boost::filesystem::path;
using boost::filesystem::native;
using boost::filesystem::directory_iterator;
using boost::gregorian::date;
using boost::gregorian::day_clock;
using boost::lexical_cast;


DbManager::DbManager(std::string appDir)
{
  applicationDir_ = appDir;
  filterPlayerW = "";
  filterPlayerB = "";
  filterResult = "";
  filterEvent = "";
  filterPlace = "";
  filterRankW = "";
  filterRankB = "";
  hasAllFiltersEmpty = true;
}



DbManager::~DbManager() {}



// Upon starting program - try to load all the databases.
void DbManager::initializeDatabases() 
{
  if (databases.empty() == false) {
    databases.clear();
  }
  try {

      string fileName = applicationDir_ + "/DbList.dat";
    
    ifstream ifs;
    ifs.clear();
    ifs.open (fileName.c_str(), ios_base::in | ios_base::binary);
    if (ifs.is_open() == false) { // Perhaps no dbList to import
      //throw ios_base::failure (fileName);
      return; //! EXIT
    }
    u16 dbSize;
    ifs.read( reinterpret_cast<char *>(&dbSize), sizeof(dbSize));
    for (u16 i = 1; i <= dbSize; i++) { //for each database entry saved
//       boost::shared_ptr<DatabaseHeader> databaseHeader (new DatabaseHeader);
      DatabaseHeader* databaseHeader = new DatabaseHeader;
      read(ifs, databaseHeader->name);
      read(ifs, databaseHeader->active);
      read(ifs, databaseHeader->sgfsPath);
      read(ifs, databaseHeader->databaseDirPath);
      read(ifs, databaseHeader->dateCreated);
      read(ifs, databaseHeader->recursive);
      read(ifs, databaseHeader->gamesTotal);
      read(ifs, databaseHeader->red);
      read(ifs, databaseHeader->green);
      read(ifs, databaseHeader->blue);
//       read(ifs, databaseHeader->order);
      databases.push_back(databaseHeader);
    }
    ifs.close();
    
//     vector<boost::shared_ptr<DatabaseHeader> >::const_iterator i;
    vector<DatabaseHeader*>::const_iterator i;
    for (i = databases.begin(); i != databases.end(); i++) {
      (*i)->database = new Database ( (*i)->name, (*i)->databaseDirPath);
      (*i)->database->openDB();
      (*i)->database->loadGamesFormatIndex();
    }
//     printDatabases();
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl; 
    abort();
  }
}



void DbManager::writeDbList() {
  try {

      string fileName = applicationDir_ + "/DbList.dat";
    
    ofstream ofs;
    ofs.clear();
    ofs.open(fileName.c_str(), ios_base::out | ios_base::binary | ios::trunc);
    if (ofs.is_open() == false) { 
      throw ios_base::failure (fileName);
    }
    u16 dbSize = databases.size(); // how many databases to save
    ofs.write(reinterpret_cast<char *>(&dbSize), sizeof(dbSize));
//     vector<boost::shared_ptr<DatabaseHeader> >::const_iterator i;
    vector<DatabaseHeader*>::const_iterator i;
    for (i = databases.begin(); i != databases.end(); i++) {
      write(ofs, (*i)->name);
      write(ofs, (*i)->active);
      write(ofs, (*i)->sgfsPath);
      write(ofs, (*i)->databaseDirPath);
      write(ofs, (*i)->dateCreated);
      write(ofs, (*i)->recursive);
      write(ofs, (*i)->gamesTotal);
      write(ofs, (*i)->red);
      write(ofs, (*i)->green);
      write(ofs, (*i)->blue);
//       write(ofs, (*i)->order);
    }
    ofs.close();
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl; 
    abort();
  }
  catch (exception& e) { 
    cerr << e.what() << endl; 
    abort();
  }
}



void DbManager::write(ostream& ofs, const string& str)
{
  u16 size = str.size(); //string::size_type better but undefined length
  ofs.write(reinterpret_cast<char *>(&size), sizeof(size));
  ofs.write(str.c_str(), size+1);
}



// Read in a string - get its length etc.
void DbManager::read(istream& ifs, string& str)
{
  try {
    u16 len = 0;
    ifs.read( reinterpret_cast<char *>(&len), sizeof(len));
    char *tmp = new char [len+1];
    ifs.read(tmp, len+1);
    str = tmp; 
    delete[] tmp;
  }
  catch (bad_alloc& e) {
    cout << "Error allocating memory " << e.what() << endl; 
    abort();
  }
}



//Check if the Directory is valid
bool DbManager::isDir(const string& dirPath) const
{
  path fullPath = system_complete(path(dirPath, native));
  if ((exists(fullPath) == true) && (is_directory(fullPath) == true)) {
    return true;
  }
  return false;
}



setupDatabaseError DbManager::addDatabase(std::string name, std::string path, bool isRecursive ) 
{
//   boost::shared_ptr<DatabaseHeader> databaseHeader (new DatabaseHeader);
  DatabaseHeader* databaseHeader = new DatabaseHeader;
  bool hasName = false;
  vector<DatabaseHeader*>::const_iterator i;
  for (i = databases.begin(); i != databases.end(); i++) {
    if ((*i)->name == name) {
      hasName = true;
    }
  }
  if (true == hasName) {
    return NAME_TAKEN;
  }

  // If no Databases dir then create one
  // if (isDir(applicationDir_ + "/Databases") == false) {
  /*
  string dirPath = applicationDir_ + "/Databases";
  boost::filesystem::path fullPath = system_complete( path(dirPath, native));
  if (exists(fullPath) == false) {
    create_directory(fullPath);
  }
  if (is_directory(fullPath) == false) {
    create_directory(fullPath);
  }
  */
    
    
  //! Doesnt check if a file is of the name - not just dir

  if (isDir(applicationDir_ + "/Databases/" + name) == true) {
    return DIR_TAKEN;
  }
  

  databaseHeader->name = name;
  

      string dirPath = applicationDir_ + "/Databases/" + name; 
    
  // PATH - probably current path + db name
  databaseHeader->databaseDirPath = dirPath; // path of generated db files to be stored
  databaseHeader->sgfsPath = path; // dir for sgf collection to start at
  if (true == isRecursive) {
    databaseHeader->recursive = "true";
  }
  else if (false == isRecursive) {
    databaseHeader->recursive = "false";
  }
  databaseHeader->active = "true";
  
  // Construct the date and time to store.
  time_t now = time(0); // Convert now to tm struct for local timezone
  // tm* localtm = localtime(&now);
  tm* gmtm = gmtime(&now); // Convert now to tm struct for UTC
  
    //if (0 == gmtm) { // The UTC date and time is: " << asctime(gmtm)
      //cerr << "Failed to get the UTC date and time" << endl; 
    //}
//   if (true == isLocalDateTime_) {
//     ofs_ << "\n\n** Badukk LogFile " << asctime(localtm) << endl;
//   }
//   else {
//     ofs_ << "\n\n** Badukk LogFile " << asctime(gmtm) << endl;
//   }
  
//   date d(day_clock::local_day());
  databaseHeader->dateCreated = asctime(gmtm);
//   databaseHeader->dateCreated = to_iso_extended_string(d) + " " + asctime(gmtm);
  
  databaseHeader->red = "200";
  databaseHeader->green = "230";
  databaseHeader->blue = "200";
  
  // Order of database as viewed in the DatabaseTab of GUI
  // Set to the last position
//   databaseHeader->order = lexical_cast<string>(databases.size());
      
  databases.push_back(databaseHeader);
  databaseHeader->database = new Database(name, dirPath, this);
  bool validDirectory = databaseHeader->database->isDir(path);
  if (false == validDirectory) {
    return BAD_DIRECTORY;
  }
  bool validCollection = databaseHeader->database->addCollection(path, isRecursive);
  if (false == validCollection) {
    return NO_SGF;
  }
//   u32 gamesTotal = databaseHeader->database->getGamesTotal();
//   if (0 == gamesTotal) {
//     return NO_SGF;
//   }

  return ACCEPT;
}



// Add database to DbList.
setupDatabaseError DbManager::setupDatabase(MainWindow* mainWindow, bool eraseDuplicates) 
{
  mainWindow_ = mainWindow;
  try {
//     boost::shared_ptr<DatabaseHeader> databaseHeader (new DatabaseHeader);
    DatabaseHeader* databaseHeader = new DatabaseHeader;
    databaseHeader = databases.back();
    
    Timer timer(); //Start clock
    
    databaseHeader->database->initializeGames();
    
    databaseHeader->database->duplicateCheck(eraseDuplicates);
    
    u32 gamesTotal = databaseHeader->database->getGamesTotal();
    mainWindow_->addDatabase_->progressPage->setLabelTotalGamesData(gamesTotal);
 
   u32 duplicatesTotal = databaseHeader->database->getDuplicatesTotal();

    mainWindow_->addDatabase_->progressPage->setLabelDuplicateGamesData(duplicatesTotal);

    databaseHeader->gamesTotal = lexical_cast<std::string>(gamesTotal);
    
    databaseHeader->database->closeDB();

    // qDebug("about to parse games");
    databaseHeader->database->parseGames();
    databaseHeader->database->initializePatterns();
    writeDbList();
  }
  catch (GuiCancelAddDatabase&) {
    return CANCEL;
  }
  catch (bad_alloc& e) {
    cout << "Error allocating memory to new database. " << 
        e.what() << endl; 
  }
  return ACCEPT;
}



// Returns pointer to database
Database* DbManager::getDatabasePointer(std::string &name)
{
  vector<DatabaseHeader*>::iterator j = databases.begin();
  while ((*j)->name != name) {
//     std::cout << (*j)->name << " : " << name << std::endl;
    j++;
  }
  return (*j)->database;
}



void DbManager::deleteDatabase(std::string &name) 
{
//   cout << "in del db" << endl;
//   assert(false == databases.empty());
  
//   cout << "db size" << databases.size() << endl;
//   if (false == databases.empty()) {
//     cout << "in del db 1a" << endl;
//     vector<boost::shared_ptr<DatabaseHeader> >::iterator j = databases.begin();
    vector<DatabaseHeader*>::iterator j = databases.begin();
    
//     cout << "in del db 2" << endl;
//     bool hasDatabase = false;
    
//     for (j = databases.begin(); j !=databases.end(); j++)
//     {
//       if ((*j)->name == name) {
//         hasDatabase = true;
//       }
//     }
    
    while ((*j)->name != name) {
//        std::cout << (*j)->name << " : " << name << std::endl;
      j++;
    } // Should always return a database match
//     std::cout << (*j)->name << " : " << name << std::endl;
//     if (true == hasDatabase) {
//       cout << "in del db 2" << endl;
    
    
      (*j)->database->~Database(); // delete object pointer
//       cout << "in del db 2a" << endl;
      
      boost::filesystem::path fullPath = system_complete(path((*j)->databaseDirPath, native));
      if (exists(fullPath)) {
        //cerr << "Error: the directory " << thePath.string( )
        //    << " does not exist.\n";
        removeRecurse(fullPath); // Try to remove directory
      }
//       cout << "in del db 3" << endl;
      databases.erase(j);
//       cout << "in del db 4" << endl;
      writeDbList();
//     else {
      //cout << "No database matches for name" << endl;
//     }
//   }
//   else {
    //cout << "No databases" << endl;
//   }
}



void DbManager::removeRecurse(const path& p) const {
  // First, remove the contents of the directory
  try {
    directory_iterator end;
    for (directory_iterator it(p); it != end; ++it) {
      if (is_directory(*it)) {
        removeRecurse(*it);
      }
      else {
        remove(*it);
      }
    }
    remove(p); // Then, remove the directory itself
  }
  catch (exception& e) {
    cerr << e.what() << endl;
  }
}


void DbManager::setFilters(string playerB, string playerW, string rankB,
    string rankW, string result)
{
  filterPlayerB = playerB;
  filterPlayerW = playerW;
  filterRankB = rankB;
  filterRankW = rankW;
  filterResult = result;
  hasAllFiltersEmpty = false;
  if (filterPlayerB == "") {
    if (filterPlayerW == "") {
      if (filterResult == "") {
        if (filterRankB == "") {
          if (filterRankW == "") {
            hasAllFiltersEmpty = true;
          }
        }
      }
    }
  }
//   filterEvent;
//   filterPlace;
}


void DbManager::searchPattern(const u16 moveNumber, const u64 value, u8 wbPatternSelected)
{
  if (totalResults.empty() == false) {
    totalResults.clear();
  }
  if (filteredResults.empty() == false) {
    filteredResults.clear();
  }
  // Only search if movenumber is within limits of patterns that were generated
  if (moveNumber < WHOLEBOARD_DEPTH) {
    
    //for each database do a search and collate results into one large one ;)
    //dbListT::const_iterator i;
    //for (i = dbList.begin(); i != dbList.end(); i++) {
    for ( u16 i = 0; i < databases.size(); i++) { // dbIndex
      //cout << "DBname=" << i->first << endl;
      if ("true" == databases[i]->active) {
        searchResults = databases[i]->database->search(i, moveNumber, value);
        // Append results to back of total result vector
//         copy(searchResults.begin(), searchResults.end(),
//               back_inserter(totalResults));
        copy(searchResults.begin(), searchResults.end(),
               back_inserter(filteredResults));
      }
//       cout << "returned searchresults" << endl;
      //string dud; cin >> dud;
      //Collect results and append to one list 
    }
  }
  
  doStats(wbPatternSelected);
//   printStats();
  
  
  // Filtering of search to include only selected games
  /*
  vector<boost::shared_ptr<GamesList> >::const_iterator i;
  if (totalResults.empty() == false) {
    for (i = totalResults.begin(); i != totalResults.end(); i++) {
      // Check if the game contains a string matching Cho for Black player
      // Get the pointer to game
      u16 dbIndex = (*i)->dbIndex;
      u32 gameIndex = (*i)->gameIndex;
      boost::shared_ptr<Game> game;
      game = databases[dbIndex]->database->Games[gameIndex];
      if (true == hasAllFiltersEmpty) { // All filters are empty
        filteredResults.push_back(*i);
      }
      else {
        if (game->getPlayerB().find( filterPlayerB ) != string::npos) {
          if (game->getPlayerW().find( filterPlayerW ) != string::npos) {
            if (game->getResultStr().find( filterResult ) != string::npos) {
              if (game->getRankB().find( filterRankB ) != string::npos) {
                if (game->getRankW().find( filterRankW ) != string::npos) {
                  filteredResults.push_back(*i);
                }
              }
            }
          }
        }
      }
    }
  }
  */
//   qDebug("before do stats");
  
//   qDebug("after do stats");
  //   sortStatsByTotalGames();
  //   printStats();
  //   sortStatsByWinPrecentage();
  
  //   sortStatsByBestVariation();
  //    printStats();
}





FullMove DbManager::unInvariantMove(FullMove move, u8 wbPatternSelected) 
{
//   cout << "wbPatternSelected_ = " << (int)wbPatternSelected << endl;
  FullMove tmp;
//   u8 count = 0;
  if (1 == wbPatternSelected) { // 1 Horizontal
    move.x = END_BOARD - move.x;
//     cout << "1 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  if (2 == wbPatternSelected) { // 2  vert
    move.y = END_BOARD - move.y;
//     cout << "2 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  if (3 == wbPatternSelected) { // 3 hor vert
    move.y = END_BOARD - move.y;
    move.x = END_BOARD - move.x;
//     cout << "3 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  if (4 == wbPatternSelected) { // 4 diag
    tmp.x = END_BOARD - move.y;
    move.y = END_BOARD - move.x;
    move.x = tmp.x;
//     ++count;
//     cout << "4 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  if (5 == wbPatternSelected) { // 5 hor + diag opp
   
    tmp.x = move.y;
    move.y = move.x;
    move.x = tmp.x;
    move.y = END_BOARD - move.y;
//     cout << "5 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  if (6 == wbPatternSelected) { // 6 vert + diag opp
    
    tmp.x = move.y;
    move.y = move.x;
    move.x = tmp.x;
    move.x = END_BOARD - move.x;
//     cout << "6 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  if (7 == wbPatternSelected) { // 7 hor + vert + diag
    
    tmp.x = END_BOARD - move.y;
    move.y = END_BOARD - move.x;
    move.x = tmp.x;
    move.x = END_BOARD - move.x;
    move.y = END_BOARD - move.y;
//     cout << "7 unIvnvariant selected : x&y=" << (int)move.x << " " << (int)move.y << endl;
  }
  return move;
}



//do some analysis of results such as for each next move
//if nextmove x&y is unique insert a new into map
//if not find and add result to nextmove in map
//create a u16 for the nextmove x,y position
void DbManager::doStats(u8 wbPatternSelected) 
{
  if (statsBlack.empty() == false) {
    statsBlack.clear();
  }
  if (statsWhite.empty() == false) {
    statsWhite.clear();
  }
  vector<boost::shared_ptr<GamesList> >::const_iterator i;
  vector<boost::shared_ptr<Continuations> >::iterator j;
  bool nextMoveFound;
  FullMove move;
  if (filteredResults.empty() == false) {
    for (i = filteredResults.begin(); i != filteredResults.end(); i++) {
//       cout << "DB Idx=" << static_cast<int>((*i)->dbIndex) << " ";
//       cout << "G Idx=" << static_cast<int>((*i)->gameIndex) << " ";
//       cout << "x&y=" << static_cast<int>((*i)->nextMoveX) << " ";
//       cout << static_cast<int>((*i)->nextMoveY) << " ";
//       cout << "imove=" << static_cast<int>((*i)->moveNumber) << " ";
//       cout << "nextcol=" << static_cast<int>((*i)->nextMoveColour) << " ";
//       cout << "res=" << static_cast<int>((*i)->result) << endl;
  
      //------------------------------------------------------------------------------
      //! Invariant next Move - need to tranform the next move to correct position
      //!
      move.x = (*i)->nextMoveX;
      move.y = (*i)->nextMoveY;
//       cout << "wbPatternSelected=" << (int)wbPatternSelected << " ";
      move = unInvariantMove(move, wbPatternSelected);
      
//       cout << "new invariant x&y=" << static_cast<int>(move.x) << " ";
//       cout << static_cast<int>(move.y) << " ";
      //------------------------------------------------------------------------------
      
      // Do two vectors of stats, one for the next move WHITE and for BLACK
      nextMoveFound = false;
      
//       assert((BLACK == (*i)->nextMoveColour) || (WHITE == (*i)->nextMoveColour));
//       if ((BLACK == (*i)->nextMoveColour) && (WHITE == (*i)->nextMoveColour)) {
//         cout << "next move colour = " << (int)(*i)->nextMoveColour << endl;
//       }
      
      if (BLACK == (*i)->nextMoveColour) {
        j = statsBlack.begin();
        while ((j != statsBlack.end()) && (false == nextMoveFound)) {
          if (((*j)->nextMoveX == move.x) && ((*j)->nextMoveY == move.y)) {
            nextMoveFound = true;
          }
          else {
            j++;
          }
        }
      }
      else if (WHITE == (*i)->nextMoveColour) {
        j = statsWhite.begin();
        while ((j != statsWhite.end()) && (false == nextMoveFound)) {
          if (((*j)->nextMoveX == move.x) && ((*j)->nextMoveY == move.y)) {
            nextMoveFound = true;
          }
          else {
            j++;
          }
        }
      }
  //     cout << "doStats2" << endl;
      if (false == nextMoveFound) {
        boost::shared_ptr<Continuations> continuations (new Continuations);
        continuations->nextMoveX = move.x;
        continuations->nextMoveY = move.y;
        continuations->blackWins = 0;
        continuations->whiteWins = 0;
        continuations->jigo = 0;
        if (BLACK == (*i)->nextMoveColour) {
          statsBlack.push_back(continuations);
          j = statsBlack.end();
          j--;
        }
        else if (WHITE == (*i)->nextMoveColour) {
          statsWhite.push_back(continuations);
          j = statsWhite.end();
          j--;
        }
      }
  //     cout << "doStats3" << endl;
      switch ((*i)->result) {
        case 0 : { //b win
          ++((*j)->blackWins); 
          break; 
        }
        case 1 : { //w win
          ++((*j)->whiteWins); 
          break; 
        }
        case 2 : { // jigo
          ++((*j)->jigo);
          break;
        }
        case 3 : { // b+t
          ++((*j)->blackWins); 
          break;  
        }
        case 4 : { //w+t
          ++((*j)->whiteWins); 
          break; 
        }
        // This should not happen since no result is done as jigo in SGF.cpp
//         default : { // In case of no result select a jigo
//           qDebug("switch default jigo");
//           ++((*j)->jigo);
//           break;
//         }
      }
    }
  }
}



vector<boost::shared_ptr<GamesList> > DbManager::getFilteredResults()
{
  return filteredResults;
}



// Returns the stats to the MainBoard. To be stored there
vector<boost::shared_ptr<Continuations> > DbManager::getStatsBlack()
{
//   qDebug("getStatsBlack");
  return statsBlack;
}



// Returns the stats to the MainBoard. To be stored there
vector<boost::shared_ptr<Continuations> > DbManager::getStatsWhite()
{
  return statsWhite;
}



//print the statistics after a search
void DbManager::printStats() const
{
//   assert(nextMoveStatsBlack.empty() != false);
//   assert(nextMoveStatsWhite.empty() != false);
  vector<boost::shared_ptr<Continuations> >::const_iterator i;
  cout << endl << "stats Black next move- continuations" << endl;
  for (i = statsBlack.begin(); i != statsBlack.end(); i++) {
    cout << "x&y=" << static_cast<int>((*i)->nextMoveX) << " "; 
    cout << static_cast<int>((*i)->nextMoveY) << " ";
    cout << "Bwins=" << static_cast<int>((*i)->blackWins) << " ";
    cout << "Wwins" << static_cast<int>((*i)->whiteWins) << " ";
    cout << "jigo" << static_cast<int>((*i)->jigo) << endl;
  }
  cout << endl << "stats White next move- continuations" << endl;
  for (i = statsWhite.begin(); i != statsWhite.end(); i++) {
    cout << "x&y=" << static_cast<int>((*i)->nextMoveX) << " "; 
    cout << static_cast<int>((*i)->nextMoveY) << " ";
    cout << "Bwins=" << static_cast<int>((*i)->blackWins) << " ";
    cout << "Wwins" << static_cast<int>((*i)->whiteWins) << " ";
  cout << "jigo" << static_cast<int>((*i)->jigo) << endl;
  }
}



//! Note on sorting stats order
//! Calculate winning percentage of the games, and multiply by number of games
//! To find the best choice of next move.
//! This eliminates low occuring variations with high winning percentages.

// Provide choices of sorting criteria, nice eye candy and easy to implement ;)

// Do a buble sort as we have typically a small amount data to sort.
// Bubble sort is inefficient compared to quicksort but adequate for small data sets.
// Typically the user will have less than 10 databases.
// It is possible to have many databases, but in practice no user would do this.
// Certainly not more than 100.
void DbManager::sortStatsByTotalGames()
{
//   cout << endl << "sort stats 1 " << endl;
  vector<boost::shared_ptr<Continuations> >::iterator i;
  vector<boost::shared_ptr<Continuations> >::iterator j;
  int totalGamesFirst;
  int totalGamesSecond;
  if (statsBlack.size() > 1) {
    for (i = statsBlack.end(); i != statsBlack.begin(); i--) {
      for (j = statsBlack.begin(); j != (i - 1); j++) {
        assert(j != statsBlack.end());
        totalGamesFirst = (*j)->blackWins + (*j)->whiteWins + (*j)->jigo;
        totalGamesSecond = (*(j + 1))->blackWins + (*(j + 1))->whiteWins + 
            (*(j + 1))->jigo;
        if (totalGamesFirst < totalGamesSecond) {
          (*j).swap(*(j + 1));
        }
      }
    }
  }
//   cout << endl << "sort stats 2 " << endl;
  if (statsWhite.size() > 1) {
    for (i = statsWhite.end(); i != statsWhite.begin(); i--) {
      for (j = statsWhite.begin(); j != (i - 1); j++) {
        assert(j != statsWhite.end());
        totalGamesFirst = (*j)->blackWins + (*j)->whiteWins + (*j)->jigo;
        totalGamesSecond = (*(j + 1))->blackWins + (*(j + 1))->whiteWins + 
            (*(j + 1))->jigo;
        if (totalGamesFirst < totalGamesSecond) {
          (*j).swap(*(j + 1));
        }
      }
    }
  }
}



void DbManager::sortStatsByWinPrecentage()
{
  vector<boost::shared_ptr<Continuations> >::iterator i;
  vector<boost::shared_ptr<Continuations> >::iterator j;
  double percentageFirst;
  double percentageSecond;
  if (statsBlack.size() > 1) {
    for (i = statsBlack.end(); i != statsBlack.begin(); i--) {
      for (j = statsBlack.begin(); j != (i - 1); j++) {
        assert(j != statsBlack.end());
        if (0 == (*j)->whiteWins) {
          percentageFirst = 1; 
        }
        else {
          percentageFirst = static_cast<double>((*j)->blackWins) / 
              (((*j)->blackWins) + (*j)->whiteWins);
//           percentageFirst = percentageFirst * 100;
        }
        if (0 == (*(j + 1))->whiteWins) {
          percentageSecond = 1; 
        }
        else {
          percentageSecond = static_cast<double>((*(j + 1))->blackWins) / 
              (((*(j + 1))->blackWins) + (*(j + 1))->whiteWins);
//           percentageSecond = percentageSecond * 100;
        }
//         cout << "Black Wins = " << (*j)->blackWins << "  percentage1 =" << percentageFirst <<
//             "  percentage2 =" << percentageSecond << endl;
        if (percentageFirst < percentageSecond) {
          (*j).swap(*(j + 1));
        }
      }
    }
  }
//   cout << endl << "sort stats 2 " << endl;
  if (statsWhite.size() > 1) {
    for (i = statsWhite.end(); i != statsWhite.begin(); i--) {
      for (j = statsWhite.begin(); j != (i - 1); j++) {
        assert(j != statsWhite.end());
        if (0 == (*j)->blackWins) {
          percentageFirst = 1;
        }
        else {
          percentageFirst = static_cast<double>((*j)->whiteWins) / 
              (((*j)->blackWins) + (*j)->whiteWins);
//           percentageFirst = percentageFirst * 100;
        }
        if (0 == (*(j + 1))->blackWins) {
          percentageSecond = 1; 
        }
        else {
          percentageSecond = static_cast<double>((*(j + 1))->whiteWins) / 
              (((*(j + 1))->blackWins) + (*(j + 1))->whiteWins);
//           percentageSecond = percentageSecond * 100;
        }
//         double r = 0.333;
//         cout << r << endl;
//         cout << "Black Wins = " << (*j)->blackWins << "  percentage1 =" << percentageFirst <<
//             "  percentage2 =" << percentageSecond << endl;
        
        if (percentageFirst < percentageSecond) {
          (*j).swap(*(j + 1));
        }
      }
    }
  }
}



//
void DbManager::sortStatsByBestVariation()
{
  vector<boost::shared_ptr<Continuations> >::iterator i;
  vector<boost::shared_ptr<Continuations> >::iterator j;
  double totalFirst;
  double totalSecond;
  double percentageFirst;
  double percentageSecond;
  if (statsBlack.size() > 1) {
    for (i = statsBlack.end(); i != statsBlack.begin(); i--) {
      for (j = statsBlack.begin(); j != (i - 1); j++) {
        assert(j != statsBlack.end());
        if (0 == (*j)->whiteWins) {
          percentageFirst = 1; 
        }
        else {
          percentageFirst = static_cast<double>((*j)->blackWins) / 
              (((*j)->blackWins) + (*j)->whiteWins);
        }
        if (0 == (*(j + 1))->whiteWins) {
          percentageSecond = 1; 
        }
        else {
          percentageSecond = static_cast<double>((*j)->blackWins) / 
              (((*j)->blackWins) + (*j)->whiteWins);
        }
        
        totalFirst = ((*j)->blackWins + (*j)->whiteWins) *
            percentageFirst;
        totalSecond = ((*(j + 1))->blackWins + (*(j + 1))->whiteWins) *
            percentageSecond;
        if (totalFirst < totalSecond) {
          (*j).swap(*(j + 1));
        }
      }
    }
  }
//   cout << endl << "sort stats 2 " << endl;
  if (statsWhite.size() > 1) {
    for (i = statsWhite.end(); i != statsWhite.begin(); i--) {
      for (j = statsWhite.begin(); j != (i - 1); j++) {
        assert(j != statsWhite.end());
        if (0 == (*j)->blackWins) {
          percentageFirst = 1; 
        }
        else {
          percentageFirst = static_cast<double>((*(j + 1))->whiteWins) / 
              (((*(j + 1))->blackWins) + (*(j + 1))->whiteWins);
        }
        if (0 == (*(j + 1))->blackWins) {
          percentageSecond = 1; 
        }
        else {
          percentageSecond = static_cast<double>((*(j + 1))->whiteWins) / 
              (((*(j + 1))->blackWins) + (*(j + 1))->whiteWins);
        }
        
        totalFirst = ((*j)->blackWins + (*j)->whiteWins) * 
            percentageFirst;
        totalSecond = ((*(j + 1))->blackWins + (*(j + 1))->whiteWins) *
            percentageSecond;
        if (totalFirst < totalSecond) {
          (*j).swap(*(j + 1));
        }
      }
    }
  }
}



/////////////////////////////////////////////////////////////////////
//! DEBUG CODE
/////////////////////////////////////////////////////////////////////

void DbManager::printDatabases() const 
{
  //print a list of databases
/*
  int count = 0;
  vector<DatabaseHeader*>::const_iterator j;
  for (j = databases.begin(); j != databases.end(); j++) {
    cout << count++ << " dbName : " << (*j)->name;
    cout << "  dbDirpath : " << (*j)->databaseDirPath << endl;
  }
*/
}



