#ifndef DBMANAGER_H
#define DBMANAGER_H

#include <map>
#include <vector>
#include <string>

#include "Const.h"
#include "Database.h"
#include "AddDatabase.h"
#include "MainWindow.h"

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\filesystem\operations.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\filesystem\fstream.hpp>
#else
#include "boost/shared_ptr.hpp"
#include "boost/filesystem/operations.hpp"
#include "boost/filesystem/fstream.hpp"
#endif





class Database;
class AddDatabase;
class MainWindow;

using namespace std;

class GuiCancelAddDatabase {
  const char* const data;
public:
  GuiCancelAddDatabase(const char* const msg = 0) : data(msg) {}
};


    


// used to create statistics on next move continuations
struct Continuations {
    u8 nextMoveX;
    u8 nextMoveY;
    u32 blackWins;
    u32 whiteWins;
    u32 jigo;
//     u8 nextMoveColour;
};



class DatabaseHeader {
  public:
    std::string name;
    std::string active;
    std::string sgfsPath;
    std::string databaseDirPath;
    std::string dateCreated;
    std::string recursive;
    std::string gamesTotal;
    std::string red;
    std::string green;
    std::string blue;
//     std::string order;
    Database* database;
};



class DbManager {
  public:
    DbManager(std::string appDir);
    void initializeDatabases();
    setupDatabaseError addDatabase(std::string name, std::string path, bool isRecursive );
    setupDatabaseError setupDatabase(MainWindow* mainWindow, bool eraseDuplicates);
    void deleteDatabase(std::string &name);
    void doStats(u8 wbPatternSelected);
    void printStats() const;
    void sortStatsByTotalGames();
    void sortStatsByWinPrecentage();
    void sortStatsByBestVariation();
    void searchPattern(const u16 moveNumber, const u64 value, u8 wbPatternSelected);
    Database* getDatabasePointer(std::string &name);
    void writeDbList();
//     vector<boost::shared_ptr<DatabaseHeader> > databases;
    vector<DatabaseHeader*> databases;
    vector<boost::shared_ptr<GamesList> > totalResults;
    vector<boost::shared_ptr<GamesList> > filteredResults;
//     AddDatabase* addDatabase_;
    MainWindow* mainWindow_;
    
    vector<boost::shared_ptr<Continuations> > statsBlack;
    vector<boost::shared_ptr<Continuations> > statsWhite;
    void setFilters(string playerB, string playerW, string rankB, string rankW, string result);

    vector<boost::shared_ptr<GamesList> > getFilteredResults();
    vector<boost::shared_ptr<Continuations> > getStatsBlack();
    vector<boost::shared_ptr<Continuations> > getStatsWhite();
    void printDatabases() const;

    
    ~DbManager();
  private:

    FullMove unInvariantMove(FullMove move, u8 wbPatternSelected);
    void removeRecurse(const boost::filesystem::path& p) const;
    bool isDir(const std::string& dirPath) const;
    
    
    std::string applicationDir_;
    void write(ostream& ofs, const std::string& str);
    void read(istream& ifs, std::string& str);
    
    vector<boost::shared_ptr<GamesList> > searchResults;
    
    bool hasAllFiltersEmpty;
    string filterPlayerW;
    string filterPlayerB;
    string filterResult;
    string filterEvent;
    string filterPlace;
    string filterRankW;
    string filterRankB;
    
//     map<u16, boost::shared_ptr<Continuations> > nextMoveStatsBlack; //nextMove x&y
//     map<u16, boost::shared_ptr<Continuations> > nextMoveStatsWhite; //nextMove x&y
   
    
    
//     friend class DatabaseTab;
};

#endif
