// Game.cpp 
// baduK++ 0.1 - Neil Mclean 
// Started 5/4/2006 

#include <iostream>
#include <fstream>
#include <string>
#include <iomanip>
#include <map>
#include <QDebug>

#include "Game.h"
#include "Sgf.h"

using namespace std;

Game::Game() : seekPos_(0), index_(0), dirkey_(0), moveCount_(0),
    result_(0), fnvSignature_(0) 
{
}


Game::Game(u32 idx, u32 key, string name) : seekPos_(0), index_(idx), 
    dirkey_(key),  moveCount_(0), result_(0), fnvSignature_(0),
    filename_(name) 
{
}


Game::~Game() { 
  //cout << "Game destructor called" << endl; 
}


FullMove Game::getMove(u16 moveNumber) const 
{
  return moves.at(moveNumber);
}



//---------------------------------------------------------
//! Public get & set functions
//---------------------------------------------------------
string Game::getFilename() const { return filename_; }
u32 Game::getDirkey() const { return dirkey_; }
u32 Game::getIndex() const { return index_; }
u16 Game::getMoveCount() const { return moveCount_; }
string Game::getPlayerW() const { return playerW_; }
string Game::getPlayerB() const { return playerB_; }
string Game::getRankW() const { return rankW_; }
string Game::getRankB() const { return rankB_; }
string Game::getKomi() const { return komi_; }
string Game::getDate() const { return date_; }
string Game::getResultStr() const { return resultString_; }
string Game::getHandicap() const { return handicap_; }
string Game::getPlace() const { return place_; }
string Game::getEvent() const { return event_; }

string Game::getApplication() const { return application_; }
string Game::getTime() const { return time_; }
string Game::getOverTime() const { return overTime_; }
string Game::getRound() const { return round_; }
string Game::getCopyright() const { return copyright_; }
string Game::getRules() const { return rules_; }
string Game::getGameName() const { return gameName_; }
string Game::getGameComment() const { return gameComment_; }
string Game::getWhiteTeam() const { return whiteTeam_; }
string Game::getBlackTeam() const { return blackTeam_; }
string Game::getSource() const { return source_; }
string Game::getAnnotator() const { return annotator_; }
string Game::getTranscriber() const { return transcriber_; }


u8 Game::getResult() const 
{ 
  return result_; 
}



u64 Game::getFnvSignature() const { return fnvSignature_; }

void Game::setDirkey(const u32 key) { dirkey_ = key; }
void Game::setMoveCount(const u16 value) { moveCount_ = value; }
void Game::setFilename(const string& value) { filename_ = value; }
void Game::setPlayerW(const string& value) { playerW_ = value; }
void Game::setPlayerB(const string& value) { playerB_ = value; }
void Game::setRankW(const string& value) { rankW_ = value; }
void Game::setRankB(const string& value) { rankB_ = value; }
void Game::setResultStr(const string& value) { resultString_ = value; }
void Game::setKomi(const string& value) { komi_ = value; }
void Game::setDate(const string& value) { date_ = value; }
void Game::setResult(const u8 value) { result_ = value; }
void Game::setHandicap(const string& value) { handicap_ = value; }
void Game::setPlace(const string& value) { place_ = value; }
void Game::setEvent(const string& value) { event_ = value; }


//Used by GUI to get the Game Info and populate GameInfo tab
void Game::initializeGameInfo(const string& dir)
{
  Sgf sgf;
  sgf.read(dir + '/' + filename_);
  playerW_ = sgf.getPlayerW();
  playerB_ = sgf.getPlayerB();
  rankW_ = sgf.getRankW();
  rankB_ = sgf.getRankB();
  resultString_ = sgf.getResultStr();
  result_ = sgf.getResult();
  komi_ = sgf.getKomi();
  date_ = sgf.getDate();
  handicap_ = sgf.getHandicap();
  place_ = sgf.getPlace();
  event_ = sgf.getEvent();
  application_ = sgf.getApplication();
  time_ = getTime();
  overTime_ = getOverTime();
  round_ = getRound();
  copyright_ = getCopyright();
  rules_ = getRules();
  gameName_ = getGameName();
  gameComment_ = getGameComment();
  whiteTeam_ = getWhiteTeam();
  blackTeam_ = getBlackTeam();
  source_ = getSource();
  annotator_ = getAnnotator();
  transcriber_ = getTranscriber();
}



//----------------------------------------------------------------------
//Collect sgf game info and extract moves into a vector
//create a temporary instance of object 'Sgf' to gather information
//----------------------------------------------------------------------
bool Game::initializeGame(const string& dir) 
{
  Sgf sgf;
  sgf.read(dir + '/' + filename_);
  playerW_ = sgf.getPlayerW();
  playerB_ = sgf.getPlayerB();
  rankW_ = sgf.getRankW();
  rankB_ = sgf.getRankB();
  resultString_ = sgf.getResultStr();
  result_ = sgf.getResult();
  komi_ = sgf.getKomi();
  date_ = sgf.getDate();
  handicap_ = sgf.getHandicap();
  place_ = sgf.getPlace();
  event_ = sgf.getEvent();
  moveCount_ = sgf.extractMoves();
  if ( moveCount_ > 0 ) {
    if (moves.empty() == false) { 
      moves.clear(); 
    }
    // ensure all games start in same first move area (top right)
    // and symmetrical games have moves that all identification of duplicates by 
    // transforming all game moves to negate symmetrical positions.
    sgf.movesInvariant(); // Return bool, not used
    
    for (int i = 0; i < moveCount_; i++) { 
      moves.push_back(sgf.getMove(i)); 
      // cout << "xy=" << (int)moves[i].x << " " << (int)moves[i].y << endl;
    }
    
    fnvSignature_ = sgf.fnvSignature();
    return true;
  }
  return false;
}



//-----------------------------------------------------------
//! For pattern generation
//! Use one single board to avoid copying boards.
//-----------------------------------------------------------
bool Game::doPatternBoard(const u16 iMove) 
{
  //   qDebug("doPatternBoard");
  patternBoard_->clearCaptures();
  //   qDebug("doPatternBoard clearcaptures done");

  if (iMove < moves.size()) {
    FullMove move = moves.at(iMove); //get the move
    //if ((move.x >= 0) && (move.x <= 19) && (move.y >= 0) && (move.y <= 19)) {
      // This is equivalent of this since we are using unsigned u8's
    if ((move.x <= 18) && (move.y <= 18)) { // Legal move
      //assert ((move.x <= 18) && (move.y <= 18)); // Legal move
	
      patternBoard_->setStone(move.x, move.y, move.colour);
      //cout << " do move xy col =" << (int)move.x << " " << (int)move.y << endl;
      bool isDead = patternBoard_->checkDeadGroup(move.x , move.y);
      //cout << "checked dead group" << endl;
      //assert ((BLACK == move.colour) || (WHITE == move.colour));
      
      u8 enemy = BLACK;
      if (BLACK == move.colour) { 
        enemy = WHITE; 
      }
      else if (WHITE == move.colour) { 
        enemy = BLACK; 
      }
      bool hasKilled = false;
      if (patternBoard_->removeDeadGroup((move.x) -1, move.y, enemy) == true) {
        hasKilled = true; 
      }
      if (patternBoard_->removeDeadGroup(move.x, (move.y) -1, enemy) == true) {
        hasKilled = true; 
      }
      if (patternBoard_->removeDeadGroup((move.x) +1, move.y, enemy) == true) {
        hasKilled = true; 
      }
      if (patternBoard_->removeDeadGroup(move.x, (move.y) +1, enemy) == true) {
        hasKilled = true; 
      }
      if ((true == isDead) && (false == hasKilled)) { // Illegal move
        // Depending on rule set ( i.e. Ing rules ) we will only use Japanese rules, so no sacrificing stones allowed
        if (false == RULESALL) { 
          patternBoard_->setStone((move.x), (move.y), EMPTY);
          cout << "illegal move!!!!!! doPatternBoard" << endl;
        }
      }
      // qDebug("doPatternBoard done");
      return true;
    }
  }
  return false;
}



void Game::initializePatternBoard() 
{
  try {
    patternBoard_ = new Board;
  }
  catch (bad_alloc&) { 
    cout << "Error allocating memory." << endl; 
    abort();
  }
}



void Game::clearPatternBoard() 
{
  patternBoard_->clear();
}



void Game::uninitializeGameMoves() 
{
  if (gameMoves.empty() == false) { 
    gameMoves.clear(); 
  }
}



//----------------------------------------------------------------------
// Create a game record in a faster traversal format.
// Store only points where stones are placed or removed.
// So that no Go Logic is needed to replay a game or when pattern searched.
// This is useful for a freeSearch such as Kombilo search.
// We do it this way as it may be useful at a later date to implement a kombilo search or other search.
//----------------------------------------------------------------------
void Game::generateGameConversions(ostream& ofs, u32& seekPos) 
{ 
  try {
    patternBoard_ = new Board;
    patternBoard_->clear();
    u8 colour;
    u16 captureCount;
    FullMove move;
    // For each move get stone placed and list of empty stones (captures)
    // output the movecount first - then all the stones played + captureMoves.

// Try opening file each time before we write it instead of passing ofs and keeping it open
// Not sure if this was causing the HEADACHE bug of first alpha release
// We could not pin down the bug, seemed to be memory allocation - but was sporadic

    for (u16 i = 0; i < moves.size(); i++) {
      doPatternBoard(i); // Do the current move to get any captureMoves
      captureCount = patternBoard_->getCaptureCount();

      //move = getMove(i);
      move = moves.at(i);
      colour = move.colour;
//       cout << "move no=" << i << " ";
//       cout << "move=" << (int)move.x << ":" <<(int)move.y;
//       cout << " colour=" << (int)colour+'0' << endl;
      colour <<= 5; // Bitshift colour 
      if (captureCount > 0) { //set highest bit true;
        colour |= 0x80; 
      } 
      move.x |= colour;
// we manipulate bits to save disk/memory space
// Highest bit is CAPTURES TRUE/FALSE
// Then colour of move
// Xor this into the x.move

      
      //Check that the FileStream is still open.
      
      ofs.write(reinterpret_cast<char *> (&move.x), sizeof (move.x));
      ofs.write(reinterpret_cast<char *> (&move.y), sizeof (move.y));
      seekPos += (sizeof(u8)) * 2; // Increase the filestream position
      
//   qDebug("before capture count");
//write out the number of captureMoved stones next
//for each captureMoved stone send it to the file

      if (captureCount > 0) { 
        ofs.write(reinterpret_cast<char *> (&captureCount), sizeof (captureCount));
        seekPos += sizeof (captureCount);
        for (u16 j = 0; j < captureCount; j++) {
          ofs.write( reinterpret_cast<char *> (&patternBoard_->getCaptureStone(j)->x), 
                    sizeof (u8));
          ofs.write( reinterpret_cast<char *> (&patternBoard_->getCaptureStone(j)->y), 
                    sizeof (u8));
          seekPos += sizeof(u8) * 2;
        }
      } // if
    } // for
    delete patternBoard_;
  } //try


//close the file

  catch ( ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl; 
    abort();
  }
  catch (...) {
    qDebug("Weve caught an exception board* new here");
  }

  if (moves.empty() == false) {
    moves.clear();
  }
//     qDebug("generateGameConversions called end");
}



//------------------------------------------------------------------------
//Store moves in a vector List
//the list contains all the stones played on that move , inc captures
//the vector contains lists of stones
//------------------------------------------------------------------------
void Game::initializeGameMoves(istream& ifs) 
{
  
// Mybe open the file here better than passing ifs
  if (gameMoves.empty() == false) {
    gameMoves.clear(); 
  }
  u8 encode;
  u8 colour;
  u16 captureCount;
  FullMove move;
//   cout << "initialize gamemoves game=" << index_ << " " << filename_ << endl;
  for (u16 i = 0; i < moveCount_; i++) {
    boost::shared_ptr<vector <boost::shared_ptr<FullMove> > > 
      vectorMove (new (vector <boost::shared_ptr<FullMove> > ));
    
    boost::shared_ptr<FullMove> fullMove (new FullMove);
    
  
    ifs.read(reinterpret_cast<char *> (&move.x), sizeof(move.x));
    ifs.read(reinterpret_cast<char *> (&move.y), sizeof(move.y));
    // cout << "x=" << (int)move.x << " y=" << (int)move.y << endl;
    encode = move.x;

// Decode move.x as we use bit shifting technique to save space
// get the move.x part of it
// then get the colour of the move
// And find out of there are captures to do as well

    move.x &= 0x1F; 
    colour = encode;
    colour >>= 5;
    colour &= 0x03; //keep only colour 2 bit part
    fullMove->x = move.x;
    fullMove->y = move.y;
    fullMove->colour = colour;
    vectorMove->push_back(fullMove);

//get number of captured stones to set to empty spaces on the board

    encode &= 0x80;
    if (encode > 0) { 
      ifs.read(reinterpret_cast<char *> (&captureCount), sizeof (captureCount));
      for (u16 j = 0; j < captureCount; j++) {
        boost::shared_ptr<FullMove> capture (new FullMove);
        ifs.read(reinterpret_cast<char *> (&move.x), sizeof (move.x));
        ifs.read(reinterpret_cast<char *> (&move.y), sizeof (move.y));
        capture->x = move.x;
        capture->y = move.y;
        capture->colour = 0;
        vectorMove->push_back( capture );
      }
    }
  gameMoves.push_back(vectorMove);
  }
}



// Used by the GUI - This generates a vector of boards to display the preview board
// User can navigate between moves.
// Add boards for how many moves, plus one for the empty board at start
void Game::setupBoards()
{
  boards_.empty();
  // For each move add to all boards past current one  
  for (int i = 0; i <= getMoveCount(); i++) { 
    boost::shared_ptr<Board> board (new Board);
    boards_.push_back(board);
  }
  FullMove move;
  FullMove capture;
  u16 captures;
  
// For each board after current move - set stone 

  for (u16 iMove = 0; iMove < getMoveCount(); iMove++) {
    move = getGameMove(iMove);    
    for (u16 i = iMove; i < getMoveCount(); i++) {
      boards_[i + 1]->setStone(move.x, move.y, move.colour);
    }
    
// For each board after current move - set stone to EMPTY

    captures = iCaptures(iMove);
    if (captures > 0) {
      for (u16 i = 0; i < captures; i++) { // For each capture
        capture = getGameCapture ( iMove , i );
        for (u16 j = iMove; j < getMoveCount(); j++) {
          boards_[j + 1]->setStone(capture.x, capture.y, EMPTY);
        }
      }
    }
  }
}



//-----------------------------------------------------------------------
//Return the number of captures for this turn (moveNumber)
//------------------------------------------------------------------------
u16 Game::iCaptures(const u16 moveNumber) const
{
  u16 captures = 0;
  if (gameMoves.at(moveNumber)->empty() == false) {
    captures = gameMoves[moveNumber]->size() - 1;
  }
  return captures;
}



//--------------------------------------------------------------------
// Return the capture number iCapture for this turn moveNumber.
// First move is always a proper move, i.e. non capture.
//---------------------------------------------------------------------
FullMove Game::getGameCapture (const u16 moveNumber, const u16 iCapture) const
{
  return *(gameMoves.at(moveNumber)->at(iCapture+1));
}



//------------------------------------------------------------------------
//Get the move played on this turn
//------------------------------------------------------------------------
FullMove Game::getGameMove(const u16 moveNumber) const
{
  return *(gameMoves.at(moveNumber)->front());
}




//------------------------------------------------
//! Serialization of Game class
//------------------------------------------------
void Game::writeGameIndex(ostream& ofs) 
{
  try {
    ofs.write(reinterpret_cast<char *>(&index_), sizeof (index_));
    ofs.write(reinterpret_cast<char *>(&dirkey_), sizeof (dirkey_));
    ofs.write(reinterpret_cast<char *>(&moveCount_), sizeof (moveCount_));
    ofs.write(reinterpret_cast<char *>(&result_), sizeof (result_));
    write(ofs, filename_);
    write(ofs, rankW_);
    write(ofs, rankB_);
    write(ofs, resultString_);
    write(ofs, komi_);
    write(ofs, date_);
    write(ofs, playerW_);
    write(ofs, playerB_);
    write(ofs, handicap_);
    write(ofs, place_);
    write(ofs, event_);
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
}



// string::size_type better but undefined length
void Game::write(ostream& ofs, const string& str) const
{
  u16 size = str.size(); 
  ofs.write(reinterpret_cast<char *>(&size), sizeof(size));
  ofs.write(str.c_str(), size+1);
}



// Initialize a Game from file - upon startup
void Game::readGameIndex(istream& ifs) 
{
  try {
    ifs.read( reinterpret_cast<char *>(&index_), sizeof (index_));
    ifs.read( reinterpret_cast<char *>(&dirkey_ ), sizeof (dirkey_));
    ifs.read( reinterpret_cast<char *>(&moveCount_), sizeof (moveCount_));
    ifs.read( reinterpret_cast<char *>(&result_ ), sizeof (result_));
    read(ifs, filename_);
    read(ifs, rankW_);
    read(ifs, rankB_);
    read(ifs, resultString_);
    read(ifs, komi_);
    read(ifs, date_);
    read(ifs, playerW_);
    read(ifs, playerB_);
    read(ifs, handicap_);
    read(ifs, place_);
    read(ifs, event_);
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
}



// Read in a string - get its length etc.
void Game::read(istream& ifs, string& str)
{
  u16 len = 0;
  ifs.read( reinterpret_cast<char *>(&len), sizeof(len));
  char *tmp = new char [len+1];
  ifs.read(tmp, len+1);
  str = tmp; 
  delete[] tmp;
}

