#ifndef GAME_H
#define GAME_H

#include <vector>

#include "Const.h"
#include "Board.h"
#include "Sgf.h"

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\scoped_ptr.hpp>
#else
#include "boost/shared_ptr.hpp"
#include "boost/scoped_ptr.hpp"
#endif




class Game {
  public:
    Game();
    Game(u32 idx, u32 key, string name);
    
    FullMove getMove(u16 moveNumber) const;
    void clearBoards();
    void setupBoards();
    void initializeGameInfo(const string& dir);
    bool initializeGame(const string& dir);
    void initializePatternBoard();
    void clearPatternBoard();
    
    void initializeGameMoves(istream& ifs);
    void uninitializeGameMoves();
    bool doPatternBoard(const u16 moveNumber);
    
    u32 getIndex() const;
    u32 getDirkey() const;
    u16 getMoveCount() const;
    u8 getResult() const;
    u64 getFnvSignature() const;
    string getFilename() const;
    string getPlayerW() const;
    string getPlayerB() const;
    string getRankW() const;
    string getRankB() const;
    string getResultStr() const;
    string getKomi() const;
    string getDate() const;
    string getHandicap() const;
    string getPlace() const;
    string getEvent() const;
    
    string getApplication() const;
    string getTime() const;
    string getOverTime() const;
    string getRound() const;
    string getCopyright() const;
    string getRules() const;
    string getGameName() const;
    string getGameComment() const;
    string getWhiteTeam() const;
    string getBlackTeam() const;
    string getSource() const;
    string getAnnotator() const;
    string getTranscriber() const;

    void setIndex(const u32 idx);
    void setDirkey(const u32 key);
    void setMoveCount(const u16 value);
    void setFilename(const string& value);
    void setPlayerW(const string& value);
    void setPlayerB(const string& value);
    void setRankW(const string& value);
    void setRankB(const string& value);
    void setResultStr(const string& value);
    void setKomi(const string& value);
    void setDate(const string& value);
    void setResult(const u8 value);
    void setHandicap(const string& value);
    void setPlace(const string& value);
    void setEvent(const string& value);

    void readGameIndex(istream& ifs);
    void writeGameIndex(ostream& ofs);

    bool showGame(const string& dir);
    bool showMenu(const string& dir);
    void printBoard();
    void printMembers() const;
    void printMoves();
    void printAllBoard();
    void displayGame();
    void generateGameConversions(ostream& ofs, u32& seekPos);
    u16 iCaptures(const u16 moveNumber) const;
    FullMove getGameMove(const u16 moveNumber) const;
    FullMove getGameCapture (const u16 moveNumber, 
                             const u16 iCapture) const;
    
    vector <boost::shared_ptr<Board> > boards_;
    
    ~Game();
  private:
    
    
    void write(ostream& ofs, const string& str) const;
    void read(istream& ifs, string& str);
    vector <FullMove> moves; //to store sgf moves once extracted from file
    //A vector of pointers to a list of moves & captures for each turn
    vector <boost::shared_ptr<vector <boost::shared_ptr<FullMove> > > > gameMoves;
    
    Board* patternBoard_;
//     Board* patternBoard_; ////used for pattern generation
    u32 seekPos_; //For gameIndex offsets
    u32 index_;
    u32 dirkey_;
    u16 moveCount_;
    u8 result_; 
    u64 fnvSignature_; //duplicate check - signature of game
    string filename_;
    string rankW_;
    string rankB_;
    string resultString_;
    string komi_;
    string date_; 
    string playerW_;
    string playerB_;
    string handicap_;
    string place_;
    string event_;
    string application_;
    string time_;
    string overTime_;
    string round_;
    string copyright_;
    string rules_;
    string gameName_;
    string gameComment_;
    string whiteTeam_;
    string blackTeam_;
    string source_;
    string annotator_;
    string transcriber_;
    
};

#endif
