

#include <QApplication>
#include <QSortFilterProxyModel>
#include <QHeaderView>
#include <QVBoxLayout>
#include <QEvent>
#include <QSettings>
#include <QContextMenuEvent>
#include <QModelIndex>
#include <QString>
#include <QScrollBar>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\lexical_cast.hpp>
#else
#include "boost/lexical_cast.hpp"
#endif

#include "GamesTab.h"
#include "GameInfo.h"
#include "Const.h"

using boost::lexical_cast;


MySortFilterProxyModel::MySortFilterProxyModel(GamesTab *parent)
  : QSortFilterProxyModel(parent)
{
  parent_ = parent;
}



void MySortFilterProxyModel::setPlayerWhiteFilter( const QString& filter )
{
  filterPlayerWhite = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
//   std::cout << "emit data changed player white" << std::endl;
}



void MySortFilterProxyModel::setPlayerBlackFilter( const QString& filter )
{
  filterPlayerBlack = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
}



void MySortFilterProxyModel::setRankWhiteFilter( const QString& filter )
{
  filterRankWhite = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
}


void MySortFilterProxyModel::setRankBlackFilter( const QString& filter )
{
  filterRankBlack = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
}



void MySortFilterProxyModel::setResultFilter( const QString& filter )
{
  filterResult = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
}


void MySortFilterProxyModel::setPlaceFilter( const QString& filter )
{
  filterPlace = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
}


void MySortFilterProxyModel::setEventFilter( const QString& filter )
{
  filterEvent = filter;
  QModelIndex topLeft( index( 0, 0 ) );
  QModelIndex bottomRight( index( rowCount() - 1, 10 ) );
  emit dataChanged( topLeft, bottomRight );
}



/*
bool MySortFilterProxyModel::filterAcceptsRow(int sourceRow,
            const QModelIndex &sourceParent) const
{
  QModelIndex index1 = sourceModel()->index(sourceRow, parent_->headerPosPlayerWhite_,
    sourceParent);
  QModelIndex index2 = sourceModel()->index(sourceRow, parent_->headerPosRankWhite_,
    sourceParent);
  QModelIndex index3 = sourceModel()->index(sourceRow, parent_->headerPosPlayerBlack_,
    sourceParent);
  QModelIndex index4 = sourceModel()->index(sourceRow, parent_->headerPosRankBlack_,
    sourceParent);
  QModelIndex index5 = sourceModel()->index(sourceRow, parent_->headerPosResult_, sourceParent);
  QModelIndex index6 = sourceModel()->index(sourceRow, parent_->headerPosPlace_, sourceParent);
  QModelIndex index7 = sourceModel()->index(sourceRow, parent_->headerPosEvent_, sourceParent);
  
  
  //Case insensitive, so extract data and filter
  return (sourceModel()->data(index1).toString().contains(filterPlayerWhite)
    && sourceModel()->data(index2).toString().contains(filterRankWhite)
    && sourceModel()->data(index3).toString().contains(filterPlayerBlack)
    && sourceModel()->data(index4).toString().contains(filterRankBlack)
    && sourceModel()->data(index5).toString().contains(filterResult)
    && sourceModel()->data(index6).toString().contains(filterPlace)
    && sourceModel()->data(index7).toString().contains(filterEvent));
  
  return true;
}
*/




/*****************************************************************************************
  RESULTS MODEL
******************************************************************************************/



ResultsModel::ResultsModel(QObject *parent) : QAbstractItemModel(parent)
{
  hasMainBoard = false;
}


void ResultsModel::setResults(MainWindow* mainWindow)
{
//   qDebug("setResults");
  mainWindow_ = mainWindow;
  hasMainBoard = true;
}



int ResultsModel::rowCount(const QModelIndex &parent) const
{
//   qDebug("Resultsmodel rowCount");
  
  if (true == hasMainBoard) {
//     qDebug("Resultsmodel rowCount board empty = false");
    return mainWindow_->selectedBoardWindow_->mainBoard->filteredResults.size();
  }
  else {
//     qDebug("Resultsmodel rowCount board empty = true");
    return 0;
  }
}



int ResultsModel::columnCount(const QModelIndex &parent) const
{
//   qDebug("Resultsmodel colCount");
  if (true == hasMainBoard) {
    return 13;
  }
  else {
    return 0;
  }
}




QVariant ResultsModel::data(const QModelIndex &index, int role) const
{
//    qDebug("ReusltsMOdel data");
  QVariant data;
  
  if (false == hasMainBoard) {
    return QVariant();
  }
  
  if (!index.isValid()) {
//     qDebug("data index is !valid");
    return QVariant();
  }

  if (true == mainWindow_->boardWindows_.empty()) {
//     qDebug("data boardswindow empty");
    return QVariant();
  }
  
  if (index.row() >= mainWindow_->selectedBoardWindow_->mainBoard->filteredResults.size()) {
//     qDebug("data index.row >=");
//     std::cout << "data row = " << index.row() << std::endl;
    return QVariant();
  }
  
  if (index.column() >= 13) {
//     qDebug("data index.col >=13");
    return QVariant();
  }
  
  if (role == Qt::DisplayRole) {
    boost::shared_ptr<MainBoard> mainBoard = mainWindow_->selectedBoardWindow_->mainBoard;

    // On windows if no items in model then header is not displayed
    // If there are no results then we want to display the header anyway
    //if (mainBoard->filteredResults.empty() == true) {
    //  return data;
   // }

    int db = mainBoard->filteredResults[index.row()]->dbIndex;
    int game = mainBoard->filteredResults[index.row()]->gameIndex;
    boost::shared_ptr<Game> gamePtr = mainWindow_->dbManager_->databases[db]->database->Games[game];
  
    switch (index.column()) {
      case 0 : {
        data = (mainWindow_->dbManager_->databases[db]->name).c_str();
        return data;
      }
      case 1 : {
        data = (gamePtr->getPlayerW()).c_str();
        return data;
      }
      case 2 : {
        data = (gamePtr->getRankW()).c_str();
        return data;
      }
      case 3 : {
        data = (gamePtr->getPlayerB()).c_str();
        return data;
      }
      case 4 : {
        data = (gamePtr->getRankB()).c_str();
        return data;
      }
      case 5 : {
        data = (gamePtr->getResultStr()).c_str();
        return data;
      }
      case 6 : {
        data = (gamePtr->getDate()).c_str();
        return data;
      }
      case 7 : {
        data = gamePtr->getMoveCount();
        return data;
      }
      case 8 : {
        data = (gamePtr->getKomi()).c_str();
        return data;
      }
      case 9 : {
        data = (gamePtr->getHandicap()).c_str();
        return data;
      }
      case 10 : {
        data = (gamePtr->getPlace()).c_str();
        return data;
      }
      case 11 : {
        data = (gamePtr->getEvent()).c_str();
        return data;
      }
      case 12 : {
//         qDebug("column data 12 ");
        int num = gamePtr->getIndex();
        data = num;
        return data;
      }
      default : {
        return QVariant();
      }
    }
  }
//   else
  return QVariant();
}



Qt::ItemFlags ResultsModel::flags(const QModelIndex &index) const
{
  if (!index.isValid())
    return Qt::ItemIsEnabled;

  return QAbstractItemModel::flags(index) | Qt::ItemIsSelectable;
}



QModelIndex ResultsModel::index(int row, int column, const QModelIndex & parent) const
{
//   qDebug("index");
  if (false == hasMainBoard) {
    return QModelIndex();
  }
//   if (!parent.isValid()) {
//     return QModelIndex();
//   }
  if (true == mainWindow_->boardWindows_.empty()) {
//     qDebug("data boardswindow empty");
    return QModelIndex();
  }
  if (row >= mainWindow_->selectedBoardWindow_->mainBoard->filteredResults.size()) {
//     std::cout << " row = " << row << std::endl;
    return QModelIndex();
  }
  if (column >= 13) {
//     qDebug("data index.col >=13");
    return QModelIndex();
  }
  return createIndex ( row, column, 0 );
}



// No hierachy in this model simply a flat 2d model
QModelIndex ResultsModel::parent (const QModelIndex & index) const
{
//    qDebug("parent");
  return QModelIndex();
}



QVariant ResultsModel::headerData(int section, Qt::Orientation orientation, int role) const
{
  if (role != Qt::DisplayRole) {
    return QVariant();
  }
  if (orientation == Qt::Horizontal) {
//     std::cout << "header section = " << section << std::endl;
//      return QString("Column %1").arg(section);
    
    switch (section) {
      case 0 : {
        return QString("Database");
      }
      case 1 : {
        return QString("White");
      }
      case 2 : {
        return QString("W Rank");
      }
      case 3 : {
        return QString("Black");
      }
      case 4 : {
        return QString("B Rank");
      }
      case 5 : {
        return QString("Result");
      }
      case 6 : {
        return QString("Date");
      }
      case 7 : {
        return QString("Moves");
      }
      case 8 : {
        return QString("Komi");
      }
      case 9 : {
        return QString("Handicap");
      }
      case 10 : {
        return QString("Place");
      }
      case 11 : {
        return QString("Event");
      }
      case 12 : {
        return QString("GameIndex");
      }
      default : {
        return QString();
      }
    
    }
    
  }
  else {
    return QString("Row %1").arg(section);
  }
}




/*****************************************************************************************
  GAMES TAB
******************************************************************************************/
GamesTab::GamesTab(DbManager* db, DatabaseViewer* databaseViewer, QWidget *parent) : QWidget(parent)
{
  dbManager_ = db;
  databaseViewer_ = databaseViewer;
  
  moveNumber_ = 0; // current search results for movenumber.
  setObjectName("GamesWindow");
  setWindowTitle("Games List");
  treeView = new QTreeView;
//   model = new QStandardItemModel(0,13);
//   qDebug("gamesTab new model");
  model = new ResultsModel();
//   qDebug("after gamesTab new model");
  treeView->setSelectionBehavior(QAbstractItemView::SelectRows);
  treeView->setSelectionMode(QAbstractItemView::SingleSelection);
  treeView->setEditTriggers(QAbstractItemView::NoEditTriggers);
  treeView->setRootIsDecorated(false);
  treeView->setAlternatingRowColors (true);
  treeView->setItemsExpandable (false);
  
  myProxyModel = new MySortFilterProxyModel(this);
  myProxyModel->setSourceModel(model);
  
  treeView->setModel(myProxyModel);
//   qDebug("before gamesTab setModel");
  //treeView->setModel(model);
//   qDebug("gamesTab setModel");
  treeView->header()->setClickable(false);
  treeView->header()->setSortIndicatorShown(false);
//    treeView->header()->setResizeMode(QHeaderView::Stretch);
   
  
  treeView->setDragEnabled(false);
  treeView->setDropIndicatorShown(false);
  treeView->setAcceptDrops(false);
  treeView->setUpdatesEnabled(true);
  treeView->setMouseTracking (true);
  setMouseTracking (true);
  
  
  QVBoxLayout *layout = new QVBoxLayout;
  layout->addWidget(treeView);
  setLayout(layout);

  gameMenu = new QMenu("Game menu", this);
  gameMenu->setTitle("Game menu");
  openGameAct_ = new QAction(tr("&Open on new board"), this);
  
  removeGameAct_ = new QAction(QIcon(":/resources/icons/editdelete.png"), tr("&Remove from Database"), this);
  removeGameAct_->setEnabled (false);
  
  updateListAct_ = new QAction(QIcon(":/resources/icons/reload.png"), tr("&Update List"), this);
  
  autoUpdateAct_ = new QAction(QIcon(":/resources/icons/connect_established.png"), tr("&Auto Update List"), this);
  autoUpdateAct_->setCheckable(true);
  autoUpdateAct_->setChecked(true);
  connect(autoUpdateAct_, SIGNAL(triggered(bool)), this, SLOT(autoUpdateClick(bool)));
  
  gameMenu->addAction (openGameAct_);
  gameMenu->addSeparator();
  gameMenu->addAction (removeGameAct_);
  gameMenu->addSeparator();
  gameMenu->addAction (autoUpdateAct_);
  gameMenu->addAction (updateListAct_);
  
  selectionModel_ = treeView->selectionModel();
  
  
  connect(openGameAct_, SIGNAL(triggered()), this, SLOT(openGame()));
//   connect(removeGameAct_, SIGNAL(triggered()), this, SLOT(removeGame()));
  connect(updateListAct_, SIGNAL(triggered()), this, SLOT(updateList()));
  connect(treeView, SIGNAL(clicked( QModelIndex )), this, SLOT(setupPreviewAndInfo()));
  connect(selectionModel_, SIGNAL( currentChanged ( const QModelIndex &, const QModelIndex &)),
           this, SLOT(setupPreviewAndInfo()));
//    connect(treeView, SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)),
//            this, SLOT(setupPreviewAndInfo()));
  
  setupModel();
  
}



void GamesTab::autoUpdateClick(bool on)
{
//   std::cout << "games tab auto update " << (bool)on << std::endl;
  autoUpdateAct_->setChecked(on);
  gameMenu->removeAction(autoUpdateAct_);
  if (true == on) {
    autoUpdateAct_ = new QAction(QIcon(":/resources/icons/connect_established.png"), tr("&Auto Update List"), this);
    autoUpdateAct_->setCheckable(true);
    autoUpdateAct_->setChecked(true);
  }
  else {
    autoUpdateAct_ = new QAction(QIcon(":/resources/icons/connect_no.png"), tr("&Auto Update List"), this);
    autoUpdateAct_->setCheckable(true);
    autoUpdateAct_->setChecked(false);
  }
  gameMenu->insertAction(updateListAct_, autoUpdateAct_);
  connect(autoUpdateAct_, SIGNAL(triggered(bool)), this, SLOT(autoUpdateClick(bool)));
}




void GamesTab::doMatches()
{
  boost::shared_ptr<MainBoard> mainBoard = 
      databaseViewer_->mainWindow_->selectedBoardWindow_->mainBoard;
  QString number = (lexical_cast<std::string>(mainBoard->filteredResults.size())).c_str();
  if (mainBoard->filteredResults.size() == 1) {
    databaseViewer_->mainWindow_->databaseActiveGames_->label_->setText(" " + number + " Game ");
  }
  else {
    databaseViewer_->mainWindow_->databaseActiveGames_->label_->setText(" " + number + " Games ");
  }
  if (true == autoUpdateAct_->isChecked()) {
    showMatches();
  }
}



void GamesTab::showMatches()
{
  boost::shared_ptr<MainBoard> mainBoard = 
      databaseViewer_->mainWindow_->selectedBoardWindow_->mainBoard;
  model->setResults(databaseViewer_->mainWindow_);
  treeView->setModel(model);
  QString number = (lexical_cast<std::string>(mainBoard->filteredResults.size())).c_str();
  if (mainBoard->filteredResults.size() == 1) {
    databaseViewer_->setWindowTitle(number + " Game matched");
  }
  else {
    databaseViewer_->setWindowTitle(number + " Games matched");
  }

  QSettings settings;
  settings.beginGroup("GamesTab");
  int size = settings.value("header_Database_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosDatabase_, size);
  size = settings.value("header_PlayerWhite_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosPlayerWhite_, size);
  size = settings.value("header_RankWhite_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosRankWhite_, size);
  size = settings.value("header_PlayerBlack_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosPlayerBlack_, size);
  size = settings.value("header_RankBlack_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosRankBlack_, size);
  size = settings.value("header_Result_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosResult_, size);
  size = settings.value("header_Date_Size", 70).toInt();
  treeView->header()->resizeSection(headerPosDate_, size);
  size = settings.value("header_MoveCount_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosMoveCount_, size);
  size = settings.value("header_Komi_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosKomi_, size);
  size = settings.value("header_Handicap_Size", 60).toInt();
  treeView->header()->resizeSection(headerPosHandicap_, size);
  size = settings.value("header_Place_Size", 100).toInt();
  treeView->header()->resizeSection(headerPosPlace_, size);
  size = settings.value("header_Event_Size", 100).toInt();
  treeView->header()->resizeSection(headerPosEvent_, size);
  size = settings.value("header_GameIndex_Size", 100).toInt();
  settings.endGroup();

//   treeView->header()->resizeSection(headerPosGameIndex_, size);
  //treeView->header()->setSectionHidden(headerPosGameIndex_, true);
  treeView->header()->setStretchLastSection (false);
  
  selectionModel_ = treeView->selectionModel();
  connect(selectionModel_, SIGNAL( currentChanged ( const QModelIndex &, 
          const QModelIndex &)), this, SLOT(setupPreviewAndInfo()));
}



void GamesTab::setupModel()
{
//   model->clear();
  
  QSettings settings;
  settings.beginGroup("GamesTab");
  headerPosDatabase_ = settings.value("header_Database_Pos", 0).toInt();
  headerPosPlayerWhite_ = settings.value("header_PlayerWhite_Pos", 1).toInt();
  headerPosRankWhite_ = settings.value("header_RankWhite_Pos", 2).toInt();
  headerPosPlayerBlack_ = settings.value("header_PlayerBlack_Pos", 3).toInt();
  headerPosRankBlack_ = settings.value("header_RankBlack_Pos", 4).toInt();
  headerPosResult_ = settings.value("header_Result_Pos", 5).toInt();
  headerPosDate_ = settings.value("header_Date_Pos", 6).toInt();
  headerPosMoveCount_ = settings.value("header_MoveCount_Pos", 7).toInt();
  headerPosKomi_ = settings.value("header_Komi_Pos", 8).toInt();
  headerPosHandicap_ = settings.value("header_Handicap_Pos", 9).toInt();
  headerPosPlace_ = settings.value("header_Place_Pos", 10).toInt();
  headerPosEvent_ = settings.value("header_Event_Pos", 11).toInt();
  headerPosGameIndex_ = settings.value("header_GameIndex_Pos", 12).toInt();
  
 

 
  model->setHeaderData ( headerPosDatabase_, Qt::Horizontal, "Database", Qt::DisplayRole );
  model->setHeaderData ( headerPosPlayerWhite_, Qt::Horizontal, "White", Qt::DisplayRole );
  model->setHeaderData ( headerPosRankWhite_, Qt::Horizontal, "W Rank", Qt::EditRole );
  model->setHeaderData ( headerPosPlayerBlack_, Qt::Horizontal, "Black", Qt::EditRole );
  model->setHeaderData ( headerPosRankBlack_, Qt::Horizontal, "B Rank", Qt::EditRole );
  model->setHeaderData ( headerPosResult_, Qt::Horizontal, "Result", Qt::EditRole );
  model->setHeaderData ( headerPosDate_, Qt::Horizontal, "Date", Qt::EditRole );
  model->setHeaderData ( headerPosMoveCount_, Qt::Horizontal, "Moves", Qt::EditRole );
  model->setHeaderData ( headerPosKomi_, Qt::Horizontal, "Komi", Qt::EditRole );
  model->setHeaderData ( headerPosHandicap_, Qt::Horizontal, "Handicap", Qt::EditRole );
  model->setHeaderData ( headerPosPlace_, Qt::Horizontal, "Place", Qt::EditRole );
  model->setHeaderData ( headerPosEvent_, Qt::Horizontal, "Event", Qt::EditRole );
  model->setHeaderData ( headerPosGameIndex_, Qt::Horizontal, "GameIndex", Qt::EditRole ); 
  

  int size = settings.value("header_Database_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosDatabase_, size);
  size = settings.value("header_PlayerWhite_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosPlayerWhite_, size);
  size = settings.value("header_RankWhite_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosRankWhite_, size);
  size = settings.value("header_PlayerBlack_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosPlayerBlack_, size);
  size = settings.value("header_RankBlack_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosRankBlack_, size);
  size = settings.value("header_Result_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosResult_, size);
  size = settings.value("header_Date_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosDate_, size);
  size = settings.value("header_MoveCount_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosMoveCount_, size);
  size = settings.value("header_Komi_Size", 50).toInt();
  treeView->header()->resizeSection(headerPosKomi_, size);
  size = settings.value("header_Handicap_Size", 80).toInt();
  treeView->header()->resizeSection(headerPosHandicap_, size);
  size = settings.value("header_Place_Size", 100).toInt();
  treeView->header()->resizeSection(headerPosPlace_, size);
  size = settings.value("header_Event_Size", 100).toInt();
  treeView->header()->resizeSection(headerPosEvent_, size);
  size = settings.value("header_GameIndex_Size", 0).toInt();
  treeView->header()->resizeSection(headerPosGameIndex_, size);
//  treeView->header()->setSectionHidden(headerPosGameIndex_, true);
  treeView->header()->setStretchLastSection (false);
  settings.endGroup();
  databaseViewer_->setWindowTitle("Search results not Listed");
}




// On Qt 4.1.4 Windows , the TreeViews show no the Headers if there is no items in the model
// This leads to saving of settings that are not correct.
// And when the settings get loaded again they are wrong causing problems.
// So make sure that only valid settings are saved.
void GamesTab::writeSettings()
{
  QSettings settings;
  settings.beginGroup("GamesTab");
  int size;
  size = treeView->header()->sectionSize(headerPosDatabase_);
  if (size == 0) { size = 80; }
  settings.setValue("header_Database_Size", size);

  size = treeView->header()->sectionSize (headerPosPlayerWhite_);
  if (size == 0) { size = 80; }
  settings.setValue("header_PlayerWhite_Size", size);

  size = treeView->header()->sectionSize (headerPosRankWhite_);
  if (size == 0) { size = 50; }
  settings.setValue("header_RankWhite_Size", size);

  size = treeView->header()->sectionSize (headerPosPlayerBlack_);
  if (size == 0) { size = 80; }
  settings.setValue("header_PlayerBlack_Size", size);

  size = treeView->header()->sectionSize (headerPosRankBlack_);
  if (size == 0) { size = 50; }
  settings.setValue("header_RankBlack_Size", size);

  size = treeView->header()->sectionSize (headerPosResult_);
  if (size == 0) { size = 50; }
  settings.setValue("header_Result_Size", size);

  size = treeView->header()->sectionSize (headerPosDate_);
  if (size == 0) { size = 80; }
  settings.setValue("header_Date_Size", size);

  size = treeView->header()->sectionSize (headerPosMoveCount_);
  if (size == 0) { size = 50; }
  settings.setValue("header_MoveCount_Size", size);

  size = treeView->header()->sectionSize (headerPosKomi_);
  if (size == 0) { size = 50; }
  settings.setValue("header_Komi_Size", size);

  size = treeView->header()->sectionSize (headerPosHandicap_);
  if (size == 0) { size = 80; }
  settings.setValue("header_Handicap_Size", size);

  size = treeView->header()->sectionSize (headerPosPlace_);
  if (size == 0) { size = 100; }
  settings.setValue("header_Place_Size", size);

  size = treeView->header()->sectionSize (headerPosEvent_);
  if (size == 0) { size = 100; }
  settings.setValue("header_Event_Size", size);

  size = treeView->header()->sectionSize (headerPosGameIndex_);
  if (size == 0) { size = 0; }
  settings.setValue("header_GameIndex_Size", size);


// Positions of Headers

  size = treeView->header()->visualIndex (headerPosDatabase_);
  if (size == -1) { size = 0; }
  settings.setValue("header_Database_Pos", size);

  size = treeView->header()->visualIndex (headerPosPlayerWhite_);
  if (size == -1) { size = 1; }
  settings.setValue("header_PlayerWhite_Pos", size);

  size = treeView->header()->visualIndex (headerPosRankWhite_);
  if (size == -1) { size = 2; }
  settings.setValue("header_RankWhite_Pos", size);

  size = treeView->header()->visualIndex (headerPosPlayerBlack_);
  if (size == -1) { size = 3; }
  settings.setValue("header_PlayerBlack_Pos", size);

  size = treeView->header()->visualIndex (headerPosRankBlack_);
  if (size == -1) { size = 4; }
  settings.setValue("header_RankBlack_Pos", size);

  size = treeView->header()->visualIndex (headerPosResult_);
  if (size == -1) { size = 5; }
  settings.setValue("header_Result_Pos", size);

  size = treeView->header()->visualIndex (headerPosDate_);
  if (size == -1) { size = 6; }
  settings.setValue("header_Date_Pos", size);

  size = treeView->header()->visualIndex (headerPosMoveCount_);
  if (size == -1) { size = 7; }
  settings.setValue("header_MoveCount_Pos", size);

  size = treeView->header()->visualIndex (headerPosKomi_);
  if (size == -1) { size = 8; }
  settings.setValue("header_Komi_Pos", size);

  size = treeView->header()->visualIndex (headerPosHandicap_);
  if (size == -1) { size = 9; }
  settings.setValue("header_Handicap_Pos", size);

  size = treeView->header()->visualIndex (headerPosPlace_);
  if (size == -1) { size = 10; }
  settings.setValue("header_Place_Pos", size);

  size = treeView->header()->visualIndex (headerPosEvent_);
  if (size == -1) { size = 11; }
  settings.setValue("header_Event_Pos", size);

  size = treeView->header()->visualIndex (headerPosGameIndex_);
  if (size == -1) { size = 12; }
  settings.setValue("header_GameIndex_Pos", size);

  settings.endGroup();
}



// For popup Menu
void GamesTab::contextMenuEvent(QContextMenuEvent *event)
{
  event->accept();
//   QModelIndex index = treeView->currentIndex();
//   if (index.row() >= 0) { //Valid item row selected
    gameMenu->exec(event->globalPos());
//   }
}



void GamesTab::setCurrentMoveNumber(int moveNumber)
{
  moveNumber_ = moveNumber;
}



void GamesTab::setupPreviewAndInfo()
{
//   qDebug("setupPreview&info");
  // QModelIndex index = myProxyModel->mapToSource(treeView->currentIndex());
  qApp->processEvents();
  QModelIndex index = treeView->currentIndex();
//   std::cout << "row = " << index.row() << " col = " << index.column() << std::endl;
  if (index.row() >= 0) { //Valid item row selected
    
//     index = model->index(index.row(), headerPosGameIndex_, modelindex);
    index = model->index(index.row(), 12, modelindex);
    QVariant vindex = model->data(index, Qt::DisplayRole);
    int gindex = vindex.toInt();
    u32 gameindex = gindex;
    
//     index = model->index(index.row(), headerPosDatabase_, modelindex);
    index = model->index(index.row(), 0, modelindex);
    QString name = model->data(index, Qt::DisplayRole).toString();
    std::string database = name.toStdString();
//    qDebug("setuppreviewInfo 1");
    // Only get Game Info if GameInfo widget is currently shown
    if (true == databaseViewer_->mainWindow_->gameInfoAct_->isChecked()) {
      databaseViewer_->mainWindow_->gameInfo_->populateInfo(database, gameindex);
    }
  //  qDebug("setuppreviewInfo 2");

    databaseViewer_->mainWindow_->dockPreviewBoard_->previewBoard_->setupBoard(database
        , gameindex, moveNumber_);
 //   qDebug("setuppreviewInfo 3");

  }
}



void GamesTab::setupGameInfo()
{
//   QModelIndex index = myProxyModel->mapToSource(treeView->currentIndex());
  QModelIndex index = treeView->currentIndex();
  if (index.row() >= 0) { //Valid item row selected
//     index = model->index(index.row(), headerPosGameIndex_, modelindex);
    index = model->index(index.row(), 12, modelindex);
    QVariant vindex = model->data(index, Qt::DisplayRole);
    int gindex = vindex.toInt();
    u32 gameindex = gindex;
//     index = model->index(index.row(), headerPosDatabase_, modelindex);
    index = model->index(index.row(), 0, modelindex);
    QString name = model->data(index, Qt::DisplayRole).toString();
    std::string database = name.toStdString();
    
    assert (true == databaseViewer_->mainWindow_->gameInfoAct_->isChecked());
    
    databaseViewer_->mainWindow_->gameInfo_->populateInfo(database, gameindex);
  }
}



//Opens a game on a new board created from MainWindow
void GamesTab::openGame()
{
//   QModelIndex index = myProxyModel->mapToSource(treeView->currentIndex());
  QModelIndex index = treeView->currentIndex();
  if (index.row() >= 0) { //Valid item row selected
//     index = model->index(index.row(), headerPosGameIndex_, modelindex);
    index = model->index(index.row(), 12, modelindex);
    QVariant vindex = model->data(index, Qt::DisplayRole);
    int gindex = vindex.toInt();
    u32 gameindex = gindex;
//     index = model->index(index.row(), headerPosDatabase_, modelindex);
    index = model->index(index.row(), 0, modelindex);
    QString name = model->data(index, Qt::DisplayRole).toString();
    std::string database = name.toStdString();
    
//     int moveNumber = databaseViewer_->mainWindow_->selectedBoardWindow_->mainBoard->
    
    //! FIXME open game at the move being shown on the mainBoard
    // Difficult beacuse we can only get stone count which may be different.
    databaseViewer_->mainWindow_->createNewBoard(database, gameindex);
  }
}



//! To remove a game from the database, we do not remove the stored pattern information
//! Instead we delete the reference to the game and so the pattern is not found in the database
  
// Not implemented - TODO need some thought about design - best way to do this
void GamesTab::removeGame()
{
  /*
//     QModelIndex index = myProxyModel->mapToSource(treeView->currentIndex());
  QModelIndex index = treeView->currentIndex();
  if (index.row() >= 0) { //Valid item row selected
    index = model->index(index.row(), headerPosGameIndex_, modelindex);
    QVariant vindex = model->data(index);
//     int gindex = vindex.toInt();
//     u32 gameindex = gindex;
    index = model->index(index.row(), headerPosDatabase_, modelindex);
    QString name = model->data(index).toString();
    std::string database = name.toStdString();
//     databaseViewer_->mainWindow_->createNewBoard(database, gameindex);
    //! delete game
  }
  */
}




void GamesTab::updateList()
{
  setupModel();
  showMatches();
}



