
#ifndef GAMESTAB_H
#define GAMESTAB_H

#include <QWidget>
#include <QLabel>
#include <QLineEdit>
#include <QAction>
#include <QMenu>
#include <QModelIndex>
#include <QTreeView>
#include <QTableView>
#include <QEvent>
#include <QStandardItemModel>
#include <QSortFilterProxyModel>

#include "DbManager.h"
#include "DatabaseViewer.h"

class DbManager;
class DatabaseViewer;
class GamesTab;


class MySortFilterProxyModel : public QSortFilterProxyModel
{
  Q_OBJECT

  public:
    MySortFilterProxyModel(GamesTab *parent);
//     bool MySortFilterProxyModel::filterAcceptsRow(int sourceRow,
//         const QModelIndex &sourceParent) const;
    void setPlayerWhiteFilter(const QString& filter);
    void setPlayerBlackFilter(const QString& filter);
    void setRankWhiteFilter(const QString& filter);
    void setRankBlackFilter(const QString& filter);
    void setResultFilter(const QString& filter);
    void setPlaceFilter(const QString& filter);
    void setEventFilter(const QString& filter);
    void showMatches();
  protected:

  private:
    DbManager* dbManager_;
    GamesTab* parent_;
    QString filterPlayerWhite;
    QString filterPlayerBlack;
    QString filterRankWhite;
    QString filterRankBlack;
    QString filterResult;
    QString filterPlace;
    QString filterEvent;
};



class ResultsModel : public QAbstractItemModel
{
  Q_OBJECT

  public:
    ResultsModel(QObject *parent = 0);
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QModelIndex index ( int row, int column, const QModelIndex & parent = QModelIndex() ) const;
    QModelIndex parent (const QModelIndex & index) const;
    void setResults(MainWindow* mainWindow);
  private:
    MainWindow* mainWindow_;
    bool hasMainBoard;
};




class GamesTab : public QWidget
{
  Q_OBJECT

  public:
    GamesTab(DbManager* db, DatabaseViewer* databaseViewer, QWidget *parent = 0);
    void writeSettings();
    void doMatches();
    
    void setCurrentMoveNumber(int moveNumber);
    QTreeView *treeView;
    void setupGameInfo();
    void setupModel();
  public slots:
    void setupPreviewAndInfo();

  protected:
//     void clicked ( const QModelIndex & index );
     
     virtual void contextMenuEvent(QContextMenuEvent *event);
//      virtual void selectionChanged(QItemSelection & selected, QItemSelection & deselected);
//     virtual bool event(QEvent *);
    
  private slots:
    
    void openGame();
    void removeGame();
    void updateList();
    void autoUpdateClick(bool on);
    
  private:
    void showMatches();
    int moveNumber_;
    DatabaseViewer* databaseViewer_;
    DbManager* dbManager_;
    QAction* openGameAct_; 
    QAction* removeGameAct_;
    QAction* updateListAct_;
    QAction* autoUpdateAct_;
    
    QItemSelectionModel* selectionModel_;
    ResultsModel* model;
    MySortFilterProxyModel *myProxyModel;
    QModelIndex modelindex;
    QMenu *gameMenu;
    
    
    int headerPosDatabase_;
    int headerPosPlayerWhite_;
    int headerPosRankWhite_;
    int headerPosPlayerBlack_;
    int headerPosRankBlack_;
    int headerPosResult_;
    int headerPosDate_;
    int headerPosMoveCount_;
    int headerPosKomi_;
    int headerPosHandicap_;
    int headerPosPlace_;
    int headerPosEvent_;
    int headerPosGameIndex_;
    
    
    friend class MainWindow; // Access from filters
    friend class MySortFilterProxyModel;
};

#endif
