
#include <time.h>
#include <iostream>
#include <string>
#include <qmessagebox.h>
#include <QtGui>
#include <QPainter>
#include <QMouseEvent>
#include <QCursor>
#include <QString>
#include <QVariant>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\format.hpp>
#include <C:\Boost\include\boost-1_33_1\boost\lexical_cast.hpp>
#else
#include "boost/format.hpp"
#include "boost/lexical_cast.hpp"
#endif


#include "MainBoard.h"
#include "Resources.h"
#include "WholeBoard.h"

using boost::lexical_cast;

MainBoard::MainBoard(MainWindow* mainWindow, EditStatus editStatus, QWidget *parent) :
    QWidget(parent), mainWindow_(mainWindow)
{
//   qDebug("MainBoard constructor");
//    std::cout << "mainBoard constructor2" << std::endl;

  setObjectName("MainBoard"); 
//   setWindowTitle(tr("Board"));
  
  setFocusPolicy ( Qt::StrongFocus );
  //seed random number with current time in secs since 1970
  srand ( time(NULL) );
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
      whiteStoneType_[x][y] = rand()%35; // between 0 and 35
//       std::cout << "stone Type = " << (int)whiteStoneType_[x][y] << std::endl;
    }
  }
  
//   std::cout << "mainBoard constructor1" << std::endl;
  boards_.clear(); // vector of boards for move navigation
  boost::shared_ptr<Board> board (new Board);
  boards_.push_back(board); // Add the first baord as empty
  moveNumber_ = 0;
  totalMoves_ = 0;
        
//   boardPosition += moveNumber_;
//   boards_.insert(boardPosition, board);
  
//   boardPosition = boards_.begin();
//   std::cout << "mainBoard constructor2" << std::endl;
  setMinimumSize ( 100, 100 ); //int minw, int minh
  setMaximumSize ( 800, 800 );
  setSizeIncrement ( 0, 0 );
  setMouseTracking (true);

  QString boardDir = Resources::path();
  boardImage = QPixmap(boardDir + "Beige1.png");
  QPalette palette;
  palette.setBrush(backgroundRole(), QBrush(boardImage));
  setPalette(palette);
  
  handCursor = new QBitmap(QPixmap(":/resources/icons/handcursor.png"));
  handCursorMask = new QBitmap(QPixmap(":/resources/icons/handcursormask.png"));
  handCursorWhite = new QBitmap(QPixmap(":/resources/icons/handcursorwhite.png"));
  handCursorWhiteMask = new QBitmap(QPixmap(":/resources/icons/handcursorwhitemask.png"));
  handCursorBlack = new QBitmap(QPixmap(":/resources/icons/handcursorblack.png"));
  handCursorBlackMask = new QBitmap(QPixmap(":/resources/icons/handcursorblackmask.png"));
  handCursorErase = new QBitmap(QPixmap(":/resources/icons/handcursorerase.png"));
  handCursorEraseMask = new QBitmap(QPixmap(":/resources/icons/handcursorerasemask.png"));
  
  QCursor cursor(*(handCursorBlack), *(handCursorBlackMask), 6, 3);
  setCursor(cursor);
  nextColour_ = BLACK;
  editStatus_ = editStatus;
  switch (editStatus) {
    case ERASE : { 
      selectErase(); 
      break;
    }
    case ADDBLACK : { 
      selectBlack(); 
      break;
    }
    case ADDWHITE : { 
      selectWhite(); 
      break;
    }
    case ADDALTERNATE : { 
      nextColour_ = WHITE;
      selectAlternate(); //This will switch to Black hand cursor and Black next
      break;
    }
  } 
  shiftKeyPress = false;
  readSettings();
  
  boardMenu = new QMenu("Board menu", this);

  boardMenu->setTitle ("Board menu");
  
  modeOneClickAct_ = new QAction(QIcon(":/resources/icons/connect_established.png"), tr("&Auto search"), this);
  modeOneClickAct_->setCheckable(true);
  modeOneClickAct_->setChecked(true);
  boardMenu->addAction (modeOneClickAct_);
  connect(modeOneClickAct_, SIGNAL(triggered(bool)), this, SLOT(changeModeOneClick(bool)));
  displayAnalysis = true;
  
  searchAct = new QAction(tr("&Search"), this);
  boardMenu->addAction (searchAct);
  boardMenu->addSeparator ();
  
  connect(searchAct, SIGNAL(triggered()), this, SLOT(search()));
  
  searchActions_ = new QActionGroup(this);
  searchActions_->setExclusive(true);
  
  searchWholeBoardAct_ = new QAction(tr("&Wholeboard pattern"), this);
  searchWholeBoardAct_->setCheckable(true);
  searchWholeBoardAct_->setChecked(true);
  searchWholeBoardAct_->setEnabled (true);
  
  searchNineByNineAct_ = new QAction(tr("&9x9 pattern"), this);
  searchNineByNineAct_->setCheckable(true);
  searchNineByNineAct_->setEnabled (false);
  
  searchTwelveByTwelveAct_ = new QAction(tr("&12x12 pattern"), this);
  searchTwelveByTwelveAct_->setCheckable(true);
  searchTwelveByTwelveAct_->setEnabled (false);
  
  searchHalfBoardAct_ = new QAction(tr("&Halfboard pattern"), this);
  searchHalfBoardAct_->setCheckable(true);
  searchHalfBoardAct_->setEnabled (false);
  
  searchActions_->addAction (searchWholeBoardAct_);
  searchActions_->addAction (searchNineByNineAct_);
  searchActions_->addAction (searchTwelveByTwelveAct_);
  searchActions_->addAction (searchHalfBoardAct_);
  
  boardMenu->addActions(searchActions_->actions());
  boardMenu->addSeparator ();
    
  rotateBoardAct_ = new QAction(QIcon(":/resources/icons/rotate.png"), tr("&Rotate board 90 degrees"), this);
  horizontalMirrorAct_ = new QAction(QIcon(":/resources/icons/horizontalflip.png"), tr("&Horizontal mirror"), this);
  verticalMirrorAct_ = new QAction(QIcon(":/resources/icons/verticalflip.png"), tr("&vertical mirror"), this);
  
  boardMenu->addAction (rotateBoardAct_);
  boardMenu->addAction (horizontalMirrorAct_);
  boardMenu->addAction (verticalMirrorAct_);
  
  connect(rotateBoardAct_, SIGNAL(triggered()), this, SLOT(selectRotate()));
  connect(horizontalMirrorAct_, SIGNAL(triggered()), this, SLOT(selectHorizontalMirror()));
  connect(verticalMirrorAct_, SIGNAL(triggered()), this, SLOT(selectVerticalMirror()));
  
  boardMenu->addSeparator ();

  coordinatesAct_ = new QAction(tr("&Board Coordinates"), this);
  coordinatesAct_->setCheckable(true);
  coordinatesAct_->setChecked(false);
  connect(coordinatesAct_, SIGNAL(triggered()), this, SLOT(update()));
  boardMenu->addAction(coordinatesAct_);

  font_ = new QFont("Helvetica [Cronyx]", 10);  
  setWindowIcon(QIcon(":/resources/icons/misc.png"));
}



MainBoard::~MainBoard() {}



//! Possible re-deign to use a class Game instead of repeating functionality
// Copy the game into MainBoards Boards vector
void MainBoard::copyGameBoards(boost::shared_ptr<Game> game)
{
  // Iterate through boards and copy them into MainBoards boards.
  // Do not inlcude the first blank move since we already have a blank move
  // At the creation of the MainBoard

  std::vector<boost::shared_ptr<Board> >::iterator i;
  for (i = game->boards_.begin() + 1; i != game->boards_.end(); i++) {
    boost::shared_ptr<Board> board (new Board);
    boards_.push_back(board); // add new board
    (*i)->copyBoard(board);
  }
}



//-------------------------------------------------------------------
//! Calls the search from mainWindow_
void MainBoard::search()
{
//   std::cout << "do search" << std::endl;
  
  // moveNumber is no of stones on board
  // moveNumber_ is current move in current boards history
  WholeBoard wholeBoard;
  u16 moveNumber = boards_[moveNumber_]->getBoardStoneCount();
  
  unsigned char board[19][19];
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
      board[x][y] = boards_[moveNumber_]->getStone(x, y);
    }
  }
  u8 wbPatternSelected = 0; // returns pattern selected from getMainBoardPattern
  u64 value = wholeBoard.getMainBoardPattern(board, wbPatternSelected);
//   std::cout << "moves=" << moveNumber << " pattern=" << value;
//   std::cout << " WbPatternSelected=" << (int)wbPatternSelected << std::endl;
  //get move number and value from board
  displayAnalysis = true; // Always display analysis from a search
  mainWindow_->search(moveNumber, value, wbPatternSelected);
  update();
  // If one click search is off then we need to tell paint event that we dont display
  // any analysis at update time.
}



// Change edit mode - 
// One click to add stone or remove and immediately search
// No searches when stones placed or removed form the board
void MainBoard::changeModeOneClick(bool on)
{
//    std::cout << "changed mode click " << (bool)on << std::endl;
  modeOneClickAct_->setChecked(on);
  boardMenu->removeAction(modeOneClickAct_);
  QString iconsDir = Resources::path("icons/");
  if (true == on) {
//     qDebug("on");
    QPixmap pixmapConnectEstablished(iconsDir + "connect_established.png");
    QIcon iconConnectEstablished(pixmapConnectEstablished);
    modeOneClickAct_ = new QAction(iconConnectEstablished, tr("&Auto search"), this);
    modeOneClickAct_->setCheckable(true);
    modeOneClickAct_->setChecked(true);
    
  }
  else {
    QPixmap pixmapConnectNo(iconsDir + "connect_no.png");
    QIcon iconConnectNo(pixmapConnectNo);
    modeOneClickAct_ = new QAction(iconConnectNo, tr("&Auto search"), this);
    modeOneClickAct_->setCheckable(true);
    modeOneClickAct_->setChecked(false);
  }
  boardMenu->insertAction (searchAct, modeOneClickAct_);
  connect(modeOneClickAct_, SIGNAL(triggered(bool)), this, SLOT(changeModeOneClick(bool)));
}



void MainBoard::contextMenuEvent(QContextMenuEvent *event)
{
  event->accept();
  boardMenu->exec(event->globalPos());
}



bool MainBoard::event(QEvent *e)
{
  return QWidget::event(e);
}



void MainBoard::readSettings()
{
  QSettings settings;
  settings.beginGroup("MainBoard");
  QPoint pos = settings.value("pos", QPoint(400, 50)).toPoint();
  QSize size = settings.value("size", QSize(400, 400)).toSize();
  settings.endGroup();
  resize(size);
  move(pos);
}


void MainBoard::writeSettings()
{
  QSettings settings;
  settings.beginGroup("MainBoard");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.endGroup();
}



void MainBoard::setIdentifier(int identifier)
{
  identifier_ = identifier;
}



// Let MainWindow know that the board is closing, so that the menu item can be deleted
void MainBoard::closeEvent(QCloseEvent *event)
{
  mainWindow_->deleteBoardWindow(identifier_);
  writeSettings();
  event->accept();
}




void MainBoard::selectErase()
{
//    std::cout << "hello in mainboard" << std::endl;
  editStatus_ = ERASE;
  QCursor cursor(*handCursorErase, *handCursorEraseMask, 6, 3);
//    std::cout << "hello in mainboard2" << std::endl;
  setCursor(cursor);
//    std::cout << "hello in mainboard3" << std::endl;
}



void MainBoard::selectAlternate()
{
//   if (ADDALTERNATE == editStatus_) {
    if (BLACK == nextColour_) {
      nextColour_ = WHITE;
      QCursor cursor(*handCursorWhite, *handCursorWhiteMask, 6, 3);
      setCursor(cursor);
    }
    else if (WHITE == nextColour_) {
      nextColour_ = BLACK;
      QCursor cursor(*handCursorBlack, *handCursorBlackMask, 6, 3);
      setCursor(cursor);
    }
    editStatus_ = ADDALTERNATE;
//   }
//   else {
//     editStatus_ = ADDALTERNATE;
//     nextColour_ = BLACK;
//     QCursor cursor(*handCursorBlack, *handCursorBlackMask, 6, 3);
//     setCursor(cursor);
//   }
}



void MainBoard::selectBlack()
{
  editStatus_ = ADDBLACK;
  nextColour_ = BLACK;
  QCursor cursor(*handCursorBlack, *handCursorBlackMask, 6, 3);
  setCursor(cursor);
}



void MainBoard::selectWhite()
{
  editStatus_ = ADDWHITE;
  nextColour_ = WHITE;
  QCursor cursor(*handCursorWhite, *handCursorWhiteMask, 6, 3);
  setCursor(cursor);
  
}



void MainBoard::selectRotate()
{
//   std::cout << "do rotate" << std::endl;
  std::vector<boost::shared_ptr<Board> >::iterator i;
  for (i = boards_.begin(); i != boards_.end(); i++) {
    (*i)->rotate();
  }
  updateBoard();
}



void MainBoard::selectHorizontalMirror()
{
//   std::cout << "do hor mirror" << std::endl;
  std::vector<boost::shared_ptr<Board> >::iterator i;
  for (i = boards_.begin(); i != boards_.end(); i++) {
    (*i)->xAxisMirror();
  }
  updateBoard();
}



void MainBoard::selectVerticalMirror()
{
//   std::cout << "do vert mirror" << std::endl;
  std::vector<boost::shared_ptr<Board> >::iterator i;
  for (i = boards_.begin(); i != boards_.end(); i++) {
    (*i)->yAxisMirror();
  }
  updateBoard();
}




void MainBoard::keyPressEvent ( QKeyEvent * event )
{
//   std::cout << event->key() << std::endl;
  if (event->key() == Qt::Key_Shift) {
//     shiftKeyHeld = true;
//     std::cout << event->key() << std::endl;
    shiftKeySwapStatus();
  }
  if (event->key() == Qt::Key_Left) {
    selectBack();
  }
  if (event->key() == Qt::Key_Right) {
    selectForward();
  }
  if (event->key() == Qt::Key_Up) {
    selectBackTen();
  }
  if (event->key() == Qt::Key_Down) {
    selectForwardTen();
  }
  if (event->key() == Qt::Key_Home) {
    selectFirst();
  }
  if (event->key() == Qt::Key_End) {
    selectLast();
  }
}




void MainBoard::keyReleaseEvent ( QKeyEvent * event )
{
  //std::cout << event->key() << std::endl;
  if (event->key() == Qt::Key_Shift) {
    //std::cout << event->key() << std::endl;
    //shiftKeyHeld = false;
    shiftKeySwapStatus();
  }
}



void MainBoard::shiftKeySwapStatus()
{
  if (ADDBLACK == editStatus_) {
    QCursor cursor(*handCursorWhite, *handCursorWhiteMask, 6, 3);
    setCursor(cursor);
    editStatus_ = ADDWHITE;
  }
  else if (ADDWHITE == editStatus_) {
    QCursor cursor(*handCursorBlack, *handCursorBlackMask, 6, 3);
    setCursor(cursor);
    editStatus_ = ADDBLACK;
  }
  else if (ADDALTERNATE == editStatus_) {
    if (BLACK == nextColour_) {
      QCursor cursor(*handCursorWhite, *handCursorWhiteMask, 6, 3);
      setCursor(cursor);
      nextColour_ = WHITE;
    }
    else if (WHITE == nextColour_) {
      QCursor cursor(*handCursorBlack, *handCursorBlackMask, 6, 3);
      setCursor(cursor);
      nextColour_ = BLACK;
    }
  }
}



void MainBoard::focusInEvent ( QFocusEvent * event )
{
//   std::cout << "in focus event" << std::endl;
  repaint(); // For Windows menu sets this board active - so repaint before game list is done
  mainWindow_->setWindow(identifier_);
  updateNavigationToolbar();
  mainWindow_->toolBarBoard->setToolBarState(true);
  mainWindow_->toolBarEdit->setToolBarState(true);
  event->accept();
//   mainWindow_->changeStatsTab();
//   mainWindow_->statisticsBlack_->showStatisticsBlack();
//   mainWindow_->statisticsWhite_->showStatisticsWhite();
  
  // Clear the Games List and show matches if auto update is set true
//   mainWindow_->databaseViewer_->gamesTab->setupModel();
//   mainWindow_->databaseViewer_->gamesTab->doMatches();
}



void MainBoard::mousePressEvent ( QMouseEvent * event )
{
  //int x=0;
  //int y=0;
//   setFocus(Qt::MouseFocusReason);
//   activateWindow(); // Set to active window
  //! Note raise() may not work properly on a Windows OS - see Qt4.1 docs
//   raise(); // raise it to top if not already 
  
  // This window was activated so signal to mainWindow to tell it
  // That we will use this board as the currently selected board.
//   mainWindow_->setWindow(identifier_);
  
  if (event->button() == Qt::LeftButton) {
    double x = event->x();
    double y = event->y();
    // std::cout << "x pos=" << x << " y pos=" << y << std::endl;
    // if ((x > xstart_) && (y > ystart_)) {
    // Determine which board coordinate mouse press was in.
    x = x - xstart_ - (spacing_ / 2);
    y = y - ystart_ - (spacing_ / 2);
    if ((x > 0) && (y > 0)) {
    //std::cout << "x widg=" << x << " y widg=" << y << std::endl;
    int xCoord = static_cast<int>(x / spacing_);
    int yCoord = static_cast<int>(y / spacing_);
    //std::cout << "x=" << xCoord << " y=" << yCoord << std::endl;
    
      if ((xCoord >= 0) && (xCoord < 19) && (yCoord >= 0) && (yCoord < 19)) {
        // Do nothing if we try to erase an empty point
        // Do nothing if we try to add a stone where there is already a stone
        if (ERASE == editStatus_) {
          if (boards_[moveNumber_]->getStone(xCoord, yCoord) == EMPTY) {
            return;
          }
        }
        else if (ERASE != editStatus_) {
          if (boards_[moveNumber_]->getStone(xCoord, yCoord) != EMPTY) {
            return;
          }
        }
        
        
        // Make a new board
        boost::shared_ptr<Board> board (new Board);
        std::vector<boost::shared_ptr<Board> >::iterator boardPosition;
        boardPosition = boards_.begin();
//         std::cout << "move number" << moveNumber_ << std::endl;
        boardPosition += moveNumber_;
        boards_[moveNumber_]->copyBoard(board); // Copy current board to new board
        boards_.insert(boardPosition, board); // Insert new board at current position in navigation
        
        ++moveNumber_;
        ++totalMoves_;
//         std::cout << "move number = " << moveNumber_ << std::endl;
//         std::cout << "total Moves = " << totalMoves_ << std::endl;
        updateNavigationToolbar();
        
        if (ERASE == editStatus_) {
          boards_[moveNumber_]->setStone(xCoord, yCoord, EMPTY);
        }
        if (ADDBLACK == editStatus_) {
          addStoneLogic(xCoord, yCoord, BLACK);
//           boards_[moveNumber_]->setStone(xCoord, yCoord, BLACK);
        }
        if (ADDWHITE == editStatus_) {
//           boards_[moveNumber_]->setStone(xCoord, yCoord, WHITE);
          addStoneLogic(xCoord, yCoord, WHITE);
        }
        if (ADDALTERNATE == editStatus_) {
          if (BLACK == nextColour_) {
//             boards_[moveNumber_]->setStone(xCoord, yCoord, BLACK);
            addStoneLogic(xCoord, yCoord, BLACK);
            nextColour_ = WHITE;
            QCursor cursor(*handCursorWhite, *handCursorWhiteMask, 6, 3);
            setCursor(cursor);
          }
          else if (WHITE == nextColour_) {
//             boards_[moveNumber_]->setStone(xCoord, yCoord, WHITE);
            addStoneLogic(xCoord, yCoord, WHITE);
            nextColour_ = BLACK;
            QCursor cursor(*handCursorBlack, *handCursorBlackMask, 6, 3);
            setCursor(cursor);
          }
        }
//         std::cout << "can do 1" << std::endl;
//         displayAnalysis = modeOneClickAct_->isChecked();
        displayAnalysis = false;
        repaint();
        if (true == modeOneClickAct_->isChecked()) {
          search();
        }
        update();
      }
    }
  }
}



bool MainBoard::addStoneLogic(i8 x, i8 y, StoneColour colour)
{
  boards_[moveNumber_]->clearCaptures();
  boards_[moveNumber_]->setStone(x, y, colour);
  bool isDead = boards_[moveNumber_]->checkDeadGroup(x, y);
  u8 enemy = BLACK; // initialize to value to avoid warning
  if (BLACK == colour) { 
    enemy = WHITE; 
  }
  if (WHITE == colour) { 
    enemy = BLACK; 
  }
  bool hasKilled = false;
  if (boards_[moveNumber_]->removeDeadGroup(x - 1, y, enemy) == true) {
    hasKilled = true; 
  }
  if (boards_[moveNumber_]->removeDeadGroup(x, y - 1, enemy) == true) {
    hasKilled = true; 
  }
  if (boards_[moveNumber_]->removeDeadGroup(x + 1, y, enemy) == true) {
    hasKilled = true; 
  }
  if (boards_[moveNumber_]->removeDeadGroup(x, y + 1, enemy) == true) {
    hasKilled = true; 
  }
  if ((true == isDead) && (false == hasKilled)) { // Illegal move
    if (false == RULESALL) { // Depending on rule set ( i.e. Ing rules )
      boards_[moveNumber_]->setStone(x, y, EMPTY);
//       cout << "illegal move!!!!!! doPatternBoard" << endl;
      string dud;
      cin >> dud;
    }
  }
  return true;
}



void MainBoard::paintEvent(QPaintEvent *)
{
  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing, true);
  
  double height = size().height();
  double width = size().width();
  double xpadding = 0 ;
  double ypadding = 0 ;
  if (height > width) {
    ypadding = (height - width) / 2;
    height = width;
  }
  else if (height < width) {
    xpadding = (width - height) /2;
    width = height;
  }
  if (coordinatesAct_->isChecked() == true) {
    edge_ = width / 40;
  }
  else {
    edge_ = 0;
  }
    

  width = width - (edge_ * 2);
  height = height - (edge_ * 2);
  
//   std::cout << "edge=" << edge << std::endl;
  double spacing = width / 20;
  spacing_ = spacing;
  
  double startposH = xpadding + edge_ + spacing;
  double endposH = xpadding + edge_ + (19 * spacing);
 
  double startposV = ypadding + edge_ + spacing;
  double endposV = ypadding + edge_ + (19 * spacing);
  
  xstart_ = xpadding + edge_;
  ystart_ = ypadding + edge_;
  
  // Draw Lines on Board
  QPen pen(Qt::black, 1);
  painter.setPen(pen);
  double pos = 0;
  for (int i = 0; i < 19 ; i++) {
    // horizontal
    painter.drawLine(static_cast<int>(startposH), static_cast<int>(startposV + pos),
                     static_cast<int>(endposH), static_cast<int>(startposV + pos));
    // vertical
    painter.drawLine(static_cast<int>(startposH + pos), static_cast<int>(startposV),
                     static_cast<int>(startposH + pos), static_cast<int>(endposV));
    pos += spacing_;
  }
  
  // Draw star points
  drawCircle(&painter, 3, 3);
  drawCircle(&painter, 3, 9);
  drawCircle(&painter, 3, 15);
  drawCircle(&painter, 9, 3);
  drawCircle(&painter, 9, 9);
  drawCircle(&painter, 9, 15);
  drawCircle(&painter, 15, 3);
  drawCircle(&painter, 15, 9);
  drawCircle(&painter, 15, 15);
  
  if (coordinatesAct_->isChecked() == true) {
    drawCoordinates(&painter);
  }
  drawStones(&painter);
  if (true == displayAnalysis) { //
    drawAnalysis(&painter);
  }
  
  painter.end();
}




//Draw at board coordinate
void MainBoard::drawCircle(QPainter* painter, int x, int y)
{
  int diameter = 5;
  int offset = 2;
  if (spacing_ < 15) {
    diameter = 3;
    offset = 1;
  }
  if (spacing_ < 8) {
    diameter = 1;
    offset = 1;
  }
  
  int xpos = static_cast<int>(xstart_ + (spacing_ * (x + 1)) - offset);
  int ypos = static_cast<int>(ystart_ + (spacing_ * (y + 1)) - offset);
  QRectF rectangle(xpos, ypos, diameter, diameter);
  QBrush brush(Qt::black, Qt::SolidPattern);
  painter->setBrush(brush);
  painter->drawEllipse(rectangle);
}



void MainBoard::drawStones(QPainter* painter)
{
  if (0 != moveNumber_) {
    std::string blackDirPath = Resources::path("BlackStones/").toStdString();
    std::string whiteDirPath = Resources::path("Tsuke.Stones/").toStdString();
    int size = static_cast<int>(spacing_ + 2);

    if ((size >= 1) && (size <=48)) {
      std::string blackPath = blackDirPath + str(boost::format("%d") % size) + ".png";
      QPixmap blackStone = QPixmap(blackPath.c_str());
    
      int offsetpixmap = static_cast<int>((spacing_ / 2) +1);
      for (int x = 0; x < 19; x++) {
        for (int y = 0; y < 19; y++) {
          if (EMPTY != boards_[moveNumber_]->getStone(x, y)) {
            
            int xpos = static_cast<int>(xstart_ + (spacing_ * (x + 1)) - offsetpixmap);
            int ypos = static_cast<int>(ystart_ + (spacing_ * (y + 1)) - offsetpixmap);
            if (BLACK == boards_[moveNumber_]->getStone(x, y)) { //colour = black
              painter->drawPixmap (xpos, ypos, blackStone );
            }
            else if (WHITE == boards_[moveNumber_]->getStone(x, y)) { // colour = white
              std::string whitePath;
              if ((size >= 20) && (size <=48)) {
                whitePath = whiteDirPath + 
                  str(boost::format("%d") % size) + "-" + 
                  str(boost::format("%d") % whiteStoneType_[x][y]) + ".png";
              }
              if (size < 20) {
                whitePath = whiteDirPath + 
                    str(boost::format("%d") % size) + "-" + "0.png";
              } 
              QPixmap whiteStone = QPixmap(whitePath.c_str());
      
              painter->drawPixmap (xpos, ypos, whiteStone );
            }
          }
        }
      }
    }
  }
}



void MainBoard::drawAnalysis(QPainter* painter)
{
//   qDebug("draw Analysis");
  int size = static_cast<int>(spacing_);
  int xpos;
  int ypos;
  int x,y;
  QString letters("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
  QString text;
  int count = 0;
  
  if ((size >= 1) && (size <=48)) {
    int offsetpixmap = static_cast<int>((spacing_ / 2) +1);

    QSettings settings;
    int num = settings.value("Configuration/NextMoveCount", 20).toInt();
    vector<boost::shared_ptr<Continuations> >::iterator j;
    // Display black next moves if black is to play next
    if (BLACK == nextColour_) {
      int i = 0;
//       j = mainWindow_->dbManager_->statsBlack.begin();
      j = statsBlack.begin();
//       while ((i < num) && (j != mainWindow_->dbManager_->statsBlack.end())) {
      
//       std::cout << "statsBlack size = " << statsBlack.size() << std::endl;
      while ((i < num) && (j != statsBlack.end())) {
        x = ((*j)->nextMoveX);
        y = ((*j)->nextMoveY);
        xpos = static_cast<int>(xstart_ + (spacing_ * (x + 1)) - offsetpixmap);
        ypos = static_cast<int>(ystart_ + (spacing_ * (y + 1)) - offsetpixmap);
        QPen pen(Qt::white, 1);
        painter->setPen(pen);
        QFont sansFont("Courier", size - 2);
        sansFont.setWeight(QFont::Normal);
        painter->setFont(sansFont);
        text.clear();
        text.append(letters[count]);
        count++;
        painter->drawText ( xpos, ypos, size, size, Qt::AlignCenter, text, 0 );
        ++i;
        ++j;
      }
    }
    else { 
      int i = 0;
//      j = mainWindow_->dbManager_->statsWhite.begin();
        j = statsWhite.begin();
//      while ((i < num) && (j != mainWindow_->dbManager_->statsWhite.end())) {
        while ((i < num) && (j != statsWhite.end())) {
        x = (*j)->nextMoveX;
        y = (*j)->nextMoveY;
        xpos = static_cast<int>(xstart_ + (spacing_ * (x + 1)) - offsetpixmap);
        ypos = static_cast<int>(ystart_ + (spacing_ * (y + 1)) - offsetpixmap);
        QPen pen(Qt::white, 1);
        painter->setPen(pen);
        QFont sansFont("Courier", size - 2);
        sansFont.setWeight(QFont::Normal);
        painter->setFont(sansFont);
        text.clear();
        text.append(letters[count]);
        count++;
        painter->drawText ( xpos, ypos, size, size, Qt::AlignCenter, text, 0 );
        ++i;
        ++j;
      }
    }
  }
}



void MainBoard::drawCoordinates(QPainter* painter)
{
  int size = static_cast<int>(spacing_ - (spacing_ / 2)) ;
  if (size <=0) {
    size = 1;
  }
  QFont sansFont("Times", size);
  QPainterPath myPath;
  QPen pen(Qt::black, 1);
  painter->setPen(pen);

  QString letters = "ABCDEFGHJKLMNOPQRST";
  QString textHorizontal;
  QString textVertical;
  QString number;
  int yPosHorizontal = static_cast<int>(ystart_ - edge_ );
  int yPosHorizontalEnd = static_cast<int>(ystart_ + (spacing_ * 19) + edge_);
  int xPosHorizontal;
  int yPosVertical;
  int xPosVertical = static_cast<int>(xstart_ - edge_);
  int xPosVerticalEnd = static_cast<int>(xstart_ + (spacing_ * 19) + edge_);

  double posH = xstart_ + (spacing_ / 2);
  double posV = ystart_ + (spacing_ / 2) + 1;
  
  for (int i = 0; i < 19; i++) {
//     QFont sansFont("Times", size);
//     sansFont.setWeight(QFont::Normal);
    font_->setPointSize(size);
    painter->setFont(*font_);
    textHorizontal.clear();
    textHorizontal.append(letters[i]);
    textVertical.clear();
    textVertical.append((lexical_cast<std::string>(19 - i)).c_str());
    
    xPosHorizontal = static_cast<int>(posH);
    yPosVertical = static_cast<int>(posV);
    painter->drawText ( xPosHorizontal, yPosHorizontal, size + size, size + size, Qt::AlignCenter,
                       textHorizontal, 0 );
    painter->drawText ( xPosVertical, yPosVertical, size + size, size + size, Qt::AlignCenter, 
                       textVertical, 0 );
    
    painter->drawText ( xPosHorizontal, yPosHorizontalEnd, size + size, size + size, Qt::AlignCenter,
                        textHorizontal, 0 );
    painter->drawText ( xPosVerticalEnd, yPosVertical, size + size, size + size, Qt::AlignCenter, 
                        textVertical, 0 );
    
    posH += spacing_;
    posV += spacing_;
  }
}



//---------------------------------------------------------
// Navigation Actions from MainWindow ToolBar Main Moves

void MainBoard::updateNavigationToolbar()
{
  mainWindow_->toolBarMainMoves->setToolBarState(true);
  if (moveNumber_ >= totalMoves_) {
    moveNumber_ = totalMoves_;
    mainWindow_->toolBarMainMoves->lastAct->setEnabled(false);
    mainWindow_->toolBarMainMoves->forwardAct->setEnabled(false);
    mainWindow_->toolBarMainMoves->forwardTenAct->setEnabled(false);
  }
  if (moveNumber_ <= 0 ) {
    moveNumber_ = 0;
    mainWindow_->toolBarMainMoves->firstAct->setEnabled(false);
    mainWindow_->toolBarMainMoves->backAct->setEnabled(false);
    mainWindow_->toolBarMainMoves->backTenAct->setEnabled(false);
  }
}



void MainBoard::selectForward()
{
  if ((moveNumber_ != totalMoves_) && (ADDALTERNATE == editStatus_)) {
    selectAlternate();
  }
//   std::cout << "selected Forward :" << std::endl;
  ++moveNumber_;
  
  updateNavigationToolbar(); 
//   std::cout << "move number = " << moveNumber_ << std::endl;
//   std::cout << "total Moves = " << totalMoves_ << std::endl;
  updateBoard();
}



void MainBoard::selectBack()
{
  if ((moveNumber_ != 0) && (ADDALTERNATE == editStatus_)) {
    selectAlternate();
  }
//   std::cout << "selected back :" << std::endl;
  --moveNumber_;
  updateNavigationToolbar();
//   std::cout << "move number = " << moveNumber_ << std::endl;
//   std::cout << "total Moves = " << totalMoves_ << std::endl;
  updateBoard();
}



void MainBoard::selectForwardTen()
{
  moveNumber_ += 10;
  updateNavigationToolbar();
//   std::cout << "move number = " << moveNumber_ << std::endl;
//   std::cout << "total Moves = " << totalMoves_ << std::endl;
  updateBoard();
}



void MainBoard::selectBackTen()
{
//   std::cout << "selected back ten :" << std::endl;
  moveNumber_ -= 10;
  updateNavigationToolbar();
// //   std::cout << "move number = " << moveNumber_ << std::endl;
//   std::cout << "total Moves = " << totalMoves_ << std::endl;
  updateBoard();
}



void MainBoard::selectFirst()
{
//   std::cout << "selected First :" << std::endl;
  moveNumber_ = 0;
  updateNavigationToolbar();
//   std::cout << "move number = " << moveNumber_ << std::endl;
//   std::cout << "total Moves = " << totalMoves_ << std::endl;
  updateBoard();
}



void MainBoard::selectLast()
{
  moveNumber_ = totalMoves_;
  updateNavigationToolbar();
//   std::cout << "move number = " << moveNumber_ << std::endl;
//   std::cout << "total Moves = " << totalMoves_ << std::endl;
  updateBoard();
}



void MainBoard::updateBoard()
{
  if (true == modeOneClickAct_->isChecked()) {
    search();
    displayAnalysis = true;
//     qDebug("displayAN = true");
  }
  else {
//     qDebug("displayAN = false");
    displayAnalysis = false;
  }
  setFocus(Qt::MouseFocusReason);
  update();
}



void MainBoard::setFont(QFont &font)
{
  font_ = &font;
}

