#ifndef MAINBOARD_H
#define MAINBOARD_H

#include <QWidget>
#include <QBitmap>
#include <QCursor>
#include <QMenu>

#include <string>
#include <vector>
#include <list>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
#include "boost/shared_ptr.hpp"
#endif



#include "Const.h"
#include "MainWindow.h"
#include "Board.h"
#include "Game.h"
#include "DbManager.h"

class MainWindow;
// class DbManager;
class Continuations;


class MainBoard : public QWidget
{
  Q_OBJECT

public:

  MainBoard(MainWindow* mainWindow, EditStatus editStatus, QWidget *parent = 0);
  ~MainBoard();
  void selectErase();
  void selectAlternate();
  void selectBlack();
  void selectWhite();
  
  void selectBack();
  void selectForward();
  void selectBackTen();
  void selectForwardTen();
  void selectLast();
  void selectFirst();
  void setFont(QFont &font);
  void copyGameBoards(boost::shared_ptr<Game> game);
  int totalMoves_;
  void setIdentifier(int identifier);
  
  
  StoneColour nextColour_;
//   std::vector<boost::shared_ptr<Board> >::iterator boardPosition;
//   unsigned char board_[19][19];
  
  vector<boost::shared_ptr<GamesList> > filteredResults;
  vector<boost::shared_ptr<Continuations> > statsBlack;
  vector<boost::shared_ptr<Continuations> > statsWhite;
  
public slots:
  void selectRotate();
  void selectHorizontalMirror();
  void selectVerticalMirror();
  void search();
  void changeModeOneClick(bool on);

protected:
  
  void paintEvent(QPaintEvent *event);
  void mousePressEvent ( QMouseEvent * event );
  void keyPressEvent ( QKeyEvent * event );
  void keyReleaseEvent ( QKeyEvent * event );
  void closeEvent(QCloseEvent *event);
  
  virtual void focusInEvent ( QFocusEvent * event );
  
  virtual void contextMenuEvent(QContextMenuEvent *event);
  virtual bool event(QEvent *);

private:
  
  bool addStoneLogic(i8 x, i8 y, StoneColour colour);
  void updateBoard();
  int moveNumber_;
  
  void updateNavigationToolbar();
  QFont* font_;
  void turnToolbarMovesOn();
  MainWindow* mainWindow_;
  QMenu* boardMenu;
  QAction* searchAct;
  
  QAction* searchWholeBoardAct_;
  QAction* searchNineByNineAct_;
  QAction* searchTwelveByTwelveAct_;
  QAction* searchHalfBoardAct_;
  QAction* verticalMirrorAct_;
  QAction* horizontalMirrorAct_;
  QAction* rotateBoardAct_;
  QAction* modeOneClickAct_;
  QActionGroup* searchActions_;
  
  QAction* coordinatesAct_;
  
  
  void shiftKeySwapStatus();
  QPixmap boardImage;
  
  
  QCursor cursor;
  
  QBitmap *handCursor;
  QBitmap *handCursorMask;
  QBitmap *handCursorWhite;
  QBitmap *handCursorWhiteMask;
  QBitmap *handCursorBlack;
  QBitmap *handCursorBlackMask;
  QBitmap *handCursorErase;
  QBitmap *handCursorEraseMask;
  
  void drawCircle(QPainter* painter, int x, int y);
  void drawStones(QPainter* painter);
  void drawAnalysis(QPainter* painter);
  void drawCoordinates(QPainter* painter);
  
  
//   Game game; // Composition of class Game.
  
  std::vector<boost::shared_ptr<Board> > boards_;
//   boost::shared_ptr<Game> game_;
  
  int whiteStoneType_[19][19];
  //std::vector<boost::shared_ptr<Analysis> > analysis;
//   std::vector<Analysis *> vecAnalysis;
  
  double spacing_;
  double edge_;
  double xstart_;
  double ystart_;
  
  EditStatus editStatus_;
  bool shiftKeyPress;
  
//   unsigned int currentMoveNo_;
  
  void readSettings();
  void writeSettings();
  int identifier_;
  bool displayAnalysis;
  
  private slots:
  
};

#endif 
