
#include <QFontDialog>
#include <QtGui>
#include <QAction>
#include <QLayout>
#include <QMenu>
#include <QMenuBar>
#include <QStatusBar>
#include <QTextEdit>
#include <QStackedWidget>
#include <QComboBox>
#include <QMessageBox>
#include <QLabel>
#include <QListWidget>
#include <QCheckBox>
#include <QFileInfo>
#include <QTreeWidget>
#include <QStandardItemModel>
#include <QTreeView>
#include <QTableView>
#include <QSortFilterProxyModel>
#include <QAbstractItemModel>
#include <QHeaderView>
#include <iostream>
#include <QString>
#include <QSettings>
#include <QRegExp>


#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\lexical_cast.hpp>
#else
#include "boost/lexical_cast.hpp"
#endif



#include "MainWindow.h"

using boost::lexical_cast;


GamesTabProgress::GamesTabProgress(QWidget *parent) : QWidget(parent)
{
  progressBar_ = new QProgressBar;
  QGridLayout *layout = new QGridLayout;
  layout->addWidget(progressBar_);
  setLayout(layout);
  layout->setMargin (0);
}



DatabaseActiveGames::DatabaseActiveGames(QWidget *parent) : QWidget(parent)
{
  label_ = new QLabel;
  label_->setFrameStyle(QFrame::NoFrame);
  label_->setAlignment(Qt::AlignRight);
  QGridLayout *layout = new QGridLayout;
  layout->addWidget(label_);
  setLayout(layout);
  layout->setMargin (0);
}



MainWindow::MainWindow(DbManager* db, QWidget *parent, Qt::WFlags flags)
  : QMainWindow(parent, flags)
{
  dbManager_ = db;
  
  setObjectName("MainWindow");
  setWindowTitle("baduK");
  
  toolBarEdit = new ToolBarEdit(this);
  toolBarBoard = new ToolBarBoard(this);
  toolBarMainMoves = new ToolBarMainMoves(this);
  toolBarMoves = new ToolBarMoves(this);

  hasBoardWindowSelected_ = false; // No boards open at application startup
  toolBarMoves->setToolBarState(false);
  toolBarMainMoves->setToolBarState(false);
  toolBarBoard->setToolBarState(false);
  toolBarEdit->setToolBarState(false);
  
  addToolBar(Qt::BottomToolBarArea, toolBarMoves);
  addToolBar(Qt::TopToolBarArea, toolBarEdit);
  addToolBar(Qt::TopToolBarArea, toolBarBoard);
  addToolBar(Qt::TopToolBarArea, toolBarMainMoves);

  dockPreviewBoard_ = new DockPreviewBoard(this);
  addDockWidget(Qt::BottomDockWidgetArea, dockPreviewBoard_);

  editStatus_ = ADDALTERNATE;
  QMenu* menuActions = menuBar()->addMenu(tr("&Actions"));
  QMenu* menuView = menuBar()->addMenu(tr("&View"));
  QMenu* menuSettings = menuBar()->addMenu(tr("&Settings"));
  menuWindow = menuBar()->addMenu(tr("&Window"));
  QMenu* menuHelp = menuBar()->addMenu(tr("&Help"));
 
  newBoardAct = new QAction( QIcon(":/resources/icons/window_new.png"), tr("&New Board"), this);

  menuWindow->addAction(newBoardAct);
  connect(newBoardAct, SIGNAL(triggered()), this, SLOT(addNewBoard()));
  menuWindow->addSeparator ();
  
  menuActions->addAction(newBoardAct);
  menuActions->addSeparator ();  

  addDatabaseAct = new QAction(QIcon(":/resources/icons/wizard.png"), tr("&Add database"), this);
  menuActions->addAction(addDatabaseAct);
  connect(addDatabaseAct, SIGNAL(triggered()), this, SLOT(addDatabaseWizard()));
  menuActions->addSeparator ();
  menuActions->addAction(QIcon(":/resources/icons/exit.png"), tr("&Quit"), this, SLOT(close()));

  databaseViewerAct_ = new QAction(QIcon(":/resources/icons/view_text.png"), tr("&Databases and Games"), this);
  databaseViewerAct_->setCheckable(true);
  databaseViewerAct_->setChecked(true);
  connect(databaseViewerAct_, SIGNAL(triggered()), this, SLOT(showDatabaseViewer()));

  gameInfoAct_ = new QAction(QIcon(":/resources/icons/messagebox_info.png"), tr("&Game Information"), this);
  gameInfoAct_->setCheckable(true);
  gameInfoAct_->setChecked(false);
  connect(gameInfoAct_, SIGNAL(triggered()), this, SLOT(showGameInfo()));

  menuView->addAction(databaseViewerAct_);
  menuView->addAction(dockPreviewBoard_->toggleViewAction());
  menuView->addAction(gameInfoAct_);

  toolBarsMenu = new QMenu;
  toolBarsMenu = menuSettings->addMenu(tr("&Toolbars")); 
  toolBarsMenu->addAction(toolBarBoard->toggleViewAction());
  toolBarsMenu->addAction(toolBarEdit->toggleViewAction());
  toolBarsMenu->addAction(toolBarMoves->toggleViewAction());
  toolBarsMenu->addAction(toolBarMainMoves->toggleViewAction());
  menuSettings->addSeparator();

  setFontAct = new QAction(QIcon(":/resources/icons/fonts.png"), tr("&Set font"), this);
  setFontAct->setEnabled(false);
  menuSettings->addAction(setFontAct);
  
  connect(setFontAct, SIGNAL(triggered()), this, SLOT(selectFont()));
  
  configureBadukAct = new QAction(QIcon(":/resources/icons/configure.png"), tr("&Configure baduK"), this);
  menuSettings->addAction(configureBadukAct);
  connect(configureBadukAct, SIGNAL(triggered()), this, SLOT(configureBaduk()));
  
    helpAction = new QAction(QIcon(":/resources/icons/contents.png"), tr("&Help"), this);
  helpAction->setEnabled(false);
  menuHelp->addAction(helpAction);
  menuHelp->addSeparator ();
  aboutBaduk = new QAction(tr("&About baduK"), this);
  menuHelp->addAction(aboutBaduk);
  connect(aboutBaduk, SIGNAL(triggered()), this, SLOT(showAboutBaduk()));
  
  aboutQtAct = new QAction(QIcon(":/resources/icons/qt-logo.png"), tr("&About &Qt"), this);
  menuHelp->addAction(aboutQtAct);
  aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
  
  tabWidget = new QTabWidget;
  tabWidget->setTabPosition (QTabWidget::West);
  
  // ----------------------------------------------------------
  statisticsBlack_ = new StatisticsBlackTab(this);
//   scrollArea_ = new QScrollArea;
//   scrollArea_->setBackgroundRole(QPalette::NoRole);
//   scrollArea_->setWidget(statisticsBlack_);
//   scrollArea_->show();
  
//   tabWidget->addTab(scrollArea_, tr("Stats"));
  tabWidget->addTab(statisticsBlack_, tr("Stats"));
  tabWidget->setTabIcon (0, QIcon(":/resources/icons/blackstoneicon.png"));
  
  statisticsWhite_ = new StatisticsWhiteTab(this);
//   filters_ = new Filters;

  
  tabWidget->addTab(statisticsWhite_, tr("Stats"));
  tabWidget->setTabIcon (1, QIcon(":/resources/icons/whitestoneicon.png"));
  
//   tabWidget->addTab(filters_, tr("Filters"));
//   tabWidget->addTab(gameInfo_, tr("Game info"));
  tabWidget->resize (300, 400); // width,height
  
  
  readSettings();
  setCentralWidget (tabWidget);
    
  connect(toolBarEdit->eraseAct, SIGNAL(triggered()), this, SLOT(selectErase()));
  connect(toolBarEdit->alternateAct, SIGNAL(triggered()), this, SLOT(selectAlternate()));
  connect(toolBarEdit->addBlackAct, SIGNAL(triggered()), this, SLOT(selectBlack()));
  connect(toolBarEdit->addWhiteAct, SIGNAL(triggered()), this, SLOT(selectWhite()));
    
//     std::cout << "hello3" << std::endl;
  //connect(tabWidget, SIGNAL(currentChanged(QWidget *)), this, SLOT(changeTabFocus()));
//   
//     std::cout << "hello4" << std::endl;
  connect(toolBarMoves->forwardAct, SIGNAL(triggered()), this, SLOT(selectPreviewBoardForward()));
  connect(toolBarMoves->backAct, SIGNAL(triggered()), this, SLOT(selectPreviewBoardBack()));
  connect(toolBarMoves->forwardTenAct, SIGNAL(triggered()), this,
          SLOT(selectPreviewBoardForwardTen()));
  connect(toolBarMoves->backTenAct, SIGNAL(triggered()), this, SLOT(selectPreviewBoardBackTen()));
  connect(toolBarMoves->firstAct, SIGNAL(triggered()), this, SLOT(selectPreviewBoardFirst()));
  connect(toolBarMoves->lastAct, SIGNAL(triggered()), this, SLOT(selectPreviewBoardLast()));
  
  connect(toolBarMainMoves->forwardAct, SIGNAL(triggered()), this, SLOT(selectMainBoardForward()));
  connect(toolBarMainMoves->backAct, SIGNAL(triggered()), this, SLOT(selectMainBoardBack()));
  connect(toolBarMainMoves->forwardTenAct, SIGNAL(triggered()), this, SLOT(selectMainBoardForwardTen()));
  connect(toolBarMainMoves->backTenAct, SIGNAL(triggered()), this, SLOT(selectMainBoardBackTen()));
  connect(toolBarMainMoves->firstAct, SIGNAL(triggered()), this, SLOT(selectMainBoardFirst()));
  connect(toolBarMainMoves->lastAct, SIGNAL(triggered()), this, SLOT(selectMainBoardLast()));
  
  connect(toolBarBoard->rotateAct, SIGNAL(triggered()), this,
          SLOT(selectRotate()));
  connect(toolBarBoard->horizontalMirrorAct, SIGNAL(triggered()), this,
          SLOT(selectHorizontalMirror()));
  connect(toolBarBoard->verticalMirrorAct, SIGNAL(triggered()), this,
          SLOT(selectVerticalMirror()));
  
  currentBoard = 0;
  
  //---------------------------------------------------
  // Connect Filters to Games Tab filtering
  //---------------------------------------------------
  /*
  connect(filters_->lineEditPlayerWhite, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  connect(filters_->lineEditPlayerBlack, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  connect(filters_->lineEditRankWhite, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  connect(filters_->lineEditRankBlack, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  connect(filters_->lineEditResult, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  connect(filters_->lineEditDateFrom, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilterDate()));
  connect(filters_->lineEditDateTo, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilterDate()));
  connect(filters_->lineEditPlace, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  connect(filters_->lineEditEvent, SIGNAL(textChanged(const QString &)),
          this, SLOT(doFilter()));
  */
  
//   configuration_ = new Configuration(this); // Configuration window
  
  databaseActiveGames_ = new DatabaseActiveGames;
  statusBar()->addPermanentWidget( databaseActiveGames_, 0 );
//   gamesTabProgress_ = new GamesTabProgress(this);
//   statusBar()->addPermanentWidget( gamesTabProgress_, 0 );
//   std::cout << "add statusbar widget progress" << std::endl;
//   std::cout << "add statusbar widget active db games" << std::endl;
  statusBar()->setMaximumHeight( 22 );
  statusBar()->setSizeGripEnabled( false );
  
  databaseViewer_ = new DatabaseViewer(this);
  databaseViewer_->show();
  
  signalMapper_ = new QSignalMapper(this);
  
  setWindowIcon(QIcon(":/resources/icons/misc.png"));
//   std::cout << "db viewer shown" << std::endl;
}



// We need to filter the games in the TotalResults from a search
// We then get a list of filtered Results which we reload into our gamesTab.
void MainWindow::doFilter()
{
  dbManager_->setFilters(filters_->lineEditPlayerBlack->text().toStdString(),
                         filters_->lineEditPlayerWhite->text().toStdString(),
                         filters_->lineEditRankBlack->text().toStdString(),
                         filters_->lineEditRankWhite->text().toStdString(),
                         filters_->lineEditResult->text().toStdString());
//   selectedBoardWindow_->mainBoard->search();
  
  
//   std::cout << "doFilter" << std::endl;
//   databaseViewer_->gamesTab->myProxyModel->setPlayerWhiteFilter(filters_->lineEditPlayerWhite->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setPlayerBlackFilter(filters_->lineEditPlayerBlack->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setRankWhiteFilter(filters_->lineEditRankWhite->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setRankBlackFilter(filters_->lineEditRankBlack->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setResultFilter(filters_->lineEditResult->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setPlaceFilter(filters_->lineEditPlace->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setEventFilter(filters_->lineEditEvent->text()); 
//   databaseViewer_->gamesTab->myProxyModel->setSourceModel(databaseViewer_->gamesTab->model);
//   databaseViewer_->gamesTab->treeView->setModel(databaseViewer_->gamesTab->myProxyModel);
}



void MainWindow::doFilterDate()
{
}



void MainWindow::showGameInfo()
{
  if (gameInfoAct_->isChecked() == true) {
    gameInfo_ = new GameInfo(this);
//     connect(gameInfo_, SIGNAL(close()), this->gameInfoAct_, SLOT(setChecked(false)));
    
    // Get game Info for currently selected game if there is one
    databaseViewer_->gamesTab->setupGameInfo();
    gameInfo_->show();
  }
  else {
    gameInfo_->close();
  }
}



void MainWindow::showDatabaseViewer()
{
  if (databaseViewerAct_->isChecked() == true) {
    databaseViewer_ = new DatabaseViewer(this);
//     connect(databaseViewer_, SIGNAL(close()), this->databaseViewerAct_, SLOT(setChecked(false)));
    databaseViewer_->show();
  }
  else {
    databaseViewer_->close();
  }
}



// Create the new board and since this is an empty board we want to perform 
// a search with it, more user friendly to have some analysis on the board to start with
void MainWindow::addNewBoard()
{
  newBoard();
  setMenuMappings();
  hasBoardWindowSelected_ = true;
  selectedBoardWindow_->mainBoard->repaint();
  selectedBoardWindow_->mainBoard->search();
}



// assert selectedBoardWindow_ == current board
void MainWindow::setMenuMappings()
{
  connect(selectedBoardWindow_->menuAction, SIGNAL(triggered()), signalMapper_, SLOT(map()));
    signalMapper_->setMapping(selectedBoardWindow_->menuAction, 
                              selectedBoardWindow_->menuAction->objectName());
  connect(signalMapper_, SIGNAL(mapped(const QString &)),
          this, SLOT(selectWindow(const QString &)));
}


// Create a blank new board to do searches from
void MainWindow::newBoard()
{
  boost::shared_ptr <BoardWindow> boardWindow (new BoardWindow);
  boost::shared_ptr <MainBoard> mainBoard (new MainBoard(this, editStatus_, 0));
  boardWindow->mainBoard = mainBoard;
  // We need to store a unique id for the board so that when it closes we know which one it is
  QString text;
  if (false == boardWindows_.empty()) {
    boardWindow->identifier = boardWindows_.back()->identifier + 1;
    text = "Board <";
    text += ((lexical_cast<std::string>(boardWindow->identifier + 1)).c_str());
    text += ">";
  }
  else {
    boardWindow->identifier = 0;
    text = "Board";
  }
  boardWindow->mainBoard->setIdentifier(boardWindow->identifier);
  boardWindow->mainBoard->setWindowTitle(text);
  boardWindow->menuAction = new QAction(text, this);
  boardWindow->menuAction->setObjectName(text);
  menuWindow->addAction(boardWindow->menuAction) ;
  
  selectedBoardWindow_ = boardWindow;
  boardWindows_.push_back(boardWindow);
  boardWindow->mainBoard->show();
  toolBarBoard->setToolBarState(true);
  toolBarEdit->setToolBarState(true);
}



// void MainWindow::selectWindow(QAction* menuAction)
void MainWindow::selectWindow(QString text)
{
//   std::cout << "select Window" << std::endl;
//   std::cout << text.toStdString() << std::endl;
  
  std::vector<boost::shared_ptr<BoardWindow> >::const_iterator i;
  for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
    if ( (*i)->menuAction->objectName() == text ) {
//       selectedBoardWindow_ = *i;
      (*i)->mainBoard->raise();
      (*i)->mainBoard->activateWindow();
      hasBoardWindowSelected_ = true;
    }
  }
}



// Find the mainBoard and associated menu Action - then remove it.
void MainWindow::deleteBoardWindow(int identifier)
{
  std::vector<boost::shared_ptr<BoardWindow> >::const_iterator i;
  for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
    if ((*i)->identifier == identifier) {
      menuWindow->removeAction ((*i)->menuAction);
      // No selected boardWindow since current deleted
      hasBoardWindowSelected_ = false;
      // Set the toolbars move navigation to all false
      toolBarMainMoves->firstAct->setEnabled(false);
      toolBarMainMoves->backAct->setEnabled(false);
      toolBarMainMoves->backTenAct->setEnabled(false);
      toolBarMainMoves->lastAct->setEnabled(false);
      toolBarMainMoves->forwardAct->setEnabled(false);
      toolBarMainMoves->forwardTenAct->setEnabled(false);
      toolBarBoard->setToolBarState(false);
      toolBarEdit->setToolBarState(false);
    }
  }
}



void MainWindow::setWindow(int identifier)
{
  std::vector<boost::shared_ptr<BoardWindow> >::const_iterator i;
  for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
    if ((*i)->identifier == identifier) {
      // Only update Games Tab and Stats Tabs if the new Selected Board is changed
      if (selectedBoardWindow_ != *i) {
        selectedBoardWindow_ = *i;
        hasBoardWindowSelected_ = true;
        
        changeStatsTab();
        statisticsBlack_->showStatisticsBlack();
        statisticsWhite_->showStatisticsWhite();
  
        // Clear the Games List and show matches if auto update is set true
        databaseViewer_->gamesTab->setupModel();
        databaseViewer_->gamesTab->doMatches();
      }
    }
  }
}




//! DESIGN NEEDS ATTENTION
//! It would be better design to inherit or compose by class Game as both 
//! MainBoard and Game share similiar functions
// Create a new board from a game selected from the GamesTab
void MainWindow::createNewBoard(std::string database, u32 gameindex)
{
//    std::cout << "create new Board" << std::endl;
  newBoard();
//    std::cout << "created a new Board" << std::endl;
  selectedBoardWindow_->mainBoard->changeModeOneClick(false);
  
  Database* db = dbManager_->getDatabasePointer(database);
  
  
  // Setup Game with moves from Parsed games file.
  // Finds the moves for the game
  // Loads moves into game class
  // Sets up a vector of boards[19][19] class for each move.
  
  db->initializeGame(gameindex);
//   qDebug("here 0");
  
  map<u32,boost::shared_ptr<Game> >::const_iterator Gpos;
  Gpos = db->Games.find(gameindex);
  assert(Gpos != db->Games.end());
  assert(Gpos->second->getMoveCount() > 0);
  
//   qDebug("here 1");
  selectedBoardWindow_->mainBoard->totalMoves_ = Gpos->second->getMoveCount();
  selectedBoardWindow_->mainBoard->copyGameBoards(Gpos->second); 
  // Copies all game boards to MainBoard boards
  // Display the players name in the dock preview Board title header
  QString title = (Gpos->second->getPlayerW() + " vs " + 
      Gpos->second->getPlayerB() + " :: " +
      Gpos->second->getResultStr()).c_str();
  selectedBoardWindow_->mainBoard->setWindowTitle(title);
  selectedBoardWindow_->menuAction->setText(title);
  setMenuMappings();
  
//   qDebug("here 2");
  
  // Goto the move in the game and display the game position
  selectedBoardWindow_->mainBoard->selectFirst();

}



void MainWindow::readSettings()
{
  QSettings settings;
  settings.beginGroup("MainWindow");
  QPoint pos = settings.value("pos", QPoint(0, 50)).toPoint();
  QSize size = settings.value("size", QSize(300, 700)).toSize();
  // Sets up the toolbars and preview board in a good way
  QByteArray dockState = settings.value("dockState", QByteArray()).toByteArray();

  settings.endGroup();
  resize(size);
  move(pos);
  restoreState (dockState, 0);
  int n = settings.value("PreviewBoard/Menu_Selection", 2).toInt();
  switch (n) {
    case 0 : {
      dockPreviewBoard_->previewBoard_->positionSelected_ = CURRENT;
      dockPreviewBoard_->previewBoard_->currentPositionAct->setChecked(true);
      break;
    }
    case 1 : {
      dockPreviewBoard_->previewBoard_->positionSelected_ = NEXT;
      dockPreviewBoard_->previewBoard_->nextPositionAct->setChecked(true);
      break;
    }
    case 2 : {
      dockPreviewBoard_->previewBoard_->positionSelected_ = END;
      dockPreviewBoard_->previewBoard_->endPositionAct->setChecked(true);
      break;
    }
  }
}



void MainWindow::writeSettings()
{
  QSettings settings;
  settings.beginGroup("MainWindow");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.setValue("dockState", saveState(0));
  settings.endGroup();  
  settings.setValue("PreviewBoard/Menu_Selection", 
                    dockPreviewBoard_->previewBoard_->positionSelected_);
}



 void MainWindow::closeEvent(QCloseEvent *event)
{
  writeSettings();
  std::vector<boost::shared_ptr<BoardWindow> >::iterator i;
  for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
    (*i)->mainBoard->close();
  }
  boardWindows_.clear(); // Deletes all mainboard objects shared ptr
  databaseViewer_->close();
  event->accept();
}



void MainWindow::addDatabaseWizard()
{
  addDatabase_ = new AddDatabase(this);
  if (addDatabase_->exec() == true) {
    databaseViewer_->databaseTab->setupModel();
  }
}



//-----------------------------------------------------
// Toolbar Edit connections
void MainWindow::selectErase()
{
//   std::cout << "erase1" << std::endl;
  assert (true == hasBoardWindowSelected_);
  std::vector<boost::shared_ptr<BoardWindow> >::iterator i;
//   for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
//     (*i)->mainBoard->selectErase();
//   }
  selectedBoardWindow_->mainBoard->selectErase();
  editStatus_ = ERASE;
}



void MainWindow::selectAlternate()
{
  assert (true == hasBoardWindowSelected_);
  std::vector<boost::shared_ptr<BoardWindow> >::const_iterator i;
//   for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
//     (*i)->mainBoard->selectAlternate();
//   }
  selectedBoardWindow_->mainBoard->selectAlternate();
  editStatus_ = ADDALTERNATE;
  changeStatsTab();
}



void MainWindow::selectBlack()
{
  assert (true == hasBoardWindowSelected_);
  std::vector<boost::shared_ptr<BoardWindow> >::const_iterator i;
//   for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
//     (*i)->mainBoard->selectBlack();
//   }
  selectedBoardWindow_->mainBoard->selectBlack();
  changeStatsTab();
  editStatus_ = ADDBLACK;
}



void MainWindow::selectWhite()
{
  assert (true == hasBoardWindowSelected_);
  std::vector<boost::shared_ptr<BoardWindow> >::const_iterator i;
//   for (i = boardWindows_.begin(); i != boardWindows_.end(); i++) {
//     (*i)->mainBoard->selectWhite();
//   }
  selectedBoardWindow_->mainBoard->selectWhite();
  changeStatsTab();
  editStatus_ = ADDWHITE;
}



void MainWindow::changeStatsTab()
{
  if (selectedBoardWindow_->mainBoard->nextColour_ == BLACK) {
    tabWidget->setCurrentWidget (statisticsBlack_);
  }
  else {
    tabWidget->setCurrentWidget (statisticsWhite_);
  }
  selectedBoardWindow_->mainBoard->update();
}


//-----------------------------------------------------------
// Toolbar Moves connections Preview Board Navigation
void MainWindow::selectPreviewBoardForward()
{
  dockPreviewBoard_->previewBoard_->selectForward();
}



void MainWindow::selectPreviewBoardBack()
{
  dockPreviewBoard_->previewBoard_->selectBack();
}



void MainWindow::selectPreviewBoardForwardTen()
{
  dockPreviewBoard_->previewBoard_->selectForwardTen();
}



void MainWindow::selectPreviewBoardBackTen()
{
  dockPreviewBoard_->previewBoard_->selectBackTen();
}



void MainWindow::selectPreviewBoardFirst()
{
  dockPreviewBoard_->previewBoard_->selectFirst();
}



void MainWindow::selectPreviewBoardLast()
{
  dockPreviewBoard_->previewBoard_->selectLast();
}



// MainBoard Navigation SLOTS
void MainWindow::selectMainBoardForward()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
//     std::cout << "selected Forward :" << std::endl;
    selectedBoardWindow_->mainBoard->selectForward();
  }
}



void MainWindow::selectMainBoardBack()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
//     std::cout << "selected back :" << std::endl;
    selectedBoardWindow_->mainBoard->selectBack();
  }
}



void MainWindow::selectMainBoardForwardTen()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectForwardTen();
  }
}



void MainWindow::selectMainBoardBackTen()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectBackTen();
  }
}



void MainWindow::selectMainBoardFirst()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectFirst();
  }
}



void MainWindow::selectMainBoardLast()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectLast();
  }
}


//------------------------------------------------
// Main Board Rotate and Board Mirror tool bar actions

void MainWindow::selectRotate()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectRotate();
  }
}


void MainWindow::selectHorizontalMirror()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectHorizontalMirror();
  }
}


void MainWindow::selectVerticalMirror()
{
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->selectVerticalMirror();
  }
}



void MainWindow::selectFont()
{
  bool ok;
  font = QFontDialog::getFont(
      &ok, QFont("Helvetica [Cronyx]", 10), this);
  if (ok) {
        // the user clicked OK and font is set to the font the user selected
    if (true == hasBoardWindowSelected_) {
      assert (false == boardWindows_.empty());
      selectedBoardWindow_->mainBoard->setFont(font);
    }
  } else {
        // the user canceled the dialog; font is set to the initial
        // value, in this case Helvetica [Cronyx], 10
  }
}



// Show the options dialog
void MainWindow::configureBaduk()
{
  configureWindow_ = new Configuration;
  configureWindow_->show();
}



// HTML text for About baduk Dialog box
const char *aboutBadukText =
"<HTML>"
//     "<img src=/home/neil/ProjectBaduk/goban/logo_header.gif alt=baduK logo />"
//     "<img src=/home/neil/ProjectBaduk/goban/logo_header2.gif alt=baduK logo />"
    "<p><b>baduK alpha release v0.1</b></p>"
    
    "<p>Developed using GCC 3.3.5 C++ , Qt 4.1.4 </p>"
    "<p>For more information visit the baduK website: "
    "<a href=http://baduk.sourceforge.net>"
    "http://baduk.sourceforge.net</a></p>"

    "<br/><center><p>Copyright (c) Neil Mclean - 18th July 2006</br>"
    "<br/><center><p>Released under the following licenses :</br>"
    "<br/><center><p>Qt License, Boost Library Licenses</br>"
"</HTML>";



void MainWindow::showAboutBaduk()
{
  QMessageBox::about(this, "About baduK", aboutBadukText);
}



// Once a MainBoard initiates a search
void MainWindow::search(u16 moveNumber, u64 value, u8 wbPatternSelected)
{
//   qDebug("mainWindow search");
  dbManager_->searchPattern(moveNumber, value, wbPatternSelected);
//   dbManager_->sortStatsByBestVariation();
//   qDebug("mainWindow search done");
  dbManager_->sortStatsByTotalGames();

  // Copy list of games to the mainboard, so it stores games tab list
  // Copy stats to the current selected board
//   qDebug("mainWindow Search");
  selectedBoardWindow_->mainBoard->filteredResults = dbManager_->getFilteredResults();
  selectedBoardWindow_->mainBoard->statsBlack = dbManager_->getStatsBlack();
  selectedBoardWindow_->mainBoard->statsWhite = dbManager_->getStatsWhite();

//   qDebug("mainWindow stats sorted");
  selectedBoardWindow_->mainBoard->repaint(); // show new analysis on board
  statisticsBlack_->showStatisticsBlack();
  statisticsWhite_->showStatisticsWhite();
  changeStatsTab(); // change the tab to currently selected nextColour of selectedBoard
  repaint(); // Update the Games matched statusbar and statistics
  update();
  
  assert (true == hasBoardWindowSelected_);
  assert (false == boardWindows_.empty());

  
  databaseViewer_->gamesTab->setupModel();
  databaseViewer_->gamesTab->doMatches(); // display list of matching games
  databaseViewer_->gamesTab->setCurrentMoveNumber(moveNumber);
}



// FIXME none of this function seems to work !
void MainWindow::focusInEvent ( QFocusEvent * event )
{
  update(); // For Windows menu sets this board active - so repaint before game list is done
  //! NOTE cant really raise each window since they will each in turn update toolbar, games 
  //! list etc...
  
  // Only raise current selected window
  if (true == hasBoardWindowSelected_) {
    assert (false == boardWindows_.empty());
    selectedBoardWindow_->mainBoard->raise();
  }
  if (databaseViewerAct_->isChecked() == true) {
    databaseViewer_->raise();
  }
  if (gameInfoAct_->isChecked() == true) {
    gameInfo_->raise();
  }
  event->accept();
}

