#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QDockWidget>
#include <QTabWidget>
#include <QFileInfo>
#include <QContextMenuEvent>
#include <QMenu>
#include <vector>
#include <QTreeView>
#include <QThread>
#include <QTimer>
#include <QStandardItemModel>
#include <QScrollArea>
#include <QAction>
#include <QSignalMapper>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
#include "boost/shared_ptr.hpp"
#endif


#include "PreviewBoard.h"
#include "DatabaseViewer.h"
#include "MainBoard.h"
#include "Filters.h"
#include "DbManager.h"
#include "ToolBarMoves.h"
#include "ToolBarMainMoves.h"
#include "ToolBarEdit.h"
#include "ToolBarBoard.h"
#include "GameInfo.h"
#include "AddDatabase.h"
#include "StatisticsBlackTab.h"
#include "StatisticsWhiteTab.h"
#include "Configuration.h"

class ToolBarMoves;
class ToolBarBoard;
class ToolBarEdit;
class ToolBarMainMoves;
class QMenu;
class MainBoard;
class DockPreviewBoard;
class PreviewBoard;
class GameInfo;
class DatabaseTab;
class QAction;
class QStandardItemModel;
class DbManager;
class AddDatabase;
class DatabaseViewer;
class StatisticsBlackTab;
class StatisticsWhiteTab;
class Configuration;

/*
class MyThread : public QThread
{
  public:
    void run();
    QTimer* timer;
};

void MyThread::run()
{
  timer = new QTimer;
//   QTcpSocket socket;
        // connect QTcpSocket's signals somewhere meaningful
//       socket.connectToHost(hostName, portNumber);
    exec();
}
*/

class GamesTabProgress : public QWidget
{
    Q_OBJECT
  public:
    GamesTabProgress(QWidget *parent = 0);
    QProgressBar* progressBar_;
  private:
};


class DatabaseActiveGames : public QWidget
{
    Q_OBJECT
  public:
    DatabaseActiveGames(QWidget *parent = 0);
    QLabel* label_;
  private:
};


// class MenuAction : public QAction
// {
//     Q_OBJECT
//   public:
//     
//   
// }

// This is a basic structure which holds information on new Board windows
// Helps to keep a track of the pointer to the board, th emenu action created and an ID
class BoardWindow // For a new board vector.
{
  public:
    QAction* menuAction;
    boost::shared_ptr<MainBoard> mainBoard;
    int identifier; // A unique id - set to last mainboard id + 1
};



class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
  MainWindow(DbManager* db, QWidget *parent = 0, Qt::WFlags flags = 0);
  DbManager* dbManager_; // Starting point of backend into database
  AddDatabase* addDatabase_;
  DockPreviewBoard *dockPreviewBoard_;
  
  DatabaseViewer *databaseViewer_;
  GameInfo* gameInfo_;
  QAction* databaseViewerAct_;
  QAction* gameInfoAct_;
  
  QScrollArea* scrollArea_;
  StatisticsBlackTab* statisticsBlack_;
  StatisticsWhiteTab* statisticsWhite_;
  Configuration* configuration_;
  ToolBarEdit* toolBarEdit;
  ToolBarMoves* toolBarMoves;
  ToolBarBoard* toolBarBoard;
  ToolBarMainMoves* toolBarMainMoves;
  QMenu* toolBarsMenu;
  
  void checkDatabaseViewer();
//   void checkPreviewBoard();
//   void checkToolbarEdit();
//   void checkToolbarMoves();
//   void checkToolbarBoard();
  void search(u16 moveNumber, u64 value, u8 wbPatternSelected);
  void createNewBoard(std::string database, u32 gameindex);
  void deleteBoardWindow(int identifier);
  void setWindow(int identifier);
  GamesTabProgress* gamesTabProgress_;
  DatabaseActiveGames* databaseActiveGames_;
  boost::shared_ptr<BoardWindow> selectedBoardWindow_;
  std::vector<boost::shared_ptr<BoardWindow> > boardWindows_;
  void changeStatsTab();
  
public slots:

  void showAboutBaduk();
  void addDatabaseWizard();
  
protected:
  
  void closeEvent(QCloseEvent *event);
  virtual void focusInEvent ( QFocusEvent * event );
  
private:
  QSignalMapper* signalMapper_;
  Configuration* configureWindow_;
  bool hasBoardWindowSelected_;
  QMenu* menuWindow;
  
  QAction* aboutBaduk;
  QAction* addDatabaseAct;
  QAction* newBoardAct;
  QAction* setFontAct;
  QAction* configureBadukAct;
  QAction* toolBarMovesAct;
  QAction* toolBarEditAct;
  QAction* toolBarBoardAct;
  QAction* aboutQtAct;
  QAction* helpAction;
  
  QTabWidget *tabWidget;
  Filters* filters_;
  
  QFont font;
  
  
  void readSettings();
  void writeSettings();
  
  unsigned int currentBoard;
  EditStatus editStatus_;
  void newBoard();
  void setMenuMappings();
  
  signals:
    void clicked(const QString &text);
    
private slots:
//   void selectWindow(QAction* menuAction);
  
  void selectWindow(QString text);
  void selectErase();
  void selectAlternate();
  void selectBlack();
  void selectWhite();
  void addNewBoard();
  

  void selectFont();
  void configureBaduk();
  
  void selectPreviewBoardForward();
  void selectPreviewBoardBack();
  void selectPreviewBoardForwardTen();
  void selectPreviewBoardBackTen();
  void selectPreviewBoardFirst();
  void selectPreviewBoardLast();
  
  void selectMainBoardForward();
  void selectMainBoardBack();
  void selectMainBoardForwardTen();
  void selectMainBoardBackTen();
  void selectMainBoardFirst();
  void selectMainBoardLast();
  
  
  void selectRotate();
  void selectHorizontalMirror();
  void selectVerticalMirror();

  void showDatabaseViewer();
  void showGameInfo();
  
  void doFilter();
  void doFilterDate();
  
};


#endif

