//Pattern.cpp
//baduK++ 0.1 - Neil Mclean 6/4/2006

#include <iostream>
#include <fstream>
#include <vector>

#include "Pattern.h"

using namespace std;

Pattern::Pattern () : zobHash_(0), matches_(0), seekPos_(0) {}

Pattern::~Pattern() {}


//---------------------------------------------------------------------
// Start search for pattern value.
// Find offset into Patterns Table.
// Do binary search to find Pattern.
//---------------------------------------------------------------------
bool Pattern::searchPatterns(ifstream& ifs, const u16 moveNumber, 
                             const u64 value) 
{
  u32 right = entryTable[moveNumber].patternCount;
  u32 startPos = entryTable[moveNumber].seekPos; 
  u32 offset = binarySearch(ifs, startPos, value, 0, right);
  if (offset != PATTERN_NOT_FOUND) {
    ifs.read(reinterpret_cast<char *> (&matches_), sizeof(matches_)); 
    ifs.read(reinterpret_cast<char *> (&seekPos_), sizeof(seekPos_)); 
//     cout << "matches_=" << matches_ << endl;
    return true;
  }
//   cout << "pattern not found in posuk database" << endl;
  return false; 
}



//------------------------------------------------------------------------
//Binary search Pattern Headers for the Zobrist hash pattern value
//Once found , we have the seekpos offset into the list of games matching
//-----------------------------------------------------------------------
u32 Pattern::binarySearch(ifstream& ifs, const u32 startPos,
                          const u64 value, u32 first, u32 last) 
{
  u64 zobhash = 0;
  //cout << "startpos=" << startPos << endl;
  while (first <= last) {
    int mid = (first + last) / 2;  // compute mid point.
    u32 offset = startPos + (mid * PATTERNS_SIZE);
    //cout << "offset=" << offset << endl;
    ifs.seekg (offset); //goto pattern header position in file
    ifs.read (reinterpret_cast<char *> (&zobhash), sizeof (zobhash)); //get value
    //cout << "value=" << value << " " << "left=" << first;
    //cout << " right=" << last << endl;
    //cout << "mid=" << mid << endl;
    //cout << "zobhash=" << zobhash << " " << "value=" << value << endl;
    //string dud;
    //cin >> dud;
    if (value > zobhash) { 
      first = mid + 1;
    }
    if (value < zobhash) {
      if (false == mid) { // using unsigned int so check
        return PATTERN_NOT_FOUND; 
      }
      last = mid - 1;
    }
    if (value == zobhash) {
      return offset;
    }
  }
  return PATTERN_NOT_FOUND;
}



//-----------------------------------------------------------
//read in list of games and store in a vector
//we use 'SeekPos' and 'matches_' found from SearchPatterns
//-----------------------------------------------------------
vector<boost::shared_ptr<GamesList> > Pattern::getGamesList(const u16 dbIndex,  
      ifstream& ifs ) 
{
  if (searchResults.empty() == false) { 
    searchResults.clear(); 
  }
  ifs.seekg(seekPos_); //goto the start of the list of matches_
  try {
    searchResults.reserve(matches_); //allocate memory for the new results
    for (u32 i = 0; i < matches_; i++) {
      boost::shared_ptr<GamesList> gameslist (new GamesList);
      ifs.read(reinterpret_cast<char *>(&gameslist->gameIndex),
               sizeof (gameslist->gameIndex));
      ifs.read(reinterpret_cast<char *>(&gameslist->nextMoveX),
               sizeof (gameslist->nextMoveX));
      ifs.read(reinterpret_cast<char *>(&gameslist->nextMoveY),
               sizeof (gameslist->nextMoveY));
      ifs.read(reinterpret_cast<char *>(&gameslist->moveNumber),
               sizeof (gameslist->moveNumber));
      
      //! --------------------------------------------------------
      //!Decode bits compacted into moveNumber - explained in Pattern.h
      //! --------------------------------------------------------
      u16 result = gameslist->moveNumber; //get result of game
      result >>= 12;
      gameslist->result = static_cast<u8> (result);
      u16 nextMoveColour = gameslist->moveNumber;
      nextMoveColour >>= 10; 
      nextMoveColour &= 0x03; // Keep the 2 lowest bits
      gameslist->nextMoveColour = static_cast<u8> (nextMoveColour);
      gameslist->moveNumber &= 0x3F; //Keep the 10 low bits
      gameslist->dbIndex = dbIndex;
      
      searchResults.push_back(gameslist);
      
//       cout << "GI=" << (int)gameslist->gameIndex << " ";
//       cout << "x&y=" << (int)gameslist->nextMoveX << " ";
//       cout << (int)gameslist->nextMoveY << " ";
//       cout << "result=" << (int)gameslist->result << " ";
//       cout << "colour=" << (int)gameslist->nextMoveColour << endl;
    }
  }
  catch (bad_alloc&) { 
    cout << "Error allocating memory." << endl; 
    abort();
  }
  catch (ios_base::failure& e) {
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
  return searchResults;
}




//-----------------------------------------------------------------
//! read in Entry table with offsets into list of Patterns file
//---------------------------------------------------------------
void Pattern::readEntry(istream& ifs) 
{
  try {
    //cout << "Reading entryTable" << endl;
    for (u8 i = 0; i < WHOLEBOARD_DEPTH; i++) { //defined in Pattern.h
      ifs.read(reinterpret_cast<char *> (&entryTable[i]), 
               sizeof (entryTable[i]));
    }
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
}



//-------------------------------------------------------------
//write out structure to provide entry into Pattern List file
//-------------------------------------------------------------
void Pattern::writeEntry(ostream& ofs) 
{
  try {
    u32 seekPos = 0;
    for (u64 i = 0; i < WHOLEBOARD_DEPTH; i++) {
      u32 numPatterns = patternMoveNumber[i].size();
      ofs.write(reinterpret_cast<char *> (&numPatterns), sizeof (numPatterns));
      ofs.write(reinterpret_cast<char *> (&seekPos), sizeof (seekPos));
      seekPos += PATTERNS_SIZE * patternMoveNumber[i].size(); //16 bytes
      //cout << "move number=" << i << " ";
      //cout << "patterncount=" << numPatterns << " ";
      //cout << "seekpos=" << seekPos_ << endl;
    }
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
}


//-------------------------------------------------------------
//! 16 byte structure - giving zobrist Hash pattern,
//! number of Game indexes matching this pattern,
//! Offset into posuklist to read game indexes.
//-------------------------------------------------------------
void Pattern::writePatterns(ostream& ofs) 
{
  try {
    u32 offSet = 0;
    map<u64,boost::shared_ptr<Patterns> >::iterator j;
    for (u16 i = 0; i < WHOLEBOARD_DEPTH; i++) {
      j = patternMoveNumber[i].begin(); //if pattern exists already
      //! calculate end position for the start of next Pattern
      for (; j != patternMoveNumber[i].end(); j++) {
        j->second->seekPos = offSet; //set to start of position
        offSet += j->second->matches * GAMESLIST_SIZE;
        ofs.write (reinterpret_cast<char *> (&j->second->zobHash),
                   sizeof (j->second->zobHash));
        ofs.write (reinterpret_cast<char *> (&j->second->matches),
                   sizeof (j->second->matches));
        ofs.write (reinterpret_cast<char *> (&j->second->seekPos),
                   sizeof (j->second->seekPos));
      }
    }
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
}



//------------------------------------------------------------
//! write games list - encode bits into moveNumber fieled
//------------------------------------------------------------
void Pattern::writeGamesList(ostream& ofs) 
{
  try {
    map<u64,boost::shared_ptr<Patterns> >::const_iterator j;
    for (u16 i = 0; i < WHOLEBOARD_DEPTH; i++) {
      j = patternMoveNumber[i].begin(); //if pattern exists already
      for (; j != patternMoveNumber[i].end(); j++) {
        for (u32 k = 0; k < j->second->patternList.size(); k++) {
          ofs.write(reinterpret_cast<char *>
                   (&j->second->patternList[k]->gameIndex), 
                    sizeof (j->second->patternList[k]->gameIndex));
          ofs.write(reinterpret_cast<char *>
                   (&j->second->patternList[k]->nextMoveX), 
                    sizeof (j->second->patternList[k]->nextMoveX));
          ofs.write(reinterpret_cast<char *>
                   (&j->second->patternList[k]->nextMoveY), 
                    sizeof (j->second->patternList[k]->nextMoveY));
          //! Encoded bits - See Pattern.h for details
          u16 encoded = j->second->patternList[k]->result;
          encoded <<= 2;
          u16 nmc = j->second->patternList[k]->nextMoveColour;
          encoded |= nmc;
          encoded <<= 10;
          encoded |= j->second->patternList[k]->moveNumber;
          ofs.write(reinterpret_cast<char *> (&encoded), sizeof (encoded));
        }
      }
    }
  }
  catch (ios_base::failure& e) { 
    cout << "Error I/O failure " << e.what() << endl;
    abort();
  }
}



//! Clean up resources
//! To free memory used by pattern generation
void Pattern::cleanupPatternGenerationResources()
{
  //map<u64, boost::shared_ptr<Patterns> > patternMoveNumber[WHOLEBOARD_DEPTH];
  for (int i = 0; i < WHOLEBOARD_DEPTH; i++) {
    patternMoveNumber[i].clear();
  }
}




//////////////////////////////////////////////////////////////////////////
//! DEBUG CODE
//////////////////////////////////////////////////////////////////////////

void Pattern::printPatterns(ifstream& ifs, const u16 moveNumber) 
{
  u32 startPos=entryTable[moveNumber].seekPos; //find offset into pattern header
  ifs.seekg(startPos);
  cout << "WB Patterns for moveNumber " << moveNumber << endl;
  for (u32 i=1; i<=entryTable[moveNumber].patternCount; i++) {
    ifs.read(reinterpret_cast<char *> (&zobHash_), sizeof (zobHash_));
    ifs.read(reinterpret_cast<char *> (&matches_), sizeof (matches_)); 
    ifs.read(reinterpret_cast<char *> (&seekPos_), sizeof (seekPos_)); 
    cout << zobHash_ << "  :  " << matches_ << endl;
  }
}


void Pattern::printPatterns(ifstream& ifs) 
{
  for ( u16 j = 0; j < WHOLEBOARD_DEPTH; j++ ) {
    u32 startPos=entryTable[j].seekPos; //find offset into pattern header
    ifs.seekg(startPos);
//     cout << "WB Patterns for moveNumber " << j << endl;
    for ( u32 i = 1; i <= entryTable[j].patternCount; i++ ) {
      ifs.read(reinterpret_cast<char *> (&zobHash_), sizeof (zobHash_));
      ifs.read(reinterpret_cast<char *> (&matches_), sizeof (matches_)); 
      ifs.read(reinterpret_cast<char *> (&seekPos_), sizeof (seekPos_)); 
//       cout << zobHash_ << "  :  " << matches_ << endl;
    }
  }
}

