#ifndef PATTERN_H
#define PATTERN_H

#include <vector>
#include <map>

#include "Const.h"

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
#include "boost/shared_ptr.hpp"
#endif


//provides an entry into Patterns table to do binary search of zobristHash
struct Entry {
    u32 patternCount; //How many patterns 
    u32 seekPos; //offset into Pattern Header file
};



//-----------------------------------------------------------------------------
//A pattern match stores information used to generate statistics
//! When GamesList is serialized, we compact u8 result and u8 nextMoveColour
//! into u16 moveNumber to save space and adhere to 8 byte boundaries.
//! USE  10 bits for (m)oveNumber
//! USE  2 bits for (n)extMoveColour
//! USE  4 bits for (r)esult
//! 16 bits arranged in this order, high to low, rrrr-nn-mmmmmmmmmm
//-----------------------------------------------------------------------------
struct GamesList {
    u16 dbIndex;
    u32 gameIndex;
    u8 nextMoveX;
    u8 nextMoveY;
    u16 moveNumber; 
    u8 nextMoveColour; //0=EMPTY , 1=BLACK, 2=WHITE
    u8 result; //game result used for pattern statistics
    u8 wbPatternSelected;
};



//for each unique Pattern value (zobrist Hash) count the number of matches,
//and offset into GamesList file to access GamesList structure
struct Patterns {
    u64 zobHash; // Pattern Value
    u32 matches; //
    u32 seekPos; 
    vector<boost::shared_ptr<GamesList> > patternList; 
};




class Pattern {
  public:
    Pattern();
    bool searchPatterns(ifstream& ifs, const u16 moveNumber, const u64 value);
    vector<boost::shared_ptr<GamesList> > getGamesList(const u16 dbIndex, 
        ifstream& ifs);
    void printPatterns(ifstream& ifs, const u16 moveNumber);
    void printPatterns(ifstream& ifs);
    //void doStats();
    //bool printStats();
    void readEntry(istream& ifs);
    void writeEntry(ostream& ofs);
    void writePatterns(ostream& ofs);
    void writeGamesList(ostream& ofs);
    
    // Destroy the smart pointers created via the Generation process
    // To free memory used.
    void cleanupPatternGenerationResources();
    ~Pattern();
  protected:
    // An array of maps constaining - Zobrist Hash, pointer to Pattern struct
    // An array of Entry structs
    // File Offsets into Patterns file structure
    map<u64, boost::shared_ptr<Patterns> > patternMoveNumber[WHOLEBOARD_DEPTH];
    Entry entryTable[WHOLEBOARD_DEPTH];
  private:
    u32 binarySearch(ifstream& ifs, const u32 startPos, const u64 value, 
                     u32 first, u32 last);
    vector<boost::shared_ptr<GamesList> > searchResults;
    u64 zobHash_;
    u32 matches_;
    u32 seekPos_;
};

#endif
