
#include <time.h>
#include <iostream>
#include <string>
#include <cassert>

#include <QMessageBox>
#include <QtGui>
#include <QPainter>
#include <QMouseEvent>
#include <QCursor>
#include <QString>
#include <QFrame>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\format.hpp>
#else
#include "boost/format.hpp"
#endif




#include "PreviewBoard.h"
#include "Resources.h"

DockPreviewBoard::DockPreviewBoard(MainWindow* mainWindow, QWidget *parent) :
    QDockWidget(parent), mainWindow_(mainWindow)
{
  previewBoard_ = new PreviewBoard(this);
  setWidget(previewBoard_);
  setWindowTitle("Board Preview");
  setObjectName("DockPreviewBoard");
}



DockPreviewBoard::~DockPreviewBoard() {}



PreviewBoard::PreviewBoard(DockPreviewBoard* dockPreviewBoard, QWidget *parent) : QWidget(parent),
  dockPreviewBoard_(dockPreviewBoard)
{
  srand ( time(NULL) );
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
      whiteStoneType_[x][y] = rand()%35; // between 0 and 35
      board_[x][y] = 0;
    }
  }
  
//   boards.clear();
//   resize(400, 400);
//   setMinimumSize ( 200, 200 ); //int minw, int minh
//   setMaximumSize ( 600, 600 );
//   setSizeIncrement ( 0, 0 );
  setMouseTracking (true);
      
  QString boardDir = Resources::path();
  origBoardImage = QPixmap(boardDir + "Beige1.png");
  QPalette palette;
  palette.setBrush(backgroundRole(), QBrush(origBoardImage));
  setPalette(palette);
  setAutoFillBackground ( true );
  //connect(this, SIGNAL(click()), this, SLOT(makeFocus()));
  
  previewBoardMenu = new QMenu("Preview board menu", this);
//   boardMenu->setTitle ("Preview board menu");
  
  
  positionActions_ = new QActionGroup(this);
  positionActions_->setExclusive(true);
  
  currentPositionAct = new QAction(tr("&Current board position"), this);
  currentPositionAct->setCheckable(true);
  
  nextPositionAct = new QAction(tr("&Board next move"), this);
  nextPositionAct->setCheckable(true);
  
  endPositionAct = new QAction(tr("&End position"), this);
  endPositionAct->setCheckable(true);
  
  positionActions_->addAction (currentPositionAct);
  positionActions_->addAction (nextPositionAct);
  positionActions_->addAction (endPositionAct);
  
  previewBoardMenu->addActions(positionActions_->actions());
  
  connect(currentPositionAct, SIGNAL(triggered()), this, SLOT(currentPosition()));
  connect(nextPositionAct, SIGNAL(triggered()), this, SLOT(nextPosition()));
  connect(endPositionAct, SIGNAL(triggered()), this, SLOT(endPosition()));
}



PreviewBoard::~PreviewBoard() {}



// For popup Menu
void PreviewBoard::contextMenuEvent(QContextMenuEvent *event)
{
  event->accept();
  // If there is a game loaded on the preview board then show popup menu
  if (0 != game_) { // Check if pointer to game is not zero
    previewBoardMenu->exec(event->globalPos());
  }
}



void PreviewBoard::setupBoard(std::string database, u32 gameindex, int moveNumber)
{
  //   std::cout << "setup Board database = " << database << " gameIdx=" << gameindex << std::endl;
  Database* db = dockPreviewBoard_->mainWindow_->dbManager_->getDatabasePointer(database);
  //   Setup Game with moves from Parsed games file.
  //Finds the moves for the game
  //Loads moves into game class
  //Sets up a vector of boards[19][19] class for each move.
  db->initializeGame(gameindex);
  map<u32,boost::shared_ptr<Game> >::const_iterator Gpos;
  Gpos = db->Games.find(gameindex);
  game_ = Gpos->second; // Store pointer to game to be used to navigate game

  assert(Gpos != db->Games.end());
  totalMoves_ = game_->getMoveCount();

  assert(totalMoves_ > 0);
//   Display the players name in the dock preview Board title header
  QString title = (Gpos->second->getPlayerW() + " vs " + 
      Gpos->second->getPlayerB() + " :: " + 
      Gpos->second->getResultStr()).c_str();  
  dockPreviewBoard_->setWindowTitle(title);
  
  originalMoveNumber_ = moveNumber;
  switch (positionSelected_) {
    case CURRENT : {
      selectMoveNumber(moveNumber);
      break;
    }
    case NEXT : {
      selectMoveNumber(moveNumber + 1);
      break;
    }
    case END : {
      selectMoveNumber(totalMoves_);
      break;
    }
  }
}



void PreviewBoard::selectMoveNumber(int moveNumber)
{
  moveNumber_ = moveNumber;
  updateNavigationToolbar();
  getBoard();
}



void PreviewBoard::currentPosition()
{
  positionSelected_ = CURRENT;
  selectMoveNumber(originalMoveNumber_);
}



void PreviewBoard::nextPosition()
{
  positionSelected_ = NEXT;
  selectMoveNumber(originalMoveNumber_ + 1);
}



void PreviewBoard::endPosition()
{
  positionSelected_ = END;
  selectMoveNumber(totalMoves_);
}



void PreviewBoard::paintEvent(QPaintEvent *)
{
  
  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing, true);
  QPen pen(Qt::black, 1);
  painter.setPen(pen);
  int height = size().height();
  int width = size().width();
  //std::cout << height << "," << width << endl;
  int xpadding = 0 ;
  int ypadding = 0 ;
  if (height > width) {
    
    xpadding = 0;
    ypadding = (height - width) / 2;
    height = width;
  }
  else if (height < width) {
    
    xpadding = (width - height) /2;
    ypadding = 0;
    width = height;
  }
  //std::cout << height << "," << width << " ";
  
  double spacing = width;
  spacing = spacing / 20;
  double current = spacing;
  int startpos = static_cast<int>(spacing);
  int endpos = static_cast<int>(width - spacing);
  
  //std::cout.precision(3);
  //std::cout << "spacing=" << spacing << std::endl;
  //std::cout << "current=" << current << std::endl;
  
  
  // Draw Lines on Board
  for (int i = 0; i<19 ; i++) {
    int pos = static_cast<int>(current);
    painter.drawLine(pos + xpadding, startpos + ypadding, pos + xpadding, endpos + ypadding);
    painter.drawLine(startpos + xpadding, pos + ypadding, endpos + xpadding, pos + ypadding);
    current += spacing;
  }
    
  painter.end();
  
  //DRAW CIRCLES ON STAR POINTS
  xstart_ = xpadding + startpos;
  ystart_ = ypadding + startpos;
  spacing_ = spacing;
  
  // Draw star points
  drawCircle(3, 3);
  drawCircle(3, 9);
  drawCircle(3, 15);
  drawCircle(9, 3);
  drawCircle(9, 9);
  drawCircle(9, 15);
  drawCircle(15, 3);
  drawCircle(15, 9);
  drawCircle(15, 15);
  //Draw stones on board and analysis letters
  drawStones();
}

 
  
void PreviewBoard::updateNavigationToolbar()
{
  dockPreviewBoard_->mainWindow_->toolBarMoves->setToolBarState(true);
  if (moveNumber_ >= totalMoves_) {
    moveNumber_ = totalMoves_;
    dockPreviewBoard_->mainWindow_->toolBarMoves->lastAct->setEnabled(false);
    dockPreviewBoard_->mainWindow_->toolBarMoves->forwardAct->setEnabled(false);
    dockPreviewBoard_->mainWindow_->toolBarMoves->forwardTenAct->setEnabled(false);
  }
  if (moveNumber_ <= 0 ) {
    moveNumber_ = 0;
    dockPreviewBoard_->mainWindow_->toolBarMoves->firstAct->setEnabled(false);
    dockPreviewBoard_->mainWindow_->toolBarMoves->backAct->setEnabled(false);
    dockPreviewBoard_->mainWindow_->toolBarMoves->backTenAct->setEnabled(false);
  }
}
 
  
  
// Toolbar Moves 
void PreviewBoard::selectForward()
{
  ++moveNumber_;
  updateNavigationToolbar();
  getBoard();
}


void PreviewBoard::selectBack()
{
  --moveNumber_;
  updateNavigationToolbar();
  getBoard();
}

void PreviewBoard::selectForwardTen()
{
  moveNumber_ += 10;
  updateNavigationToolbar();
  getBoard();
}



void PreviewBoard::selectBackTen()
{
  moveNumber_ -= 10;
  updateNavigationToolbar();
  getBoard();
}



void PreviewBoard::selectFirst()
{
  moveNumber_ = 0;
  updateNavigationToolbar();
  getBoard();
}



void PreviewBoard::selectLast()
{
  moveNumber_ = totalMoves_;
  updateNavigationToolbar();
  getBoard();
}





void PreviewBoard::getBoard()
{
  srand ( time(NULL) ); 
  for (int x = 0; x < 19; x++) {
    for (int y = 0; y < 19; y++) {
      board_[x][y] = game_->boards_[moveNumber_]->getStone(x, y);
    }
  }
  update();
  // Focus this widget so to accept keyboard events and allow navigation via cursor keys
  makeFocus(); 
}



//Draw at board coordinate
void PreviewBoard::drawCircle(int x, int y)
{
  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing, true);
  int diameter = 5;
  int offset = 2;
  if (spacing_ < 15) {
    diameter = 3;
    offset = 1;
  }
  if (spacing_ < 8) {
    diameter = 1;
    offset = 1;
  }
  
  int xpos = static_cast<int>(xstart_ + (spacing_ * x) - offset);
  int ypos = static_cast<int>(ystart_ + (spacing_ * y) - offset);
  QRectF rectangle(xpos, ypos, diameter, diameter);
  QBrush brush(Qt::black, Qt::SolidPattern);
  painter.setBrush(brush);
  painter.drawEllipse(rectangle);

  painter.end();
}




void PreviewBoard::drawStones()
{
  QPainter painter(this);
  painter.setRenderHint(QPainter::Antialiasing, true);
  std::string blackDirPath = Resources::path("BlackStones/").toStdString();
  std::string whiteDirPath = Resources::path("Tsuke.Stones/").toStdString();
  int size = static_cast<int>(spacing_ + 2);
//   int diameter = static_cast<int>(spacing_ - 1);
  if ((size >= 1) && (size <=48)) {
    
    std::string blackPath = blackDirPath + str(boost::format("%d") % size) + ".png";
    //std::cout << "bpath=" << blackPath << std::endl;
    QPixmap blackStone = QPixmap(blackPath.c_str());
   
//     int offset = static_cast<int>((spacing_ / 2) - 1);
    int offsetpixmap = static_cast<int>((spacing_ / 2) +1);
    for (int x = 0; x < 19; x++) {
      for (int y = 0; y < 19; y++) {
        if (0 != board_[x][y]) {
          
          int xpos = static_cast<int>(xstart_ + (spacing_ * x) - offsetpixmap);
          int ypos = static_cast<int>(ystart_ + (spacing_ * y) - offsetpixmap);
          if (1 == board_[x][y]) { //colour = black
            //brush.setColor(Qt::black);
            painter.drawPixmap (xpos, ypos, blackStone );
          }
          else if (2 == board_[x][y]) { // colour = white
            //brush.setColor(Qt::white);
            std::string whitePath;
            if ((size >= 20) && (size <=48)) {
              
              whitePath = whiteDirPath + 
                  str(boost::format("%d") % size) + "-" + 
                  str(boost::format("%d") % whiteStoneType_[x][y]) + ".png";
//               std::cout << "white stone size " << whitePath << std::endl;
            }
            if (size < 20) {
              whitePath = whiteDirPath + 
                  str(boost::format("%d") % size) + "-" + "0.png";
            } 
            QPixmap whiteStone = QPixmap(whitePath.c_str());
            //std::cout << "wpath=" << whitePath << std::endl;
            painter.drawPixmap (xpos, ypos, whiteStone );
          }
        }
      }
    }
  }
  painter.end();
}



    

void PreviewBoard::keyPressEvent ( QKeyEvent * event )
{
//   std::cout << event->key() << std::endl;
  if (event->key() == Qt::Key_Left) {
    selectBack();
  }
  if (event->key() == Qt::Key_Right) {
    selectForward();
  }
  if (event->key() == Qt::Key_Up) {
    selectBackTen();
  }
  if (event->key() == Qt::Key_Down) {
    selectForwardTen();
  }
  if (event->key() == Qt::Key_Home) {
    selectFirst();
  }
  if (event->key() == Qt::Key_End) {
    selectLast();
  }
}





void PreviewBoard::mousePressEvent ( QMouseEvent * event )
{
  setFocus(Qt::MouseFocusReason);
  event->accept();
}



void PreviewBoard::makeFocus() 
{
  setFocus(Qt::MouseFocusReason);
}

