#ifndef PREVIEWBOARD_H
#define PREVIEWBOARD_H


#include <QDockWidget>
#include <QBitmap>

#include <string>
#include <vector>
#include <list>

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
#include "boost/shared_ptr.hpp"
#endif



#include "MainBoard.h"
#include "MainWindow.h"
#include "Game.h"


enum PositionSelected {
  CURRENT = 0, NEXT = 1, END = 2 
};


class DockPreviewBoard;
class MainWindow;

class PreviewBoard : public QWidget
{
  Q_OBJECT

  public:
    PreviewBoard(DockPreviewBoard* dockPreviewBoard, QWidget *parent = 0);
    ~PreviewBoard();
    DockPreviewBoard* dockPreviewBoard_;
    void setupBoard(std::string database, u32 gameindex, int moveNumber);
    void selectBack();
    void selectForward();
    void selectBackTen();
    void selectForwardTen();
    void selectLast();
    void selectFirst();
    void selectMoveNumber(int moveNumber);
    PositionSelected positionSelected_;
    QAction* currentPositionAct;
    QAction* nextPositionAct;
    QAction* endPositionAct;
  
  public slots:

  protected:
    void paintEvent(QPaintEvent *event);
    void keyPressEvent ( QKeyEvent * event );
    void mousePressEvent ( QMouseEvent * event );

    virtual void contextMenuEvent(QContextMenuEvent *event);
    
  private:

   
    QMenu* previewBoardMenu;
   
    QActionGroup* positionActions_;
    void updateNavigationToolbar();
    void getBoard();
    
    boost::shared_ptr<Game> game_;
    QPixmap origBoardImage;
    void drawCircle(int x, int y);
    void drawStones();
    unsigned char board_[19][19];
    int whiteStoneType_[19][19];
    double spacing_;
    double xstart_;
    double ystart_;
    

    
    // originalMoveNumber stores the move number passed when the preview board
    // is originally setup
    int originalMoveNumber_;
    int moveNumber_; // Of current game Board
    int totalMoves_;
  private slots:
    void makeFocus();
    void currentPosition();
    void nextPosition();
    void endPosition();
};


class DockPreviewBoard : public QDockWidget
{
  Q_OBJECT
  public:
    DockPreviewBoard(MainWindow* mainWindow, QWidget *parent = 0);
    ~DockPreviewBoard();
    PreviewBoard *previewBoard_;
    MainWindow* mainWindow_;

  public slots:
    
  protected:

  private:

};



#endif 
