#ifndef SGF_H
#define SGF_H

//Strict SGF 4 Parsing -Will tolerate some extra spaces, line breaks and chars, 
//we will not attempt to fix sgf if deviates from SGF 4 standard

#include <vector>
#include <string>

#include "Const.h"

/*
enum sgfprops {
  // Move properties  
  SP_B, SP_KO,  SP_MN,  SP_W,
  // Setup properties  
  SP_AB,  SP_AE,  SP_AW,  SP_PL,
  // Go-specific properties  
  SP_HA, SP_KM, SP_TB, SP_TW,
  // Node annotations  
  SP_C, SP_DM,  SP_GB,  SP_GW,  SP_HO,  SP_N, SP_UC,  SP_V,
  // Move annotations  
  SP_BM,  SP_DO,  SP_IT,  SP_TE,
  // Markup properties  
  SP_AR,  SP_CR,  SP_DD,  SP_LB,  SP_LN,  SP_MA,  SP_SL,  SP_SQ,  SP_TR,
  // Root properties  
  SP_AP,  SP_CA,  SP_FF,  SP_GM,  SP_ST,  SP_SZ,
  // Game info properties  
  SP_AN, SP_BR, SP_BT, SP_CP, SP_DT, SP_EV, SP_GN, SP_GC, SP_ON, SP_OT,
  SP_PB, SP_PC, SP_PW, SP_RE, SP_RO, SP_RU, SP_SO, SP_TM, SP_US, SP_WR, SP_WT,
  // Timing properties  
  SP_BL, SP_OB, SP_OW, SP_WL,
  // Misc properties  
  SP_FG, SP_PM, SP_VW,
  // Frequent pre FF[4] properties  
  SP_M, SP_L,
};
*/

//! SGF constants - 
//! must be a better way to define these (not using #define anyway)
const string SgfAudio("AU");
const string SgfCA("CA"); //?? UTF-8 Character Encoding
const string SgfCR("CR"); //??
const string SgfAE("AE"); //??
const string SgfLB("LB"); //??

//! results field constants
const string SgfBlackWIN("b+");
const string SgfWhiteWIN("w+");
const string SgfJIGO("jigo");
const char SgfWinTIME = 't'; //includes 'Time' or 'time'

//! moves section constants
const string SgfBLACKMOVE(";B[");
const string SgfWHITEMOVE(";W[");
const string SgfADDBLACKMOVE("AB[");
const string SgfADDWHITEMOVE("AW[");


struct FullMove {
    u8 x;
    u8 y;
    u8 colour;
};


class Sgf {
  public:
    Sgf();
    FullMove getMove(const u16 number) const;
    void read(const string& filename); //reads entire sgf file
    u16 extractMoves(); //returns moveCount & fills vector<FullMove> moves
    bool movesInvariant();
    u64 fnvSignature();
    void xAxisMirror();
    void yAxisMirror();
    void diagonalMirror();
    void rotate();
    
    string getPlayerW() const;
    string getPlayerB() const;
    string getRankW() const;
    string getRankB() const;
    string getResultStr() const;
    string getKomi() const;
    string getDate() const;
    string getGame() const;
    string getFF() const;
    string getST() const;
    string getApplication() const;
    string getSize() const;
    string getHandicap() const;
    string getTime() const;
    string getOverTime() const;
    string getPlace() const;
    string getComment() const;
    string getEvent() const;
    string getRound() const;
    string getRules() const;
    string getGameName() const;
    string getCopyright() const; 
    string getTranscriber() const; 
    string getAnnotator() const ;
    string getSource() const ;
    string getBlackTeam() const; 
    string getWhiteTeam() const ;
    string getGameComment() const; 
    
    u8 getResult();

    void printHead();
    void printMoves();
    ~Sgf();
  private:
    void initTransArrays(); //Creates Transform arrays
    string getString(string value) const;
    string getString();
    void StringToLower(string& strToConvert);
    void addHandicapStones(string::size_type& pos, StoneColour colour);
    u64 FNV1aHash(const string& sig);
    vector <FullMove> moves;
    FullMove transHorizontal_[19];
    FullMove transVertical_[19];
    FullMove transMirror_[19][19];
    FullMove transRotate_[19][19];
    //store sgf file temporarily here to extract info & moves
    string sgfFile_;
    void doSymmetries(int iMove);
};

#endif
