#ifndef WHOLEBOARD_H
#define WHOLEBOARD_H

#include <vector>
#include <map>

#include "Const.h"
#include "Pattern.h"
#include "Game.h"

#ifdef WIN32
#include <C:\Boost\include\boost-1_33_1\boost\shared_ptr.hpp>
#else
#include "boost/shared_ptr.hpp"
#endif




class WholeBoard : public Pattern {
  public:
    WholeBoard();
    ~WholeBoard();
    void generatePattern(boost::shared_ptr<Game> game);
    u16 getMainBoardMoves(unsigned char board[19][19]);
//     u64 getMainBoardPattern(unsigned char board[19][19]);
    u64 getMainBoardPattern(unsigned char board[19][19], u8& WbPatternSelected);
  private:
    u64 patternValue(const boost::shared_ptr<Game>& game , u16 iMove);
    FullMove invariantMove(FullMove move);
    void initializeMtrand64();
    void initializeMtrandVariations();
    void diagonalMirrorBlackMtrand(const u8 in, const u8 out);
    void xAxisMirrorBlackMtrand(const u8 in, const u8 out);
    void diagonalMirrorWhiteMtrand(const u8 in, const u8 out);
    void xAxisMirrorWhiteMtrand(const u8 in, const u8 out);
    
    //For loading MTran64 on first object creation
    //8 variantions, since there are 8 wholeboard orientations
    //typically 8 invariants, board size=19x19;
    static bool isMtrand64FirstInstance_; 
    static u64 mtrand64Black_[WB_INVARIANTS][BOARD_SIZE][BOARD_SIZE]; 
    static u64 mtrand64White_[WB_INVARIANTS][BOARD_SIZE][BOARD_SIZE];
    u64 wbPatterns_[WB_INVARIANTS]; // WB_INVARIANTS=8;
    u8 wbPatternSelected_; // which invariant pattern chosen 1-8
};

#endif
